@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014 Canonical Ltd.
+// Copyright (C) 2014-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,6 +18,8 @@
 #include "oxideqcertificateerror.h"
 #include "oxideqcertificateerror_p.h"
 
+#include <utility>
+
 #include <QString>
 #include <QtDebug>
 #include <QUrl>
@@ -27,17 +29,16 @@
 #include "base/macros.h"
 #include "net/cert/x509_certificate.h"
 
-#include "shared/browser/oxide_certificate_error.h"
 #include "shared/browser/oxide_security_types.h"
+#include "shared/browser/ssl/oxide_certificate_error.h"
 
-#include "oxideqsslcertificate.h"
 #include "oxideqsslcertificate_p.h"
 
 OxideQCertificateErrorPrivate::OxideQCertificateErrorPrivate(
     scoped_ptr<oxide::CertificateError> error)
     : q_ptr(nullptr),
-      certificate_(OxideQSslCertificatePrivate::Create(error->cert())),
-      error_(error.Pass()),
+      certificate_(OxideQSslCertificateData::Create(error->cert())),
+      error_(std::move(error)),
       did_respond_(false) {}
 
 void OxideQCertificateErrorPrivate::OnCancel() {
@@ -59,7 +60,7 @@ void OxideQCertificateErrorPrivate::respond(bool accept) {
     return;
   }
 
-  if (error_->is_cancelled()) {
+  if (error_->IsCancelled()) {
     qWarning() << "Cannot respond to a CertificateError that has been cancelled";
     return;
   }
@@ -79,7 +80,7 @@ OxideQCertificateError* OxideQCertificateErrorPrivate::Create(
     scoped_ptr<oxide::CertificateError> error,
     QObject* parent) {
   return new OxideQCertificateError(
-      *new OxideQCertificateErrorPrivate(error.Pass()),
+      *new OxideQCertificateErrorPrivate(std::move(error)),
       parent);
 }
 
@@ -94,36 +95,38 @@ OxideQCertificateError::OxideQCertificateError(
 
   d->error_->SetCancelCallback(
       base::Bind(&OxideQCertificateErrorPrivate::OnCancel,
+                 // The callback cannot run after |d| is deleted, as it
+                 // excusively owns |error_|
                  base::Unretained(d)));
   
-  COMPILE_ASSERT(
+  static_assert(
       OK == static_cast<Error>(oxide::CERT_OK),
-      error_enums_ok_doesnt_match);
-  COMPILE_ASSERT(
+      "Error and oxide::CertError enums dont match: OK");
+  static_assert(
       ErrorBadIdentity == static_cast<Error>(oxide::CERT_ERROR_BAD_IDENTITY),
-      error_enums_bad_identity_doesnt_match);
-  COMPILE_ASSERT(
+      "Error and oxide::CertError enums dont match: ErrorBadIdentity");
+  static_assert(
       ErrorExpired == static_cast<Error>(oxide::CERT_ERROR_EXPIRED),
-      error_enums_expired_doesnt_match);
-  COMPILE_ASSERT(
+      "Error and oxide::CertError enums dont match: ErrorExpired");
+  static_assert(
       ErrorDateInvalid == static_cast<Error>(oxide::CERT_ERROR_DATE_INVALID),
-      error_enums_date_invalid_doesnt_match);
-  COMPILE_ASSERT(
+      "Error and oxide::CertError enums dont match: ErrorDateInvalid");
+  static_assert(
       ErrorAuthorityInvalid ==
         static_cast<Error>(oxide::CERT_ERROR_AUTHORITY_INVALID),
-      error_enums_authority_invalid_doesnt_match);
-  COMPILE_ASSERT(
+      "Error and oxide::CertError enums dont match: ErrorAuthorityInvalid");
+  static_assert(
       ErrorRevoked == static_cast<Error>(oxide::CERT_ERROR_REVOKED),
-      error_enums_revoked_doesnt_match);
-  COMPILE_ASSERT(
+      "Error and oxide::CertError enums dont match: ErrorRevoked");
+  static_assert(
       ErrorInvalid == static_cast<Error>(oxide::CERT_ERROR_INVALID),
-      error_enums_invalid_doesnt_match);
-  COMPILE_ASSERT(
+      "Error and oxide::CertError enums dont match: ErrorInvalid");
+  static_assert(
       ErrorInsecure == static_cast<Error>(oxide::CERT_ERROR_INSECURE),
-      error_enums_insecure_doesnt_match);
-  COMPILE_ASSERT(
+      "Error and oxide::CertError enums dont match: ErrorInsecure");
+  static_assert(
       ErrorGeneric == static_cast<Error>(oxide::CERT_ERROR_GENERIC),
-      error_enums_generic_doesnt_match);
+      "Error and oxide::CertError enums dont match: ErrorGeneric");
 }
 
 OxideQCertificateError::~OxideQCertificateError() {}
@@ -137,7 +140,7 @@ QUrl OxideQCertificateError::url() const {
 bool OxideQCertificateError::isCancelled() const {
   Q_D(const OxideQCertificateError);
 
-  return d->error_->is_cancelled();
+  return d->error_->IsCancelled();
 }
 
 bool OxideQCertificateError::isMainFrame() const {
@@ -164,10 +167,10 @@ bool OxideQCertificateError::strictEnforcement() const {
   return d->error_->strict_enforcement();
 }
 
-OxideQSslCertificate* OxideQCertificateError::certificate() const {
+OxideQSslCertificate OxideQCertificateError::certificate() const {
   Q_D(const OxideQCertificateError);
 
-  return d->certificate_.get();
+  return d->certificate_;
 }
 
 OxideQCertificateError::Error OxideQCertificateError::certError() const {
