@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014 Canonical Ltd.
+// Copyright (C) 2014-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -15,17 +15,19 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef OXIDE_Q_CERTIFICATE_ERROR
-#define OXIDE_Q_CERTIFICATE_ERROR
+#ifndef OXIDE_QTCORE_CERTIFICATE_ERROR
+#define OXIDE_QTCORE_CERTIFICATE_ERROR
 
-#include <QObject>
-#include <QtGlobal>
-#include <QUrl>
+#include <QtCore/QObject>
+#include <QtCore/QtGlobal>
+#include <QtCore/QUrl>
+
+#include <OxideQtCore/oxideqglobal.h>
+#include <OxideQtCore/oxideqsslcertificate.h>
 
 class OxideQCertificateErrorPrivate;
-class OxideQSslCertificate;
 
-class Q_DECL_EXPORT OxideQCertificateError Q_DECL_FINAL : public QObject {
+class OXIDE_QTCORE_EXPORT OxideQCertificateError : public QObject {
   Q_OBJECT
 
   Q_PROPERTY(QUrl url READ url CONSTANT)
@@ -36,7 +38,7 @@ class Q_DECL_EXPORT OxideQCertificateError Q_DECL_FINAL : public QObject {
   Q_PROPERTY(bool overridable READ overridable CONSTANT)
   Q_PROPERTY(bool strictEnforcement READ strictEnforcement CONSTANT)
 
-  Q_PROPERTY(OxideQSslCertificate* certificate READ certificate CONSTANT)
+  Q_PROPERTY(OxideQSslCertificate certificate READ certificate CONSTANT)
   Q_PROPERTY(Error certError READ certError CONSTANT)
 
   Q_ENUMS(Error)
@@ -57,7 +59,7 @@ class Q_DECL_EXPORT OxideQCertificateError Q_DECL_FINAL : public QObject {
     ErrorGeneric
   };
 
-  ~OxideQCertificateError();
+  ~OxideQCertificateError() Q_DECL_OVERRIDE;
 
   QUrl url() const;
   QUrl embedder() const;
@@ -70,7 +72,7 @@ class Q_DECL_EXPORT OxideQCertificateError Q_DECL_FINAL : public QObject {
   bool overridable() const;
   bool strictEnforcement() const;
 
-  OxideQSslCertificate* certificate() const;
+  OxideQSslCertificate certificate() const;
   Error certError() const;
 
  public Q_SLOTS:
@@ -81,10 +83,10 @@ class Q_DECL_EXPORT OxideQCertificateError Q_DECL_FINAL : public QObject {
   void cancelled();
 
  private:
-  OxideQCertificateError(OxideQCertificateErrorPrivate& dd,
-                         QObject* parent = nullptr);
+  Q_DECL_HIDDEN OxideQCertificateError(OxideQCertificateErrorPrivate& dd,
+                                       QObject* parent = nullptr);
 
   QScopedPointer<OxideQCertificateErrorPrivate> d_ptr;
 };
 
-#endif // OXIDE_Q_CERTIFICATE_ERROR
+#endif // OXIDE_QTCORE_CERTIFICATE_ERROR
