@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014 Canonical Ltd.
+// Copyright (C) 2014-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -26,8 +26,8 @@
 #include "qt/core/api/oxideqpermissionrequest.h"
 
 namespace oxide {
+class MediaAccessPermissionRequest;
 class PermissionRequest;
-class SimplePermissionRequest;
 }
 
 class OxideQPermissionRequest;
@@ -38,6 +38,9 @@ class OxideQPermissionRequestPrivate {
  public:
   virtual ~OxideQPermissionRequestPrivate();
 
+  static OxideQPermissionRequest* Create(
+      scoped_ptr<oxide::PermissionRequest> request);
+
  protected:
   OxideQPermissionRequestPrivate(scoped_ptr<oxide::PermissionRequest> request);
 
@@ -45,41 +48,39 @@ class OxideQPermissionRequestPrivate {
   scoped_ptr<oxide::PermissionRequest> request_;
 
  private:
+  bool canRespond() const;
+
   void OnCancelled();
 };
 
-class OxideQSimplePermissionRequestPrivate
+class OxideQGeolocationPermissionRequestPrivate
     : public OxideQPermissionRequestPrivate {
  public:
-  virtual ~OxideQSimplePermissionRequestPrivate();
-
-  static OxideQSimplePermissionRequest* Create(
-      scoped_ptr<oxide::SimplePermissionRequest> request);
-
- protected:
-  OxideQSimplePermissionRequestPrivate(
-      scoped_ptr<oxide::SimplePermissionRequest> request);
-
- private:
-  friend class OxideQSimplePermissionRequest;
-
-  bool canRespond() const;
-  oxide::SimplePermissionRequest* request() const;
-
-  bool did_respond_;
-};
-
-class OxideQGeolocationPermissionRequestPrivate final
-    : public OxideQSimplePermissionRequestPrivate {
- public:
-  ~OxideQGeolocationPermissionRequestPrivate();
+  ~OxideQGeolocationPermissionRequestPrivate() override;
 
   static OxideQGeolocationPermissionRequest* Create(
-      scoped_ptr<oxide::SimplePermissionRequest> request);
+      scoped_ptr<oxide::PermissionRequest> request);
 
  private:
   OxideQGeolocationPermissionRequestPrivate(
-      scoped_ptr<oxide::SimplePermissionRequest> request);
+      scoped_ptr<oxide::PermissionRequest> request);
+};
+
+class OxideQMediaAccessPermissionRequestPrivate
+    : public OxideQPermissionRequestPrivate {
+ public:
+  ~OxideQMediaAccessPermissionRequestPrivate() override;
+
+  static OxideQMediaAccessPermissionRequest* Create(
+      scoped_ptr<oxide::MediaAccessPermissionRequest> request);
+
+ private:
+  friend class OxideQMediaAccessPermissionRequest;
+
+  OxideQMediaAccessPermissionRequestPrivate(
+      scoped_ptr<oxide::MediaAccessPermissionRequest> request);
+
+  oxide::MediaAccessPermissionRequest* request() const;
 };
 
 #endif // _OXIDE_QT_CORE_API_PERMISSION_REQUEST_P_H_
