@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014 Canonical Ltd.
+// Copyright (C) 2014-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -15,28 +15,30 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef OXIDE_Q_SECURITY_STATUS
-#define OXIDE_Q_SECURITY_STATUS
+#ifndef OXIDE_QTCORE_SECURITY_STATUS
+#define OXIDE_QTCORE_SECURITY_STATUS
 
-#include <QObject>
-#include <QScopedPointer>
-#include <QtGlobal>
+#include <QtCore/QObject>
+#include <QtCore/QScopedPointer>
+#include <QtCore/QtGlobal>
+#include <QtCore/QVariant>
+
+#include <OxideQtCore/oxideqglobal.h>
 
 class OxideQSecurityStatusPrivate;
-class OxideQSslCertificate;
 
-class Q_DECL_EXPORT OxideQSecurityStatus Q_DECL_FINAL : public QObject {
+class OXIDE_QTCORE_EXPORT OxideQSecurityStatus : public QObject {
   Q_OBJECT
 
   Q_PROPERTY(SecurityLevel securityLevel READ securityLevel NOTIFY securityLevelChanged)
-  Q_PROPERTY(ContentStatusFlags contentStatus READ contentStatus NOTIFY contentStatusChanged)
-  Q_PROPERTY(CertStatusFlags certStatus READ certStatus NOTIFY certStatusChanged)
+  Q_PROPERTY(ContentStatus contentStatus READ contentStatus NOTIFY contentStatusChanged)
+  Q_PROPERTY(CertStatus certStatus READ certStatus NOTIFY certStatusChanged)
 
-  Q_PROPERTY(OxideQSslCertificate* certificate READ certificate NOTIFY certificateChanged)
+  Q_PROPERTY(QVariant certificate READ certificate NOTIFY certificateChanged)
 
   Q_ENUMS(SecurityLevel)
-  Q_FLAGS(ContentStatusFlags)
-  Q_FLAGS(CertStatusFlags)
+  Q_FLAGS(ContentStatus)
+  Q_FLAGS(CertStatus)
  
   Q_DECLARE_PRIVATE(OxideQSecurityStatus)
   Q_DISABLE_COPY(OxideQSecurityStatus)
@@ -51,14 +53,14 @@ class Q_DECL_EXPORT OxideQSecurityStatus Q_DECL_FINAL : public QObject {
     SecurityLevelError
   };
 
-  enum ContentStatus {
+  enum ContentStatusFlags {
     ContentStatusNormal = 0,
     ContentStatusDisplayedInsecure = 1 << 0,
     ContentStatusRanInsecure = 1 << 1
   };
-  Q_DECLARE_FLAGS(ContentStatusFlags, ContentStatus)
+  Q_DECLARE_FLAGS(ContentStatus, ContentStatusFlags)
 
-  enum CertStatus {
+  enum CertStatusFlags {
     CertStatusOk = 0,
     CertStatusBadIdentity = 1 << 0,
     CertStatusExpired = 1 << 1,
@@ -70,15 +72,15 @@ class Q_DECL_EXPORT OxideQSecurityStatus Q_DECL_FINAL : public QObject {
     CertStatusInsecure = 1 << 7,
     CertStatusGenericError = 1 << 8
   };
-  Q_DECLARE_FLAGS(CertStatusFlags, CertStatus)
+  Q_DECLARE_FLAGS(CertStatus, CertStatusFlags)
 
-  ~OxideQSecurityStatus();
+  ~OxideQSecurityStatus() Q_DECL_OVERRIDE;
 
   SecurityLevel securityLevel() const;
-  ContentStatusFlags contentStatus() const;
-  CertStatusFlags certStatus() const;
+  ContentStatus contentStatus() const;
+  CertStatus certStatus() const;
 
-  OxideQSslCertificate* certificate() const;
+  QVariant certificate() const;
 
  Q_SIGNALS:
   void securityLevelChanged();
@@ -87,10 +89,9 @@ class Q_DECL_EXPORT OxideQSecurityStatus Q_DECL_FINAL : public QObject {
   void certificateChanged();
 
  private:
-  OxideQSecurityStatus(OxideQSecurityStatusPrivate& dd,
-                       QObject* parent = nullptr);
+  Q_DECL_HIDDEN OxideQSecurityStatus();
 
   QScopedPointer<OxideQSecurityStatusPrivate> d_ptr;
 };
 
-#endif // OXIDE_Q_SECURITY_STATUS
+#endif // OXIDE_QTCORE_SECURITY_STATUS
