@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014 Canonical Ltd.
+// Copyright (C) 2014-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -15,20 +15,23 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef OXIDE_Q_SSL_CERTIFICATE
-#define OXIDE_Q_SSL_CERTIFICATE
+#ifndef OXIDE_QTCORE_SSL_CERTIFICATE
+#define OXIDE_QTCORE_SSL_CERTIFICATE
 
-#include <QDateTime>
-#include <QObject>
-#include <QScopedPointer>
-#include <QString>
-#include <QStringList>
-#include <QtGlobal>
+#include <QtCore/QDateTime>
+#include <QtCore/QMetaType>
+#include <QtCore/QSharedDataPointer>
+#include <QtCore/QString>
+#include <QtCore/QStringList>
+#include <QtCore/QtGlobal>
+#include <QtCore/QVariant>
 
-class OxideQSslCertificatePrivate;
+#include <OxideQtCore/oxideqglobal.h>
 
-class Q_DECL_EXPORT OxideQSslCertificate Q_DECL_FINAL : public QObject {
-  Q_OBJECT
+class OxideQSslCertificateData;
+
+class OXIDE_QTCORE_EXPORT OxideQSslCertificate {
+  Q_GADGET
 
   Q_PROPERTY(QString serialNumber READ serialNumber CONSTANT)
   Q_PROPERTY(QString subjectDisplayName READ subjectDisplayName CONSTANT)
@@ -39,13 +42,10 @@ class Q_DECL_EXPORT OxideQSslCertificate Q_DECL_FINAL : public QObject {
 
   Q_PROPERTY(bool isExpired READ isExpired CONSTANT)
 
-  Q_PROPERTY(OxideQSslCertificate* issuer READ issuer CONSTANT)
+  Q_PROPERTY(QVariant issuer READ issuer CONSTANT)
 
   Q_ENUMS(PrincipalAttr)
 
-  Q_DECLARE_PRIVATE(OxideQSslCertificate)
-  Q_DISABLE_COPY(OxideQSslCertificate)
-
  public:
 
   enum PrincipalAttr {
@@ -57,8 +57,15 @@ class Q_DECL_EXPORT OxideQSslCertificate Q_DECL_FINAL : public QObject {
     PrincipalAttrStateOrProvinceName
   };
 
+  OxideQSslCertificate();
   ~OxideQSslCertificate();
 
+  OxideQSslCertificate(const OxideQSslCertificate& other);
+  OxideQSslCertificate operator=(const OxideQSslCertificate& other);
+
+  bool operator==(const OxideQSslCertificate& other) const;
+  bool operator!=(const OxideQSslCertificate& other) const;
+
   QString serialNumber() const;
 
   QString subjectDisplayName() const;
@@ -74,16 +81,20 @@ class Q_DECL_EXPORT OxideQSslCertificate Q_DECL_FINAL : public QObject {
 
   bool isExpired() const;
 
-  OxideQSslCertificate* issuer() const;
-  Q_INVOKABLE OxideQSslCertificate* copy() const;
+  QVariant issuer() const;
+  Q_INVOKABLE OxideQSslCertificate copy() const;
 
   Q_INVOKABLE QString toPem() const;
 
- private:
-  OxideQSslCertificate(OxideQSslCertificatePrivate& dd,
-                       QObject* parent = nullptr);
+  bool isValid() const;
 
-  QScopedPointer<OxideQSslCertificatePrivate> d_ptr;
+ private:
+  friend class OxideQSslCertificateData;
+  OxideQSslCertificate(const QSharedDataPointer<OxideQSslCertificateData>& dd);
+
+  QSharedDataPointer<OxideQSslCertificateData> d;
 };
 
-#endif // OXIDE_Q_SSL_CERTIFICATE
+Q_DECLARE_METATYPE(OxideQSslCertificate)
+
+#endif // OXIDE_QTCORE_SSL_CERTIFICATE
