@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014 Canonical Ltd.
+// Copyright (C) 2014-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,28 +18,29 @@
 #ifndef _OXIDE_QT_CORE_API_SSL_CERTIFICATE_P_H_
 #define _OXIDE_QT_CORE_API_SSL_CERTIFICATE_P_H_
 
+#include <QSharedData>
 #include <QtGlobal>
 
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 
-class OxideQSslCertificate;
-
 namespace net {
 class X509Certificate;
 }
 
-class OxideQSslCertificatePrivate final {
- public:
-  ~OxideQSslCertificatePrivate();
+class OxideQSslCertificate;
 
-  static OxideQSslCertificate* Create(net::X509Certificate* cert,
-                                      QObject* parent = nullptr);
+class OxideQSslCertificateData : public QSharedData {
+ public:
+  ~OxideQSslCertificateData();
+
+  static OxideQSslCertificate Create(net::X509Certificate* cert);
 
  private:
   friend class OxideQSslCertificate;
 
-  OxideQSslCertificatePrivate(net::X509Certificate* cert);
+  OxideQSslCertificateData();
+  OxideQSslCertificateData(net::X509Certificate* cert);
 
   scoped_refptr<net::X509Certificate> x509_cert_;
   mutable scoped_ptr<OxideQSslCertificate> issuer_;
