@@ -45,23 +45,30 @@
 #include "net/url_request/url_request.h"
 #include "url/gurl.h"
 
-#include "qt/core/api/oxideqnetworkcallbackevents.h"
 #include "qt/core/api/oxideqnetworkcallbackevents_p.h"
-#include "qt/core/api/oxideqstoragepermissionrequest.h"
-#include "qt/core/api/oxideqstoragepermissionrequest_p.h"
+#include "qt/core/api/oxideqnetworkcallbackevents_p_p.h"
 #include "qt/core/browser/oxide_qt_url_request_delegated_job.h"
 #include "qt/core/browser/oxide_qt_user_script.h"
 #include "qt/core/glue/oxide_qt_web_context_proxy_client.h"
+#include "shared/browser/media/oxide_media_capture_devices_context.h"
 #include "shared/browser/oxide_browser_context.h"
 #include "shared/browser/oxide_browser_context_delegate.h"
 #include "shared/browser/oxide_browser_process_main.h"
+#include "shared/browser/oxide_devtools_manager.h"
+#include "shared/browser/oxide_user_agent_settings.h"
 #include "shared/browser/oxide_user_script_master.h"
+#include "shared/browser/permissions/oxide_temporary_saved_permission_context.h"
 
 #include "oxide_qt_browser_startup.h"
 
 namespace oxide {
 namespace qt {
 
+using oxide::DevToolsManager;
+using oxide::MediaCaptureDevicesContext;
+using oxide::UserAgentSettings;
+using oxide::UserScriptMaster;
+
 namespace {
 
 const unsigned kDefaultDevtoolsPort = 8484;
@@ -106,12 +113,7 @@ class WebContext::BrowserContextDelegate
                           net::HttpRequestHeaders* headers) override;
   int OnBeforeRedirect(net::URLRequest* request,
                        const GURL& new_location) override;
-  oxide::StoragePermission CanAccessStorage(const GURL& url,
-                                            const GURL& first_party_url,
-                                            bool write,
-                                            oxide::StorageType type) override;
-  bool GetUserAgentOverride(const GURL& url,
-                            std::string* user_agent) override;
+  std::string GetUserAgentOverride(const GURL& url) override;
   bool IsCustomProtocolHandlerRegistered(
       const std::string& scheme) const override;
   oxide::URLRequestDelegatedJob* CreateCustomURLRequestJob(
@@ -136,7 +138,9 @@ struct WebContext::ConstructProperties {
         session_cookie_mode(content::CookieStoreConfig::EPHEMERAL_SESSION_COOKIES),
         popup_blocker_enabled(true),
         devtools_enabled(false),
-        devtools_port(kDefaultDevtoolsPort) {}
+        devtools_port(-1),
+        legacy_user_agent_override_enabled(false),
+        do_not_track(false) {}
 
   std::string product;
   std::string user_agent;
@@ -151,6 +155,11 @@ struct WebContext::ConstructProperties {
   int devtools_port;
   std::string devtools_ip;
   std::vector<std::string> host_mapping_rules;
+  std::string default_audio_capture_device_id;
+  std::string default_video_capture_device_id;
+  std::vector<UserAgentSettings::UserAgentOverride> user_agent_overrides;
+  bool legacy_user_agent_override_enabled;
+  bool do_not_track;
 };
 
 class SetCookiesContext : public base::RefCounted<SetCookiesContext> {
@@ -265,41 +274,17 @@ int WebContext::BrowserContextDelegate::OnBeforeRedirect(
   return eventp->request_cancelled ? net::ERR_ABORTED : net::OK;
 }
 
-oxide::StoragePermission WebContext::BrowserContextDelegate::CanAccessStorage(
-    const GURL& url,
-    const GURL& first_party_url,
-    bool write,
-    oxide::StorageType type) {
+std::string WebContext::BrowserContextDelegate::GetUserAgentOverride(
+    const GURL& url) {
   QSharedPointer<WebContextProxyClient::IOClient> io_client = GetIOClient();
   if (!io_client) {
-    return oxide::STORAGE_PERMISSION_UNDEFINED;
+    return std::string();
   }
 
-  OxideQStoragePermissionRequest req(
-      QUrl(QString::fromStdString(url.spec())),
-      QUrl(QString::fromStdString(first_party_url.spec())),
-      write,
-      static_cast<OxideQStoragePermissionRequest::Type>(type));
+  QString user_agent = io_client->GetUserAgentOverride(
+      QUrl(QString::fromStdString(url.spec())));
 
-  io_client->HandleStoragePermissionRequest(&req);
-
-  return OxideQStoragePermissionRequestPrivate::get(&req)->permission;
-}
-
-bool WebContext::BrowserContextDelegate::GetUserAgentOverride(
-    const GURL& url,
-    std::string* user_agent) {
-  QSharedPointer<WebContextProxyClient::IOClient> io_client = GetIOClient();
-  if (!io_client) {
-    return false;
-  }
-
-  QString new_user_agent;
-  bool overridden = io_client->GetUserAgentOverride(
-      QUrl(QString::fromStdString(url.spec())), &new_user_agent);
-
-  *user_agent = new_user_agent.toStdString();
-  return overridden;
+  return user_agent.toStdString();
 }
 
 bool WebContext::BrowserContextDelegate::IsCustomProtocolHandlerRegistered(
@@ -359,15 +344,15 @@ void WebContext::UpdateUserScripts() {
 
   for (int i = 0; i < user_scripts_.size(); ++i) {
     UserScript* script = UserScript::FromProxyHandle(user_scripts_.at(i));
-    if (script->state() == UserScript::Loading ||
-        script->state() == UserScript::Constructing) {
+    if (!script || script->state() == UserScript::Loading) {
       return;
     } else if (script->state() == UserScript::Loaded) {
       scripts.push_back(script->impl());
     }
   }
 
-  context_->UserScriptManager().SerializeUserScriptsAndSendUpdates(scripts);
+  UserScriptMaster::Get(context_.get())
+      ->SerializeUserScriptsAndSendUpdates(scripts);
 }
 
 void WebContext::CookieSetCallback(
@@ -437,31 +422,37 @@ WebContext::WebContext(WebContextProxyClient* client)
       weak_factory_(this) {
   delegate_ = new BrowserContextDelegate(weak_factory_.GetWeakPtr());
 
-  COMPILE_ASSERT(
+  static_assert(
       CookiePolicyAllowAll == static_cast<CookiePolicy>(
         net::StaticCookiePolicy::ALLOW_ALL_COOKIES),
-      cookie_enums_allowall_doesnt_match);
-  COMPILE_ASSERT(
+      "CookiePolicy and net::StaticCookiePolicy::Type values don't match: "
+      "CookiePolicyAllowAll");
+  static_assert(
       CookiePolicyBlockAll == static_cast<CookiePolicy>(
         net::StaticCookiePolicy::BLOCK_ALL_COOKIES),
-      cookie_enums_blockall_doesnt_match);
-  COMPILE_ASSERT(
+      "CookiePolicy and net::StaticCookiePolicy::Type values don't match: "
+      "CookiePolicyBlockAll");
+  static_assert(
       CookiePolicyBlockThirdParty == static_cast<CookiePolicy>(
         net::StaticCookiePolicy::BLOCK_ALL_THIRD_PARTY_COOKIES),
-      cookie_enums_blockall3rdparty_doesnt_match);
+      "CookiePolicy and net::StaticCookiePolicy::Type values don't match: "
+      "CookiePolicyBlockThirdParty");
 
-  COMPILE_ASSERT(
+  static_assert(
       SessionCookieModeEphemeral == static_cast<SessionCookieMode>(
         content::CookieStoreConfig::EPHEMERAL_SESSION_COOKIES),
-      session_cookie_mode_enums_ephemeral_doesnt_match);
-  COMPILE_ASSERT(
+      "SessionCookieMode and net::CookieStoreConfig::SessionCookieMode values "
+      "don't match: SessionCookieModeEphemeral");
+  static_assert(
       SessionCookieModePersistent == static_cast<SessionCookieMode>(
         content::CookieStoreConfig::PERSISTANT_SESSION_COOKIES),
-      session_cookie_mode_enums_persistent_doesnt_match);
-  COMPILE_ASSERT(
+      "SessionCookieMode and net::CookieStoreConfig::SessionCookieMode values "
+      "don't match: SessionCookieModePersistent");
+  static_assert(
       SessionCookieModeRestored == static_cast<SessionCookieMode>(
         content::CookieStoreConfig::RESTORED_SESSION_COOKIES),
-      session_cookie_mode_enums_restored_doesnt_match);
+      "SessionCookieMode and net::CookieStoreConfig::SessionCookieMode values "
+      "don't match: SessionCookieModeRestored");
 }
 
 WebContext::~WebContext() {
@@ -471,6 +462,7 @@ WebContext::~WebContext() {
 
   if (context_.get()) {
     context_->SetDelegate(nullptr);
+    MediaCaptureDevicesContext::Get(context_.get())->set_client(nullptr);
   }
 }
 
@@ -516,25 +508,56 @@ oxide::BrowserContext* WebContext::GetContext() {
       construct_props_->data_path,
       construct_props_->cache_path,
       construct_props_->max_cache_size_hint,
-      construct_props_->session_cookie_mode,
-      construct_props_->devtools_enabled,
-      construct_props_->devtools_port,
-      construct_props_->devtools_ip);
+      construct_props_->session_cookie_mode);
   params.host_mapping_rules = construct_props_->host_mapping_rules;
 
   context_ = oxide::BrowserContext::Create(params);
 
+  UserAgentSettings* ua_settings = UserAgentSettings::Get(context_.get());
+
   if (!construct_props_->product.empty()) {
-    context_->SetProduct(construct_props_->product);
+    ua_settings->SetProduct(construct_props_->product);
   }
   if (!construct_props_->user_agent.empty()) {
-    context_->SetUserAgent(construct_props_->user_agent);
+    ua_settings->SetUserAgent(construct_props_->user_agent);
   }
   if (!construct_props_->accept_langs.empty()) {
-    context_->SetAcceptLangs(construct_props_->accept_langs);
+    ua_settings->SetAcceptLangs(construct_props_->accept_langs);
   }
+  ua_settings->SetUserAgentOverrides(construct_props_->user_agent_overrides);
+  ua_settings->SetLegacyUserAgentOverrideEnabled(
+      construct_props_->legacy_user_agent_override_enabled);
+
   context_->SetCookiePolicy(construct_props_->cookie_policy);
   context_->SetIsPopupBlockerEnabled(construct_props_->popup_blocker_enabled);
+  context_->SetDoNotTrack(construct_props_->do_not_track);
+
+  MediaCaptureDevicesContext* dc =
+      MediaCaptureDevicesContext::Get(context_.get());
+
+  if (!construct_props_->default_audio_capture_device_id.empty()) {
+    if (!dc->SetDefaultAudioDeviceId(
+        construct_props_->default_audio_capture_device_id)) {
+      client_->DefaultAudioCaptureDeviceChanged();
+    }
+  }
+  if (!construct_props_->default_video_capture_device_id.empty()) {
+    if (!dc->SetDefaultVideoDeviceId(
+        construct_props_->default_video_capture_device_id)) {
+      client_->DefaultVideoCaptureDeviceChanged();
+    }
+  }
+
+  dc->set_client(this);
+
+  DevToolsManager* devtools = DevToolsManager::Get(context_.get());
+  if (!construct_props_->devtools_ip.empty()) {
+    devtools->SetAddress(construct_props_->devtools_ip);
+  }
+  if (construct_props_->devtools_port != -1) {
+    devtools->SetPort(construct_props_->devtools_port);
+  }
+  devtools->SetEnabled(construct_props_->devtools_enabled);
 
   context_->SetDelegate(delegate_.get());
 
@@ -576,7 +599,8 @@ void WebContext::makeDefault() {
 
 QString WebContext::product() const {
   if (IsInitialized()) {
-    return QString::fromStdString(context_->GetProduct());
+    return QString::fromStdString(
+        UserAgentSettings::Get(context_.get())->GetProduct());
   }
 
   return QString::fromStdString(construct_props_->product);
@@ -584,7 +608,8 @@ QString WebContext::product() const {
 
 void WebContext::setProduct(const QString& product) {
   if (IsInitialized()) {
-    context_->SetProduct(product.toStdString());
+    UserAgentSettings::Get(context_.get())->SetProduct(
+        product.toStdString());
   } else {
     construct_props_->product = product.toStdString();
   }
@@ -592,7 +617,8 @@ void WebContext::setProduct(const QString& product) {
 
 QString WebContext::userAgent() const {
   if (IsInitialized()) {
-    return QString::fromStdString(context_->GetUserAgent());
+    return QString::fromStdString(
+        UserAgentSettings::Get(context_.get())->GetUserAgent());
   }
 
   return QString::fromStdString(construct_props_->user_agent);
@@ -600,7 +626,8 @@ QString WebContext::userAgent() const {
 
 void WebContext::setUserAgent(const QString& user_agent) {
   if (IsInitialized()) {
-    context_->SetUserAgent(user_agent.toStdString());
+    UserAgentSettings::Get(context_.get())->SetUserAgent(
+        user_agent.toStdString());
   } else {
     construct_props_->user_agent = user_agent.toStdString();
   }
@@ -652,7 +679,8 @@ void WebContext::setCachePath(const QUrl& url) {
 
 QString WebContext::acceptLangs() const {
   if (IsInitialized()) {
-    return QString::fromStdString(context_->GetAcceptLangs());
+    return QString::fromStdString(
+        UserAgentSettings::Get(context_.get())->GetAcceptLangs());
   }
 
   return QString::fromStdString(construct_props_->accept_langs);
@@ -660,7 +688,8 @@ QString WebContext::acceptLangs() const {
 
 void WebContext::setAcceptLangs(const QString& langs) {
   if (IsInitialized()) {
-    context_->SetAcceptLangs(langs.toStdString());
+    UserAgentSettings::Get(context_.get())->SetAcceptLangs(
+        langs.toStdString());
   } else {
     construct_props_->accept_langs = langs.toStdString();
   }
@@ -728,41 +757,51 @@ void WebContext::setPopupBlockerEnabled(bool enabled) {
 
 bool WebContext::devtoolsEnabled() const {
   if (IsInitialized()) {
-    return context_->GetDevtoolsEnabled();
+    return DevToolsManager::Get(context_.get())->enabled();
   }
 
   return construct_props_->devtools_enabled;
 }
 
 void WebContext::setDevtoolsEnabled(bool enabled) {
-  DCHECK(!IsInitialized());
-  construct_props_->devtools_enabled = enabled;
+  if (IsInitialized()) {
+    DevToolsManager::Get(context_.get())->SetEnabled(enabled);
+  } else {
+    construct_props_->devtools_enabled = enabled;
+  }
 }
 
 int WebContext::devtoolsPort() const {
   if (IsInitialized()) {
-    return context_->GetDevtoolsPort();
+    return DevToolsManager::Get(context_.get())->port();
   }
 
   return construct_props_->devtools_port;
 }
 
 void WebContext::setDevtoolsPort(int port) {
-  DCHECK(!IsInitialized());
-  construct_props_->devtools_port = port;
+  if (IsInitialized()) {
+    DevToolsManager::Get(context_.get())->SetPort(port);
+  } else {
+    construct_props_->devtools_port = port;
+  }
 }
 
 QString WebContext::devtoolsBindIp() const {
   if (IsInitialized()) {
-    return QString::fromStdString(context_->GetDevtoolsBindIp());
+    return QString::fromStdString(
+        DevToolsManager::Get(context_.get())->address());
   }
 
   return QString::fromStdString(construct_props_->devtools_ip);
 }
 
 void WebContext::setDevtoolsBindIp(const QString& ip) {
-  DCHECK(!IsInitialized());
-  construct_props_->devtools_ip = ip.toStdString();
+  if (IsInitialized()) {
+    DevToolsManager::Get(context_.get())->SetAddress(ip.toStdString());
+  } else {
+    construct_props_->devtools_ip = ip.toStdString();
+  }
 }
 
 int WebContext::setCookies(const QUrl& url,
@@ -798,7 +837,9 @@ int WebContext::setCookies(const QUrl& url,
         expiry,
         cookie.isSecure(),
         cookie.isHttpOnly(),
-        false,
+        false, // first_party
+        false, // enforce_prefixes
+        false, // enforce_strict_secure
         net::COOKIE_PRIORITY_DEFAULT,
         base::Bind(&WebContext::CookieSetCallback,
                    weak_factory_.GetWeakPtr(), ctxt, cookie));
@@ -883,7 +924,7 @@ void WebContext::setHostMappingRules(const QStringList& rules) {
 void WebContext::setAllowedExtraUrlSchemes(const QStringList& schemes) {
   std::set<std::string> set;
   for (int i = 0; i < schemes.size(); ++i) {
-    set.insert(base::StringToLowerASCII(schemes.at(i).toStdString()));
+    set.insert(base::ToLowerASCII(schemes.at(i).toStdString()));
   }
   delegate_->SetAllowedExtraURLSchemes(set);
 }
@@ -901,5 +942,136 @@ void WebContext::setMaxCacheSizeHint(int size) {
   construct_props_->max_cache_size_hint = size;
 }
 
+QString WebContext::defaultAudioCaptureDeviceId() const {
+  if (IsInitialized()) {
+    return QString::fromStdString(
+        MediaCaptureDevicesContext::Get(context_.get())
+          ->GetDefaultAudioDeviceId());
+  }
+
+  return QString::fromStdString(
+      construct_props_->default_audio_capture_device_id);
+}
+
+bool WebContext::setDefaultAudioCaptureDeviceId(const QString& id) {
+  if (IsInitialized()) {
+    return MediaCaptureDevicesContext::Get(context_.get())
+        ->SetDefaultAudioDeviceId(id.toStdString());
+  }
+
+  // XXX(chrisccoulson): We don't check if this is a valid ID here
+  construct_props_->default_audio_capture_device_id = id.toStdString();
+  client_->DefaultAudioCaptureDeviceChanged();
+  return true;
+}
+
+QString WebContext::defaultVideoCaptureDeviceId() const {
+  if (IsInitialized()) {
+    return QString::fromStdString(
+        MediaCaptureDevicesContext::Get(context_.get())
+          ->GetDefaultVideoDeviceId());
+  }
+
+  return QString::fromStdString(
+      construct_props_->default_video_capture_device_id);
+}
+
+bool WebContext::setDefaultVideoCaptureDeviceId(const QString& id) {
+  if (IsInitialized()) {
+    return MediaCaptureDevicesContext::Get(context_.get())
+        ->SetDefaultVideoDeviceId(id.toStdString());
+  }
+
+  // XXX(chrisccoulson): We don't check if this is a valid ID here
+  construct_props_->default_video_capture_device_id = id.toStdString();
+  client_->DefaultVideoCaptureDeviceChanged();
+  return true;
+}
+
+QList<WebContextProxy::UserAgentOverride>
+WebContext::userAgentOverrides() const {
+  QList<UserAgentOverride> rv;
+
+  std::vector<UserAgentSettings::UserAgentOverride> overrides;
+  if (IsInitialized()) {
+    overrides =
+        UserAgentSettings::Get(context_.get())->GetUserAgentOverrides();
+  } else {
+    overrides = construct_props_->user_agent_overrides;
+  }
+
+  for (const auto& entry : overrides) {
+    rv.append(
+        qMakePair(QString::fromStdString(entry.first),
+                  QString::fromStdString(entry.second)));
+  }
+
+  return rv;
+}
+
+void WebContext::setUserAgentOverrides(
+    const QList<UserAgentOverride>& overrides) {
+  std::vector<UserAgentSettings::UserAgentOverride> o;
+  for (auto it = overrides.begin(); it != overrides.end(); ++it) {
+    o.push_back(
+        std::make_pair((*it).first.toStdString(),
+                       (*it).second.toStdString()));
+  }
+
+  if (IsInitialized()) {
+    UserAgentSettings::Get(context_.get())->SetUserAgentOverrides(o);
+  } else {
+    construct_props_->user_agent_overrides = o;
+  }
+}
+
+void WebContext::clearTemporarySavedPermissionStatuses() {
+  if (!context_.get()) {
+    return;
+  }
+
+  context_->GetTemporarySavedPermissionContext()->Clear();
+  if (!context_->HasOffTheRecordContext()) {
+    return;
+  }
+
+  context_->GetOffTheRecordContext()
+      ->GetTemporarySavedPermissionContext()
+      ->Clear();
+}
+
+void WebContext::setLegacyUserAgentOverrideEnabled(bool enabled) {
+  if (IsInitialized()) {
+    UserAgentSettings::Get(context_.get())->SetLegacyUserAgentOverrideEnabled(
+        enabled);
+  } else {
+    construct_props_->legacy_user_agent_override_enabled = enabled;
+  }
+}
+
+void WebContext::DefaultAudioDeviceChanged() {
+  client_->DefaultAudioCaptureDeviceChanged();
+}
+
+void WebContext::DefaultVideoDeviceChanged() {
+  client_->DefaultVideoCaptureDeviceChanged();
+}
+
+bool WebContext::doNotTrack() const {
+    if (IsInitialized()) {
+    return context_->GetDoNotTrack();
+  }
+
+  return construct_props_->do_not_track;
+}
+
+void WebContext::setDoNotTrack(bool dnt) {
+  if (IsInitialized()) {
+    context_->SetDoNotTrack(dnt);
+  } else {
+    construct_props_->do_not_track = dnt;
+  }
+}
+
 } // namespace qt
 } // namespace oxide
