@@ -15,8 +15,8 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "oxideqquickwebcontext.h"
 #include "oxideqquickwebcontext_p.h"
-#include "oxideqquickwebcontext_p_p.h"
 
 #include <limits>
 
@@ -30,18 +30,20 @@
 #include <QtQuickVersion>
 #include <QWeakPointer>
 
-#include "qt/core/api/oxideqnetworkcallbackevents.h"
-#include "qt/core/api/oxideqstoragepermissionrequest.h"
+#include "qt/core/api/oxideqglobal_p.h"
+#include "qt/core/api/oxideqnetworkcallbackevents_p.h"
 #include "qt/quick/oxide_qquick_init.h"
 
 #include "oxideqquickcookiemanager_p.h"
+#include "oxideqquickuserscript.h"
 #include "oxideqquickuserscript_p.h"
-#include "oxideqquickuserscript_p_p.h"
 #include "oxideqquickwebcontextdelegateworker_p.h"
 #include "oxideqquickwebcontextdelegateworker_p_p.h"
 #include "oxidequseragentoverriderequest_p.h"
 #include "oxidequseragentoverriderequest_p_p.h"
 
+using oxide::qt::WebContextProxy;
+
 namespace {
 
 QVariant networkCookiesToVariant(const QList<QNetworkCookie>& cookies) {
@@ -67,6 +69,46 @@ QVariant networkCookiesToVariant(const QList<QNetworkCookie>& cookies) {
   return QVariant(list);
 }
 
+WebContextProxy::UserAgentOverride ToUserAgentOverride(
+    const QVariant& v,
+    bool* valid) {
+  *valid = false;
+
+  QVariantList entry = v.toList();
+  if (entry.size() != 2) {
+    return WebContextProxy::UserAgentOverride();
+  }
+
+  WebContextProxy::UserAgentOverride rv;
+
+  QRegExp re;
+  if (entry[0].canConvert<QRegExp>()) {
+    re = entry[0].toRegExp();
+  } else {
+    re = QRegExp(entry[0].toString());
+  }
+
+  if (!re.isValid()) {
+    return WebContextProxy::UserAgentOverride();
+  }
+
+  *valid = true;
+
+  rv.first = re.pattern();
+  rv.second = entry[1].toString();
+
+  return rv;
+}
+
+QVariant FromUserAgentOverride(
+    const WebContextProxy::UserAgentOverride& o) {
+  QVariantList rv;
+  rv.append(o.first);
+  rv.append(o.second);
+
+  return rv;
+}
+
 }
 
 namespace oxide {
@@ -82,14 +124,11 @@ class WebContextIODelegate : public oxide::qt::WebContextProxyClient::IOClient {
   void OnBeforeURLRequest(OxideQBeforeURLRequestEvent* event) override;
   void OnBeforeRedirect(OxideQBeforeRedirectEvent* event) override;
   void OnBeforeSendHeaders(OxideQBeforeSendHeadersEvent* event) override;
-  void HandleStoragePermissionRequest(
-      OxideQStoragePermissionRequest* req) override;
-  bool GetUserAgentOverride(const QUrl& url, QString* user_agent) override;
+  QString GetUserAgentOverride(const QUrl& url) override;
 
   QMutex lock;
 
   QWeakPointer<IOThreadController> network_request_delegate;
-  QWeakPointer<IOThreadController> storage_access_permission_delegate;
   QWeakPointer<IOThreadController> user_agent_override_delegate;
 };
 
@@ -135,42 +174,23 @@ void WebContextIODelegate::OnBeforeRedirect(
   emit delegate->CallEntryPointInWorker("onBeforeRedirect", event);
 }
 
-void WebContextIODelegate::HandleStoragePermissionRequest(
-    OxideQStoragePermissionRequest* req) {
-  QSharedPointer<IOThreadController> delegate;
-  {
-    QMutexLocker locker(&lock);
-    delegate = storage_access_permission_delegate.toStrongRef();
-  }
-  if (!delegate) {
-    return;
-  }
-
-  delegate->CallEntryPointInWorker("onStoragePermissionRequest", req);
-}
-
-bool WebContextIODelegate::GetUserAgentOverride(const QUrl& url,
-                                                QString* user_agent) {
+QString WebContextIODelegate::GetUserAgentOverride(const QUrl& url) {
   QSharedPointer<IOThreadController> delegate;
   {
     QMutexLocker locker(&lock);
     delegate = user_agent_override_delegate.toStrongRef();
   }
   if (!delegate) {
-    return false;
+    return QString();
   }
 
-  bool did_override = false;
-
   OxideQUserAgentOverrideRequest req(url);
-
   delegate->CallEntryPointInWorker("onGetUserAgentOverride", &req);
 
   OxideQUserAgentOverrideRequestPrivate* p =
       OxideQUserAgentOverrideRequestPrivate::get(&req);
-  *user_agent = p->user_agent;
 
-  return p->did_override;
+  return p->user_agent;
 }
 
 } // namespace qquick
@@ -183,11 +203,11 @@ OXIDE_Q_IMPL_PROXY_HANDLE_CONVERTER(OxideQQuickWebContext,
 
 OxideQQuickWebContextPrivate::OxideQQuickWebContextPrivate(
     OxideQQuickWebContext* q)
-    : oxide::qt::WebContextProxyHandle(oxide::qt::WebContextProxy::create(this), q),
+    : oxide::qt::WebContextProxyHandle(WebContextProxy::create(this), q),
       constructed_(false),
       io_(new oxide::qquick::WebContextIODelegate()),
       network_request_delegate_(nullptr),
-      storage_access_permission_delegate_(nullptr),
+      unused_storage_access_permission_delegate_(nullptr),
       user_agent_override_delegate_(nullptr),
       cookie_manager_(nullptr) {}
 
@@ -232,7 +252,7 @@ void OxideQQuickWebContextPrivate::userScript_append(
 
 int OxideQQuickWebContextPrivate::userScript_count(
     QQmlListProperty<OxideQQuickUserScript>* prop) {
-  oxide::qt::WebContextProxy* p =
+  WebContextProxy* p =
       OxideQQuickWebContextPrivate::get(
         static_cast<OxideQQuickWebContext*>(prop->object))->proxy();
 
@@ -242,7 +262,7 @@ int OxideQQuickWebContextPrivate::userScript_count(
 OxideQQuickUserScript* OxideQQuickWebContextPrivate::userScript_at(
     QQmlListProperty<OxideQQuickUserScript>* prop,
     int index) {
-  oxide::qt::WebContextProxy* p =
+  WebContextProxy* p =
       OxideQQuickWebContextPrivate::get(
         static_cast<OxideQQuickWebContext*>(prop->object))->proxy();
 
@@ -258,7 +278,7 @@ void OxideQQuickWebContextPrivate::userScript_clear(
     QQmlListProperty<OxideQQuickUserScript>* prop) {
   OxideQQuickWebContext* context =
       static_cast<OxideQQuickWebContext *>(prop->object);
-  oxide::qt::WebContextProxy* p =
+  WebContextProxy* p =
       OxideQQuickWebContextPrivate::get(context)->proxy();
 
   while (p->userScripts().size() > 0) {
@@ -350,6 +370,18 @@ void OxideQQuickWebContextPrivate::DestroyDefault() {
   delete q;
 }
 
+void OxideQQuickWebContextPrivate::DefaultAudioCaptureDeviceChanged() {
+  Q_Q(OxideQQuickWebContext);
+
+  emit q->defaultAudioCaptureDeviceIdChanged();
+}
+
+void OxideQQuickWebContextPrivate::DefaultVideoCaptureDeviceChanged() {
+  Q_Q(OxideQQuickWebContext);
+
+  emit q->defaultVideoCaptureDeviceIdChanged();
+}
+
 OxideQQuickWebContextPrivate::~OxideQQuickWebContextPrivate() {}
 
 void OxideQQuickWebContextPrivate::delegateWorkerDestroyed(
@@ -372,6 +404,10 @@ OxideQQuickWebContextPrivate* OxideQQuickWebContextPrivate::get(
   return context->d_func();
 }
 
+void OxideQQuickWebContextPrivate::clearTemporarySavedPermissionStatuses() {
+  proxy()->clearTemporarySavedPermissionStatuses();
+}
+
 bool OxideQQuickWebContextPrivate::isInitialized() const {
   return proxy()->isInitialized();
 }
@@ -394,16 +430,25 @@ int OxideQQuickWebContextPrivate::deleteAllCookies() {
   return proxy()->deleteAllCookies();
 }
 
+void OxideQQuickWebContext::classBegin() {}
+
+void OxideQQuickWebContext::componentComplete() {
+  Q_D(OxideQQuickWebContext);
+
+  d->constructed_ = true;
+  emit d->constructed();
+}
+
 OxideQQuickWebContext::OxideQQuickWebContext(QObject* parent) :
-    QObject(parent),
-    d_ptr(new OxideQQuickWebContextPrivate(this)) {
+    QObject(parent) {
+  oxide::qquick::EnsureChromiumStarted();
+  d_ptr.reset(new OxideQQuickWebContextPrivate(this));
+
   Q_D(OxideQQuickWebContext);
 
   QSharedPointer<oxide::qt::WebContextProxyClient::IOClient> io =
       qSharedPointerCast<oxide::qt::WebContextProxyClient::IOClient>(d->io_);
   d->proxy()->init(io.toWeakRef());
-
-  oxide::qquick::EnsureChromiumStarted();
 }
 
 OxideQQuickWebContext::~OxideQQuickWebContext() {
@@ -418,23 +463,13 @@ OxideQQuickWebContext::~OxideQQuickWebContext() {
   // These call back in to us when destroyed, so delete them now in order
   // to avoid a reentrancy crash
   delete d->network_request_delegate_;
-  delete d->storage_access_permission_delegate_;
+  delete d->unused_storage_access_permission_delegate_;
   delete d->user_agent_override_delegate_;
 }
 
-void OxideQQuickWebContext::classBegin() {}
-
-void OxideQQuickWebContext::componentComplete() {
-  Q_D(OxideQQuickWebContext);
-
-  d->constructed_ = true;
-  emit d->constructed();
-}
-
 // static
 OxideQQuickWebContext* OxideQQuickWebContext::defaultContext(bool create) {
-  oxide::qt::WebContextProxyHandle* h =
-      oxide::qt::WebContextProxy::defaultContext();
+  oxide::qt::WebContextProxyHandle* h = WebContextProxy::defaultContext();
   if (h) {
     return OxideQQuickWebContextPrivate::fromProxyHandle(h);
   }
@@ -448,7 +483,7 @@ OxideQQuickWebContext* OxideQQuickWebContext::defaultContext(bool create) {
   QQmlEngine::setObjectOwnership(c, QQmlEngine::CppOwnership);
 
   OxideQQuickWebContextPrivate::get(c)->proxy()->makeDefault();
-  Q_ASSERT(oxide::qt::WebContextProxy::defaultContext());
+  Q_ASSERT(WebContextProxy::defaultContext());
 
   return c;
 }
@@ -637,15 +672,15 @@ OxideQQuickWebContext::CookiePolicy OxideQQuickWebContext::cookiePolicy() const
   Q_STATIC_ASSERT(
       CookiePolicyAllowAll ==
       static_cast<CookiePolicy>(
-        oxide::qt::WebContextProxy::CookiePolicyAllowAll));
+        WebContextProxy::CookiePolicyAllowAll));
   Q_STATIC_ASSERT(
       CookiePolicyBlockAll ==
       static_cast<CookiePolicy>(
-        oxide::qt::WebContextProxy::CookiePolicyBlockAll));
+        WebContextProxy::CookiePolicyBlockAll));
   Q_STATIC_ASSERT(
       CookiePolicyBlockThirdParty ==
       static_cast<CookiePolicy>(
-        oxide::qt::WebContextProxy::CookiePolicyBlockThirdParty));
+        WebContextProxy::CookiePolicyBlockThirdParty));
 
   return static_cast<CookiePolicy>(d->proxy()->cookiePolicy());
 }
@@ -653,8 +688,8 @@ OxideQQuickWebContext::CookiePolicy OxideQQuickWebContext::cookiePolicy() const
 void OxideQQuickWebContext::setCookiePolicy(CookiePolicy policy) {
   Q_D(OxideQQuickWebContext);
 
-  oxide::qt::WebContextProxy::CookiePolicy p =
-      static_cast<oxide::qt::WebContextProxy::CookiePolicy>(policy);
+  WebContextProxy::CookiePolicy p =
+      static_cast<WebContextProxy::CookiePolicy>(policy);
 
   if (policy == cookiePolicy()) {
     return;
@@ -672,15 +707,15 @@ OxideQQuickWebContext::sessionCookieMode() const {
   Q_STATIC_ASSERT(
       SessionCookieModeEphemeral ==
       static_cast<SessionCookieMode>(
-        oxide::qt::WebContextProxy::SessionCookieModeEphemeral));
+        WebContextProxy::SessionCookieModeEphemeral));
   Q_STATIC_ASSERT(
       SessionCookieModePersistent ==
       static_cast<SessionCookieMode>(
-        oxide::qt::WebContextProxy::SessionCookieModePersistent));
+        WebContextProxy::SessionCookieModePersistent));
   Q_STATIC_ASSERT(
       SessionCookieModeRestored ==
       static_cast<SessionCookieMode>(
-        oxide::qt::WebContextProxy::SessionCookieModeRestored));
+        WebContextProxy::SessionCookieModeRestored));
   return static_cast<SessionCookieMode>(d->proxy()->sessionCookieMode());
 }
 
@@ -694,8 +729,8 @@ void OxideQQuickWebContext::setSessionCookieMode(SessionCookieMode mode) {
     return;
   }
 
-  oxide::qt::WebContextProxy::SessionCookieMode m =
-      static_cast<oxide::qt::WebContextProxy::SessionCookieMode>(mode);
+  WebContextProxy::SessionCookieMode m =
+      static_cast<WebContextProxy::SessionCookieMode>(mode);
 
   if (mode == sessionCookieMode()) {
     return;
@@ -765,33 +800,28 @@ OxideQQuickWebContextDelegateWorker*
 OxideQQuickWebContext::storageAccessPermissionDelegate() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->storage_access_permission_delegate_;
+  return d->unused_storage_access_permission_delegate_;
 }
 
 void OxideQQuickWebContext::setStorageAccessPermissionDelegate(
     OxideQQuickWebContextDelegateWorker* delegate) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->storage_access_permission_delegate_ == delegate) {
+  if (d->unused_storage_access_permission_delegate_ == delegate) {
     return;
   }
 
+  WARN_DEPRECATED_API_USAGE() <<
+      "OxideQQuickWebContext: storageAccessPermissionDelegate is deprecated "
+      "and no longer works";
+
   if (delegate && !d->prepareToAttachDelegateWorker(delegate)) {
     return;
   }
 
-  QSharedPointer<webcontextdelegateworker::IOThreadController> io_delegate;
-  if (delegate) {
-    io_delegate = OxideQQuickWebContextDelegateWorkerPrivate::get(
-        delegate)->io_thread_controller();
-  }
-
-  OxideQQuickWebContextDelegateWorker* old = d->storage_access_permission_delegate_;
-  d->storage_access_permission_delegate_ = delegate;
-  {
-    QMutexLocker lock(&d->io_->lock);
-    d->io_->storage_access_permission_delegate = io_delegate.toWeakRef();
-  }
+  OxideQQuickWebContextDelegateWorker* old =
+      d->unused_storage_access_permission_delegate_;
+  d->unused_storage_access_permission_delegate_ = delegate;
 
   d->detachedDelegateWorker(old);
 
@@ -813,6 +843,10 @@ void OxideQQuickWebContext::setUserAgentOverrideDelegate(
     return;
   }
 
+  WARN_DEPRECATED_API_USAGE() <<
+      "OxideQQuickWebContext: userAgentOverrideDelegate is deprecated. "
+      "Please use userAgentOverrides instead";
+
   if (delegate && !d->prepareToAttachDelegateWorker(delegate)) {
     return;
   }
@@ -832,6 +866,8 @@ void OxideQQuickWebContext::setUserAgentOverrideDelegate(
 
   d->detachedDelegateWorker(old);
 
+  d->proxy()->setLegacyUserAgentOverrideEnabled(!!delegate);
+
   emit userAgentOverrideDelegateChanged();
 }
 
@@ -844,13 +880,6 @@ bool OxideQQuickWebContext::devtoolsEnabled() const {
 void OxideQQuickWebContext::setDevtoolsEnabled(bool enabled) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->proxy()->isInitialized()) {
-    qWarning() <<
-        "OxideQQuickWebContext: Cannot set devToolsEnabled once the context "
-        "is in use";
-    return;
-  }
-
   if (devtoolsEnabled() == enabled) {
     return;
   }
@@ -869,10 +898,10 @@ int OxideQQuickWebContext::devtoolsPort() const {
 void OxideQQuickWebContext::setDevtoolsPort(int port) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->proxy()->isInitialized()) {
+  if (devtoolsEnabled() && d->proxy()->isInitialized()) {
     qWarning() <<
-        "OxideQQuickWebContext: Cannot set devToolsPort once the context is "
-        "in use";
+        "OxideQQuickWebContext: Cannot set devtoolsPort whilst "
+        "devtoolsEnabled is set to true";
     return;
   }
 
@@ -880,6 +909,16 @@ void OxideQQuickWebContext::setDevtoolsPort(int port) {
     return;
   }
 
+  int min, max;
+  WebContextProxy::getValidDevtoolsPorts(&min, &max);
+
+  if (port < min || port > max) {
+    qWarning() <<
+        "OxideQQuickWebContext: devtoolsPort was set to an invalid value. "
+        "It must be set between " << min << " and " << max;
+    return;
+  }
+
   d->proxy()->setDevtoolsPort(port);
 
   emit devtoolsPortChanged();
@@ -894,10 +933,10 @@ QString OxideQQuickWebContext::devtoolsBindIp() const {
 void OxideQQuickWebContext::setDevtoolsBindIp(const QString& bindIp) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->proxy()->isInitialized()) {
+  if (devtoolsEnabled() && d->proxy()->isInitialized()) {
     qWarning() <<
-        "OxideQQuickWebContext: Cannot set devToolsBindIp once the context is "
-        "in use";
+        "OxideQQuickWebContext: Cannot set devtoolsBindIp whilst "
+        "devtoolsEnabled is set to true";
     return;
   }
 
@@ -905,6 +944,13 @@ void OxideQQuickWebContext::setDevtoolsBindIp(const QString& bindIp) {
     return;
   }
 
+  if (!WebContextProxy::checkIPAddress(bindIp)) {
+    qWarning() <<
+        "OxideQQuickWebContext: devtoolsBindIp was set to an invalid value. "
+        "It must be set to a valid IPv4 or IPv6 address";
+    return;
+  }
+
   d->proxy()->setDevtoolsBindIp(bindIp);
 
   emit devtoolsBindIpChanged();
@@ -1003,4 +1049,110 @@ void OxideQQuickWebContext::setMaxCacheSizeHint(int size) {
   emit maxCacheSizeHintChanged();
 }
 
-#include "moc_oxideqquickwebcontext_p.cpp"
+QString OxideQQuickWebContext::defaultAudioCaptureDeviceId() const {
+  Q_D(const OxideQQuickWebContext);
+
+  return d->proxy()->defaultAudioCaptureDeviceId();
+}
+
+void OxideQQuickWebContext::setDefaultAudioCaptureDeviceId(const QString& id) {
+  Q_D(OxideQQuickWebContext);
+
+  if (defaultAudioCaptureDeviceId() == id) {
+    return;
+  }
+
+  if (!d->proxy()->setDefaultAudioCaptureDeviceId(id)) {
+    qWarning() <<
+        "OxideQQuickWebContext: Invalid defaultAudioCaptureDeviceId \"" <<
+        id << "\"";
+  }
+
+  // Oxide loops back in to us to emit the signal, as the default will clear
+  // if the actual device is removed
+}
+
+QString OxideQQuickWebContext::defaultVideoCaptureDeviceId() const {
+  Q_D(const OxideQQuickWebContext);
+
+  return d->proxy()->defaultVideoCaptureDeviceId();
+}
+
+void OxideQQuickWebContext::setDefaultVideoCaptureDeviceId(const QString& id) {
+  Q_D(OxideQQuickWebContext);
+
+  if (defaultVideoCaptureDeviceId() == id) {
+    return;
+  }
+
+  if (!d->proxy()->setDefaultVideoCaptureDeviceId(id)) {
+    qWarning() <<
+        "OxideQQuickWebContext: Invalid defaultVideoCaptureDeviceId \"" <<
+        id << "\"";
+  }
+
+  // Oxide loops back in to us to emit the signal, as the default will clear
+  // if the actual device is removed
+}
+
+
+QVariantList OxideQQuickWebContext::userAgentOverrides() const {
+  Q_D(const OxideQQuickWebContext);
+
+  QVariantList rv;
+  QList<WebContextProxy::UserAgentOverride> overrides =
+      d->proxy()->userAgentOverrides();
+
+  for (auto it = overrides.begin(); it != overrides.end(); ++it) {
+    rv.append(FromUserAgentOverride(*it));
+  }
+
+  return rv;
+}
+
+bool OxideQQuickWebContext::doNotTrack() const {
+  Q_D(const OxideQQuickWebContext);
+
+  return d->proxy()->doNotTrack();
+}
+
+void OxideQQuickWebContext::setUserAgentOverrides(
+    const QVariantList& overrides) {
+  Q_D(OxideQQuickWebContext);
+
+  QList<WebContextProxy::UserAgentOverride> entries;
+
+  for (auto it = overrides.begin(); it != overrides.end(); ++it) {
+    bool valid = false;
+    WebContextProxy::UserAgentOverride entry = ToUserAgentOverride(*it, &valid);
+
+    if (!valid) {
+      qWarning() <<
+          "OxideQQuickWebContext::userAgentOverride: Each entry must be a "
+          "list of size 2, with the first item being a valid regular "
+          "expression for URL matching and the second item being the user "
+          "agent string";
+      return;
+    }
+
+    entries.append(entry);
+  }
+
+  d->proxy()->setUserAgentOverrides(entries);
+
+  emit userAgentOverridesChanged();
+}
+
+void OxideQQuickWebContext::setDoNotTrack(bool dnt) {
+  Q_D(OxideQQuickWebContext);
+
+  if (doNotTrack() == dnt) {
+    return;
+  }
+
+  d->proxy()->setDoNotTrack(dnt);
+
+  emit doNotTrackEnabledChanged();
+}
+
+#include "moc_oxideqquickwebcontext.cpp"
