@@ -32,9 +32,10 @@
 #include <QVariant>
 
 #include "qt/core/api/oxideqglobal.h"
+#include "qt/quick/oxide_qquick_init.h"
 
+#include "oxideqquickwebcontext.h"
 #include "oxideqquickwebcontext_p.h"
-#include "oxideqquickwebcontext_p_p.h"
 
 // WebContextDelegateWorker runs a script on Chromium's IO thread. It can
 // exchange messages with the UI thread, and provides entry points for
@@ -121,13 +122,14 @@ void Api::sendMessage(const QVariant& message) {
   if (aux.userType() == qMetaTypeId<QJSValue>()) {
     aux = aux.value<QJSValue>().toVariant();
   }
-  if (aux.type() != QVariant::Map &&
-      aux.type() != QVariant::List &&
-      aux.type() != QVariant::StringList) {
+
+  QJsonDocument doc = QJsonDocument::fromVariant(aux);
+  if (doc.isNull()) {
+    qWarning() << "Api::sendMessage: Invalid message from worker";
     return;
   }
 
-  Q_EMIT controller_->sendMessage(aux);
+  Q_EMIT controller_->sendMessage(doc.toVariant());
 }
 
 void IOThreadControllerImpl::CallEntryPointInWorker(
@@ -237,6 +239,7 @@ OxideQQuickWebContextDelegateWorker::OxideQQuickWebContextDelegateWorker() :
     d_ptr(new OxideQQuickWebContextDelegateWorkerPrivate()) {
   Q_D(OxideQQuickWebContextDelegateWorker);
 
+  oxide::qquick::EnsureChromiumStarted();
   d->io_thread_controller_->moveToThread(oxideGetIOThread());
 
   connect(d->io_thread_controller_.data(), SIGNAL(error(const QString&)),
@@ -310,16 +313,16 @@ void OxideQQuickWebContextDelegateWorker::sendMessage(const QVariant& message) {
     aux = aux.value<QJSValue>().toVariant();
   }
 
-  if (aux.type() != QVariant::Map &&
-      aux.type() != QVariant::List &&
-      aux.type() != QVariant::StringList) {
-    qWarning() << "Called WebContextDelegateWorker.sendMessage with an invalid argument" << aux;
+  QJsonDocument doc = QJsonDocument::fromVariant(aux);
+  if (doc.isNull()) {
+    qWarning() <<
+        "OxideQQuickWebContextDelegateWorker::sendMessage: Invalid message";
     return;
   }
 
   QMetaObject::invokeMethod(d->io_thread_controller_.data(),
                             "receiveMessage",
-                            Q_ARG(QVariant, aux));
+                            Q_ARG(QVariant, doc.toVariant()));
 }
 
 #include "oxideqquickwebcontextdelegateworker.moc"
