@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -22,19 +22,24 @@
 #include "base/base_paths.h"
 #include "base/command_line.h"
 #include "base/files/file_path.h"
-#include "base/lazy_instance.h"
+#include "base/i18n/rtl.h"
 #include "base/logging.h"
 #include "base/path_service.h"
+#include "base/sys_info.h"
 #include "content/public/common/content_client.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/renderer/content_renderer_client.h"
 #include "content/public/utility/content_utility_client.h"
+#include "third_party/skia/include/core/SkGraphics.h"
 #include "ui/base/layout.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/base/ui_base_switches.h"
 
 #include "shared/browser/oxide_browser_process_main.h"
 #include "shared/browser/oxide_content_browser_client.h"
+#include "shared/common/oxide_constants.h"
 #include "shared/common/oxide_content_client.h"
+#include "shared/common/oxide_form_factor.h"
 #include "shared/common/oxide_paths.h"
 #include "shared/renderer/oxide_content_renderer_client.h"
 
@@ -42,15 +47,6 @@
 
 namespace oxide {
 
-namespace {
-base::LazyInstance<ContentBrowserClient> g_content_browser_client =
-    LAZY_INSTANCE_INITIALIZER;
-base::LazyInstance<ContentRendererClient> g_content_renderer_client =
-    LAZY_INSTANCE_INITIALIZER;
-base::LazyInstance<::content::ContentUtilityClient> g_content_utility_client =
-    LAZY_INSTANCE_INITIALIZER;
-}
-
 ContentMainDelegate::ContentMainDelegate(PlatformDelegate* delegate)
     : delegate_(delegate) {
   CHECK(delegate_);
@@ -67,8 +63,17 @@ bool ContentMainDelegate::BasicStartupComplete(int* exit_code) {
 }
 
 void ContentMainDelegate::PreSandboxStartup() {
+  base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
+
+  std::string app_locale;
+  if (command_line->HasSwitch(switches::kLang)) {
+    app_locale = command_line->GetSwitchValueASCII(switches::kLang);
+  } else {
+    app_locale = delegate_->GetApplicationLocale();
+  }
   ui::ResourceBundle::InitSharedInstanceWithLocale(
-      std::string(), nullptr,
+      base::i18n::GetCanonicalLocale(app_locale),
+      nullptr,
       ui::ResourceBundle::DO_NOT_LOAD_COMMON_RESOURCES);
 
   base::FilePath dir_exe;
@@ -83,6 +88,14 @@ void ContentMainDelegate::PreSandboxStartup() {
   ui::ResourceBundle::GetSharedInstance().AddDataPackFromPath(
       dir_exe.Append(FILE_PATH_LITERAL("oxide_200_percent.pak")),
       ui::SCALE_FACTOR_200P);
+
+  if (GetFormFactorHint() != FORM_FACTOR_DESKTOP) {
+    // Limit the Skia font cache on mobile
+    const int kMB = 1024 * 1024;
+    size_t font_cache_limit =
+        base::SysInfo::IsLowEndDevice() ? kMB : 8 * kMB;
+    SkGraphics::SetFontCacheLimit(font_cache_limit);
+  }
 }
 
 int ContentMainDelegate::RunProcess(
@@ -104,24 +117,28 @@ void ContentMainDelegate::ProcessExiting(const std::string& process_type) {
 content::ContentBrowserClient*
 ContentMainDelegate::CreateContentBrowserClient() {
   CHECK(BrowserProcessMain::GetInstance()->IsRunning());
+  DCHECK(!content_browser_client_);
+  content_browser_client_.reset(
+      new ContentBrowserClient(delegate_->GetApplicationLocale(),
+                               delegate_->CreateBrowserIntegration()));
 
-  g_content_browser_client.Get().SetPlatformIntegration(
-      delegate_->CreateBrowserIntegration());
-
-  return g_content_browser_client.Pointer();
+  return content_browser_client_.get();
 }
 
 content::ContentRendererClient*
 ContentMainDelegate::CreateContentRendererClient() {
-  //g_content_renderer_client.Get().SetPlatformIntegration(
-  //    delegate_->CreateRendererIntegration());
+  DCHECK(!content_renderer_client_);
+  content_renderer_client_.reset(new ContentRendererClient());
 
-  return g_content_renderer_client.Pointer();
+  return content_renderer_client_.get();
 }
 
 content::ContentUtilityClient*
 ContentMainDelegate::CreateContentUtilityClient() {
-  return g_content_utility_client.Pointer();
+  DCHECK(!content_utility_client_);
+  content_utility_client_.reset(new content::ContentUtilityClient());
+
+  return content_utility_client_.get();
 }
 
 } // namespace oxide
