@@ -19,11 +19,11 @@
 
 #include <utility>
 
-#include "base/lazy_instance.h"
 #include "base/logging.h"
+#include "base/thread_task_runner_handle.h"
 #include "cc/layers/layer.h"
+#include "cc/layers/layer_settings.h"
 #include "cc/output/context_provider.h"
-#include "cc/resources/task_graph_runner.h"
 #include "cc/scheduler/begin_frame_source.h"
 #include "cc/trees/layer_tree_host.h"
 #include "cc/trees/layer_tree_settings.h"
@@ -39,7 +39,9 @@
 #include "url/gurl.h"
 
 #include "oxide_compositor_client.h"
+#include "oxide_compositor_frame_data.h"
 #include "oxide_compositor_frame_handle.h"
+#include "oxide_compositor_observer.h"
 #include "oxide_compositor_output_surface_gl.h"
 #include "oxide_compositor_output_surface_software.h"
 #include "oxide_compositor_software_output_device.h"
@@ -48,9 +50,6 @@
 
 namespace oxide {
 
-base::LazyInstance<cc::TaskGraphRunner> g_task_graph_runner =
-    LAZY_INSTANCE_INITIALIZER;
-
 namespace {
 
 typedef content::WebGraphicsContext3DCommandBufferImpl WGC3DCBI;
@@ -87,56 +86,33 @@ Compositor::Compositor(CompositorClient* client)
     : client_(client),
       num_failed_recreate_attempts_(0),
       device_scale_factor_(1.0f),
-      root_layer_(cc::Layer::Create()),
+      root_layer_(cc::Layer::Create(cc::LayerSettings())),
       proxy_(new CompositorThreadProxy(this)),
       next_output_surface_id_(1),
-      lock_count_(0),
       weak_factory_(this) {
   DCHECK(CalledOnValidThread());
 }
 
 void Compositor::SendSwapCompositorFrameToClient(
-    uint32 surface_id,
-    CompositorFrameHandle* frame) {
+    scoped_ptr<CompositorFrameData> frame) {
   DCHECK(CalledOnValidThread());
   // XXX: What if we are hidden?
   // XXX: Should we check that surface_id matches the last created
   //  surface?
-  client_->CompositorSwapFrame(surface_id, frame);
-}
-
-void Compositor::LockCompositor() {
-  DCHECK(CalledOnValidThread());
-  if (lock_count_++ > 0) {
-    return;
-  }
-
-  if (layer_tree_host_) {
-    layer_tree_host_->SetDeferCommits(true);
-  }
-}
-
-void Compositor::UnlockCompositor() {
-  DCHECK(CalledOnValidThread());
-  DCHECK(lock_count_ > 0);
-  if (--lock_count_ > 0) {
-    return;
-  }
-
-  if (layer_tree_host_) {
-    layer_tree_host_->SetDeferCommits(false);
-  }
+  scoped_refptr<CompositorFrameHandle> handle =
+      new CompositorFrameHandle(proxy_, std::move(frame));
+  client_->CompositorSwapFrame(handle.get());
 }
 
 scoped_ptr<cc::OutputSurface> Compositor::CreateOutputSurface() {
   DCHECK(CalledOnValidThread());
 
-  uint32 output_surface_id = next_output_surface_id_++;
+  uint32_t output_surface_id = next_output_surface_id_++;
 
   if (CompositorUtils::GetInstance()->CanUseGpuCompositing()) {
     scoped_refptr<cc::ContextProvider> context_provider =
         content::ContextProviderCommandBuffer::Create(
-          CreateOffscreenContext3D(), "OxideWebViewCompositor");
+          CreateOffscreenContext3D(), content::CONTEXT_TYPE_UNKNOWN);
     if (!context_provider.get()) {
       return scoped_ptr<cc::OutputSurface>();
     }
@@ -144,7 +120,7 @@ scoped_ptr<cc::OutputSurface> Compositor::CreateOutputSurface() {
         new CompositorOutputSurfaceGL(output_surface_id,
                                       context_provider,
                                       proxy_));
-    return output.Pass();
+    return std::move(output);
   }
 
   scoped_ptr<CompositorSoftwareOutputDevice> output_device(
@@ -152,25 +128,30 @@ scoped_ptr<cc::OutputSurface> Compositor::CreateOutputSurface() {
   scoped_ptr<CompositorOutputSurfaceSoftware> output(
       new CompositorOutputSurfaceSoftware(
         output_surface_id,
-        output_device.Pass(),
+        std::move(output_device),
         proxy_));
-  return output.Pass();
+  return std::move(output);
+}
+
+void Compositor::AddObserver(CompositorObserver* observer) {
+  observers_.AddObserver(observer);
+}
+
+void Compositor::RemoveObserver(CompositorObserver* observer) {
+  observers_.RemoveObserver(observer);
 }
 
 void Compositor::WillBeginMainFrame() {}
 void Compositor::BeginMainFrame(const cc::BeginFrameArgs& args) {}
 void Compositor::BeginMainFrameNotExpectedSoon() {}
 void Compositor::DidBeginMainFrame() {}
-void Compositor::Layout() {}
+void Compositor::UpdateLayerTreeHost() {}
 void Compositor::ApplyViewportDeltas(
     const gfx::Vector2dF& inner_delta,
     const gfx::Vector2dF& outer_delta,
     const gfx::Vector2dF& elastic_overscroll_delta,
     float page_scale,
     float top_controls_delta) {}
-void Compositor::ApplyViewportDeltas(const gfx::Vector2d& scroll_delta,
-                                     float page_scale,
-                                     float top_controls_delta) {}
 
 void Compositor::RequestNewOutputSurface() {
   scoped_ptr<cc::OutputSurface> surface(CreateOutputSurface());
@@ -179,7 +160,7 @@ void Compositor::RequestNewOutputSurface() {
     return;
   }
 
-  layer_tree_host_->SetOutputSurface(surface.Pass());
+  layer_tree_host_->SetOutputSurface(std::move(surface));
 }
 
 void Compositor::DidInitializeOutputSurface() {
@@ -199,11 +180,14 @@ void Compositor::DidFailToInitializeOutputSurface() {
 void Compositor::WillCommit() {}
 
 void Compositor::DidCommit() {
-  client_->CompositorDidCommit();
+  FOR_EACH_OBSERVER(CompositorObserver, observers_, CompositorDidCommit());
 }
 
 void Compositor::DidCommitAndDrawFrame() {}
 void Compositor::DidCompleteSwapBuffers() {}
+void Compositor::RecordFrameTimingEvents(
+    scoped_ptr<cc::FrameTimingTracker::CompositeTimingSet> composite_events,
+    scoped_ptr<cc::FrameTimingTracker::MainFrameTimingSet> main_frame_events) {}
 void Compositor::DidCompletePageScaleAnimation() {}
 
 // static
@@ -216,7 +200,7 @@ scoped_ptr<Compositor> Compositor::Create(CompositorClient* client) {
 }
 
 Compositor::~Compositor() {
-  CHECK_EQ(lock_count_, 0U);
+  FOR_EACH_OBSERVER(CompositorObserver, observers_, OnCompositorDestruction());
   proxy_->CompositorDestroyed();
 }
 
@@ -230,37 +214,36 @@ void Compositor::SetVisibility(bool visible) {
     layer_tree_host_.reset();
   } else if (!layer_tree_host_) {
     cc::LayerTreeSettings settings;
-    settings.renderer_settings.allow_antialiasing = false;
     settings.use_external_begin_frame_source = false;
-    settings.throttle_frame_production = false;
-    // XXX(chrisccoulson): Hangs from 43.0.2357.2
-    //settings.using_synchronous_renderer_compositor = true;
+    settings.renderer_settings.allow_antialiasing = false;
+
+    cc::LayerTreeHost::InitParams params;
+    params.client = this;
+    params.shared_bitmap_manager = content::HostSharedBitmapManager::current();
+    params.gpu_memory_buffer_manager =
+        content::BrowserGpuMemoryBufferManager::current();
+    params.task_graph_runner =
+        CompositorUtils::GetInstance()->GetTaskGraphRunner();
+    params.settings = &settings;
+    params.main_task_runner = base::ThreadTaskRunnerHandle::Get();
 
     layer_tree_host_ = cc::LayerTreeHost::CreateThreaded(
-        this,
-        content::HostSharedBitmapManager::current(),
-        content::BrowserGpuMemoryBufferManager::current(),
-        g_task_graph_runner.Pointer(),
-        settings,
-        base::MessageLoopProxy::current(),
         CompositorUtils::GetInstance()->GetTaskRunner(),
-        nullptr);
+        &params);
 
     layer_tree_host_->SetRootLayer(root_layer_);
     layer_tree_host_->SetVisible(true);
     layer_tree_host_->SetViewportSize(size_);
     layer_tree_host_->SetDeviceScaleFactor(device_scale_factor_);
-
-    if (lock_count_ > 0) {
-      layer_tree_host_->SetDeferCommits(true);
-    }
-
-    layer_tree_host_->SetLayerTreeHostClientReady();
   }
 }
 
 void Compositor::SetDeviceScaleFactor(float scale) {
   DCHECK(CalledOnValidThread());
+  if (scale == device_scale_factor_) {
+    return;
+  }
+
   device_scale_factor_ = scale;
 
   if (layer_tree_host_) {
@@ -284,13 +267,14 @@ void Compositor::SetViewportSize(const gfx::Size& size) {
 
 void Compositor::SetRootLayer(scoped_refptr<cc::Layer> layer) {
   DCHECK(CalledOnValidThread());
+
   root_layer_->RemoveAllChildren();
   if (layer.get()) {
     root_layer_->AddChild(layer);
   }
 }
 
-void Compositor::DidSwapCompositorFrame(uint32 surface_id,
+void Compositor::DidSwapCompositorFrame(uint32_t surface_id,
                                         FrameHandleVector returned_frames) {
   proxy_->DidSwapCompositorFrame(surface_id, std::move(returned_frames));
 }
