@@ -23,6 +23,7 @@
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/scoped_vector.h"
 #include "base/memory/weak_ptr.h"
+#include "base/observer_list.h"
 #include "base/threading/non_thread_safe.h"
 #include "cc/trees/layer_tree_host_client.h"
 #include "ui/gfx/geometry/size.h"
@@ -39,8 +40,9 @@ class Size;
 namespace oxide {
 
 class CompositorClient;
+class CompositorFrameData;
 class CompositorFrameHandle;
-class CompositorLock;
+class CompositorObserver;
 class CompositorThreadProxy;
 
 class Compositor final : public cc::LayerTreeHostClient,
@@ -58,37 +60,33 @@ class Compositor final : public cc::LayerTreeHostClient,
   void SetViewportSize(const gfx::Size& bounds);
   void SetRootLayer(scoped_refptr<cc::Layer> layer);
 
-  void DidSwapCompositorFrame(uint32 surface_id,
+  void DidSwapCompositorFrame(uint32_t surface_id,
                               FrameHandleVector returned_frames);
 
  private:
-  friend class CompositorLock;
+  friend class CompositorObserver;
   friend class CompositorThreadProxy;
 
   Compositor(CompositorClient* client);
 
-  void SendSwapCompositorFrameToClient(uint32 surface_id,
-                                       CompositorFrameHandle* frame);
-
-  void LockCompositor();
-  void UnlockCompositor();
+  void SendSwapCompositorFrameToClient(scoped_ptr<CompositorFrameData> frame);
 
   scoped_ptr<cc::OutputSurface> CreateOutputSurface();
 
+  void AddObserver(CompositorObserver* observer);
+  void RemoveObserver(CompositorObserver* observer);
+
   // cc::LayerTreeHostClient implementation
   void WillBeginMainFrame() final;
   void BeginMainFrame(const cc::BeginFrameArgs& args) final;
   void BeginMainFrameNotExpectedSoon() final;
   void DidBeginMainFrame() final;
-  void Layout() final;
+  void UpdateLayerTreeHost() final;
   void ApplyViewportDeltas(const gfx::Vector2dF& inner_delta,
                            const gfx::Vector2dF& outer_delta,
                            const gfx::Vector2dF& elastic_overscroll_delta,
                            float page_scale,
                            float top_controls_delta) final;
-  void ApplyViewportDeltas(const gfx::Vector2d& scroll_delta,
-                           float page_scale,
-                           float top_controls_delta) final;
   void RequestNewOutputSurface() final;
   void DidInitializeOutputSurface() final;
   void DidFailToInitializeOutputSurface() final;
@@ -96,6 +94,9 @@ class Compositor final : public cc::LayerTreeHostClient,
   void DidCommit() final;
   void DidCommitAndDrawFrame() final;
   void DidCompleteSwapBuffers() final;
+  void RecordFrameTimingEvents(
+      scoped_ptr<cc::FrameTimingTracker::CompositeTimingSet> composite_events,
+      scoped_ptr<cc::FrameTimingTracker::MainFrameTimingSet> main_frame_events) override;
   void DidCompletePageScaleAnimation() final;
 
   CompositorClient* client_;
@@ -110,36 +111,15 @@ class Compositor final : public cc::LayerTreeHostClient,
   scoped_ptr<cc::LayerTreeHost> layer_tree_host_;
   scoped_refptr<CompositorThreadProxy> proxy_;
 
-  uint32 next_output_surface_id_;
+  uint32_t next_output_surface_id_;
 
-  uint32 lock_count_;
+  base::ObserverList<CompositorObserver> observers_;
 
   base::WeakPtrFactory<Compositor> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(Compositor);
 };
 
-class CompositorLock final {
- public:
-  CompositorLock(Compositor* compositor)
-      : compositor_(compositor) {
-    if (compositor_) {
-      compositor_->LockCompositor();
-    }
-  }
-
-  ~CompositorLock() {
-    if (compositor_) {
-      compositor_->UnlockCompositor();
-    }
-  }
-
- private:
-  Compositor* compositor_;
-
-  DISALLOW_COPY_AND_ASSIGN(CompositorLock);
-};
-
 } // namespace oxide
 
 #endif // _OXIDE_SHARED_BROWSER_COMPOSITOR_COMPOSITOR_H_
