@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014 Canonical Ltd.
+// Copyright (C) 2014-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -17,8 +17,12 @@
 
 #include "oxide_compositor_output_surface.h"
 
+#include <utility>
+
+#include "cc/output/compositor_frame_ack.h"
 #include "cc/output/output_surface_client.h"
 
+#include "oxide_compositor_frame_data.h"
 #include "oxide_compositor_thread_proxy.h"
 
 namespace oxide {
@@ -33,8 +37,15 @@ bool CompositorOutputSurface::BindToClient(cc::OutputSurfaceClient* client) {
   return true;
 }
 
+void CompositorOutputSurface::DetachFromClient() {
+  DCHECK(CalledOnValidThread());
+  proxy_->SetOutputSurface(nullptr);
+  cc::OutputSurface::DetachFromClient();
+  DetachFromThread();
+}
+
 CompositorOutputSurface::CompositorOutputSurface(
-    uint32 surface_id,
+    uint32_t surface_id,
     scoped_refptr<cc::ContextProvider> context_provider,
     scoped_refptr<CompositorThreadProxy> proxy)
     : cc::OutputSurface(context_provider),
@@ -44,34 +55,36 @@ CompositorOutputSurface::CompositorOutputSurface(
 }
 
 CompositorOutputSurface::CompositorOutputSurface(
-    uint32 surface_id,
+    uint32_t surface_id,
     scoped_ptr<cc::SoftwareOutputDevice> software_device,
     scoped_refptr<CompositorThreadProxy> proxy)
-    : cc::OutputSurface(software_device.Pass()),
+    : cc::OutputSurface(std::move(software_device)),
       proxy_(proxy),
       surface_id_(surface_id) {
   DetachFromThread();
 }
 
-void CompositorOutputSurface::SwapBuffers(cc::CompositorFrame* frame) {
+void CompositorOutputSurface::DoSwapBuffers(CompositorFrameData* frame) {
   DCHECK(CalledOnValidThread());
+  DCHECK(frame->gl_frame_data || frame->software_frame_data);
+
+  frame->surface_id = surface_id();
+
   proxy_->SwapCompositorFrame(frame);
   client_->DidSwapBuffers();
 }
 
-CompositorOutputSurface::~CompositorOutputSurface() {
-  proxy_->SetOutputSurface(nullptr);
-}
+CompositorOutputSurface::~CompositorOutputSurface() {}
 
 void CompositorOutputSurface::DidSwapBuffers() {
   DCHECK(CalledOnValidThread());
   client_->DidSwapBuffersComplete();
 }
 
-void CompositorOutputSurface::ReclaimResources(
-    const cc::CompositorFrameAck& ack) {
+void CompositorOutputSurface::ReclaimResources(const CompositorFrameAck& ack) {
   DCHECK(CalledOnValidThread());
-  cc::OutputSurface::ReclaimResources(&ack);
+  cc::CompositorFrameAck unused; // Not used for the GL or software renderer
+  cc::OutputSurface::ReclaimResources(&unused);
 }
 
 } // namespace oxide
