@@ -21,12 +21,13 @@
 #include "base/containers/hash_tables.h"
 #include "base/logging.h"
 #include "base/memory/singleton.h"
-#include "base/message_loop/message_loop_proxy.h"
 #include "base/single_thread_task_runner.h"
 #include "base/synchronization/waitable_event.h"
+#include "base/threading/simple_thread.h"
 #include "base/threading/thread.h"
 #include "base/threading/thread_restrictions.h"
 #include "cc/output/context_provider.h"
+#include "cc/raster/single_thread_task_graph_runner.h"
 #include "content/browser/gpu/browser_gpu_channel_host_factory.h"
 #include "content/browser/gpu/gpu_data_manager_impl.h"
 #include "content/common/gpu/client/context_provider_command_buffer.h"
@@ -161,24 +162,24 @@ class CompositorUtilsImpl : public CompositorUtils,
   void GetTextureFromMailbox(
       cc::ContextProvider* context_provider,
       const gpu::Mailbox& mailbox,
-      uint32 sync_point,
+      uint32_t sync_point,
       const CompositorUtils::GetTextureFromMailboxCallback& callback,
       scoped_refptr<base::SingleThreadTaskRunner> task_runner) override;
   void CreateEGLImageFromMailbox(
       cc::ContextProvider* context_provider,
       const gpu::Mailbox& mailbox,
-      uint32 sync_point,
+      uint32_t sync_point,
       const CompositorUtils::CreateEGLImageFromMailboxCallback& callback,
       scoped_refptr<base::SingleThreadTaskRunner> task_runner) override;
-  gfx::GLSurfaceHandle GetSharedSurfaceHandle() override;
   bool CanUseGpuCompositing() const override;
   CompositingMode GetCompositingMode() const override;
+  cc::TaskGraphRunner* GetTaskGraphRunner() const override;
 
   bool CalledOnMainOrCompositorThread() const;
   bool CalledOnGpuThread() const;
 
  private:
-  friend struct DefaultSingletonTraits<CompositorUtilsImpl>;
+  friend struct base::DefaultSingletonTraits<CompositorUtilsImpl>;
 
   CompositorUtilsImpl();
   ~CompositorUtilsImpl() override;
@@ -197,7 +198,7 @@ class CompositorUtilsImpl : public CompositorUtils,
   void DidProcessTask(const base::PendingTask& pending_task) override;
 
   // The client ID for this process's GPU channel
-  int32 client_id_;
+  int32_t client_id_;
 
   // Whether or not we have a shared GL context
   bool has_share_context_;
@@ -231,6 +232,7 @@ class CompositorUtilsImpl : public CompositorUtils,
 
   struct MainData {
     scoped_ptr<CompositorThread> compositor_thread;
+    scoped_ptr<cc::SingleThreadTaskGraphRunner> task_graph_runner;
   } main_unsafe_access_;
 
   struct GpuData {
@@ -239,7 +241,9 @@ class CompositorUtilsImpl : public CompositorUtils,
     bool has_shutdown;
   } gpu_unsafe_access_;
 
+  const MainData& main() const;
   MainData& main();
+
   GpuData& gpu();
 
   DISALLOW_COPY_AND_ASSIGN(CompositorUtilsImpl);
@@ -441,6 +445,11 @@ CompositorUtilsImpl::GpuData& CompositorUtilsImpl::gpu() {
   return gpu_unsafe_access_;
 }
 
+const CompositorUtilsImpl::MainData& CompositorUtilsImpl::main() const {
+  DCHECK(main_thread_checker_.CalledOnValidThread());
+  return main_unsafe_access_;
+}
+
 CompositorUtilsImpl::MainData& CompositorUtilsImpl::main() {
   DCHECK(main_thread_checker_.CalledOnValidThread());
   return main_unsafe_access_;
@@ -448,7 +457,7 @@ CompositorUtilsImpl::MainData& CompositorUtilsImpl::main() {
 
 // static
 CompositorUtilsImpl* CompositorUtilsImpl::GetInstance() {
-  return Singleton<CompositorUtilsImpl>::get();
+  return base::Singleton<CompositorUtilsImpl>::get();
 }
 
 void CompositorUtilsImpl::Initialize(bool has_share_context) {
@@ -461,7 +470,11 @@ void CompositorUtilsImpl::Initialize(bool has_share_context) {
   main().compositor_thread.reset(new CompositorThread());
   main().compositor_thread->Start();
 
-  compositor_task_runner_ = main().compositor_thread->message_loop_proxy();
+  compositor_task_runner_ = main().compositor_thread->task_runner();
+
+  main().task_graph_runner.reset(new cc::SingleThreadTaskGraphRunner());
+  main().task_graph_runner->Start("CompositorTileWorker1",
+                                  base::SimpleThread::Options());
 
   content::CauseForGpuLaunch cause =
       content::CAUSE_FOR_GPU_LAUNCH_WEBGRAPHICSCONTEXT3DCOMMANDBUFFERIMPL_INITIALIZE;
@@ -484,6 +497,9 @@ void CompositorUtilsImpl::Shutdown() {
   // CreateGLFrameHandle
   main().compositor_thread.reset();
 
+  main().task_graph_runner->Shutdown();
+  main().task_graph_runner.reset();
+
   // Detach the GPU thread MessageLoop::TaskObserver, to stop processing
   // and existing incoming requests
   scoped_refptr<base::SingleThreadTaskRunner> gpu_task_runner =
@@ -529,7 +545,7 @@ CompositorUtilsImpl::GetTaskRunner() {
 void CompositorUtilsImpl::GetTextureFromMailbox(
     cc::ContextProvider* context_provider,
     const gpu::Mailbox& mailbox,
-    uint32 sync_point,
+    uint32_t sync_point,
     const CompositorUtils::GetTextureFromMailboxCallback& callback,
     scoped_refptr<base::SingleThreadTaskRunner> task_runner) {
   DCHECK(context_provider);
@@ -569,7 +585,7 @@ void CompositorUtilsImpl::GetTextureFromMailbox(
 void CompositorUtilsImpl::CreateEGLImageFromMailbox(
     cc::ContextProvider* context_provider,
     const gpu::Mailbox& mailbox,
-    uint32 sync_point,
+    uint32_t sync_point,
     const CompositorUtils::CreateEGLImageFromMailboxCallback& callback,
     scoped_refptr<base::SingleThreadTaskRunner> task_runner) {
   DCHECK(context_provider);
@@ -606,13 +622,6 @@ void CompositorUtilsImpl::CreateEGLImageFromMailbox(
   incoming_texture_resource_fetches_.queue.push(info);
 }
 
-gfx::GLSurfaceHandle CompositorUtilsImpl::GetSharedSurfaceHandle() {
-  gfx::GLSurfaceHandle handle(gfx::kNullPluginWindow, gfx::NULL_TRANSPORT);
-  handle.parent_client_id = client_id_;
-
-  return handle;
-}
-
 bool CompositorUtilsImpl::CanUseGpuCompositing() const {
   return GetCompositingMode() != COMPOSITING_MODE_SOFTWARE;
 }
@@ -622,18 +631,22 @@ CompositingMode CompositorUtilsImpl::GetCompositingMode() const {
     return COMPOSITING_MODE_SOFTWARE;
   }
 
-  if (has_share_context_) {
-    return COMPOSITING_MODE_TEXTURE;
-  }
-
   if (gfx::GetGLImplementation() == gfx::kGLImplementationEGLGLES2 &&
       GpuUtils::CanUseEGLImage()) {
     return COMPOSITING_MODE_EGLIMAGE;
   }
 
+  if (has_share_context_) {
+    return COMPOSITING_MODE_TEXTURE;
+  }
+
   return COMPOSITING_MODE_SOFTWARE;
 }
 
+cc::TaskGraphRunner* CompositorUtilsImpl::GetTaskGraphRunner() const {
+  return main().task_graph_runner.get();
+}
+
 bool CompositorUtilsImpl::CalledOnMainOrCompositorThread() const {
   return main_thread_checker_.CalledOnValidThread() ||
          compositor_thread_checker_.CalledOnValidThread();
