@@ -25,7 +25,6 @@
 #include "base/callback.h"
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "ui/gfx/native_widget_types.h"
 
 #include "shared/browser/compositor/oxide_compositing_mode.h"
 
@@ -35,6 +34,7 @@ class SingleThreadTaskRunner;
 
 namespace cc {
 class ContextProvider;
+class TaskGraphRunner;
 }
 
 namespace gpu {
@@ -76,7 +76,7 @@ class CompositorUtils {
   virtual void GetTextureFromMailbox(
       cc::ContextProvider* context_provider,
       const gpu::Mailbox& mailbox,
-      uint32 sync_point,
+      uint32_t sync_point,
       const GetTextureFromMailboxCallback& callback,
       scoped_refptr<base::SingleThreadTaskRunner> task_runner) = 0;
 
@@ -91,16 +91,16 @@ class CompositorUtils {
   virtual void CreateEGLImageFromMailbox(
       cc::ContextProvider* context_provider,
       const gpu::Mailbox& mailbox,
-      uint32 sync_point,
+      uint32_t sync_point,
       const CreateEGLImageFromMailboxCallback& callback,
       scoped_refptr<base::SingleThreadTaskRunner> task_runner) = 0;
 
-  virtual gfx::GLSurfaceHandle GetSharedSurfaceHandle() = 0;
-
   virtual bool CanUseGpuCompositing() const = 0;
 
   virtual CompositingMode GetCompositingMode() const = 0;
 
+  virtual cc::TaskGraphRunner* GetTaskGraphRunner() const = 0;
+
  protected:
   virtual ~CompositorUtils();
 };
