@@ -18,8 +18,9 @@
 #include "oxide_browser_context.h"
 
 #include <algorithm>
-#include <libintl.h>
 #include <limits>
+#include <set>
+#include <utility>
 #include <vector>
 
 #include "base/files/file_enumerator.h"
@@ -28,18 +29,16 @@
 #include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/memory/weak_ptr.h"
-#include "base/strings/stringprintf.h"
 #include "base/supports_user_data.h"
 #include "base/synchronization/lock.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/threading/worker_pool.h"
+#include "components/keyed_service/content/browser_context_dependency_manager.h"
 #include "content/browser/loader/resource_dispatcher_host_impl.h"
 #include "content/public/browser/browser_thread.h"
-#include "content/public/browser/devtools_http_handler.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/resource_context.h"
 #include "content/public/browser/storage_partition.h"
-#include "content/public/common/user_agent.h"
 #include "net/base/host_mapping_rules.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_util.h"
@@ -50,7 +49,6 @@
 #include "net/http/http_server_properties_impl.h"
 #include "net/http/transport_security_persister.h"
 #include "net/http/transport_security_state.h"
-#include "net/socket/tcp_server_socket.h"
 #include "net/ssl/channel_id_service.h"
 #include "net/ssl/channel_id_store.h"
 #include "net/ssl/default_channel_id_store.h"
@@ -60,30 +58,30 @@
 #include "net/url_request/url_request_intercepting_job_factory.h"
 #include "net/url_request/url_request_job_factory_impl.h"
 
-#include "shared/common/chrome_version.h"
+#include "shared/browser/permissions/oxide_permission_manager.h"
+#include "shared/browser/permissions/oxide_temporary_saved_permission_context.h"
 #include "shared/common/oxide_constants.h"
 #include "shared/common/oxide_content_client.h"
-#include "shared/common/oxide_messages.h"
 
 #include "oxide_browser_context_delegate.h"
+#include "oxide_browser_context_destroyer.h"
 #include "oxide_browser_context_observer.h"
 #include "oxide_browser_process_main.h"
-#include "oxide_devtools_http_handler_delegate.h"
+#include "oxide_download_manager_delegate.h"
 #include "oxide_http_user_agent_settings.h"
 #include "oxide_io_thread.h"
 #include "oxide_network_delegate.h"
-#include "oxide_permission_manager.h"
 #include "oxide_ssl_config_service.h"
 #include "oxide_ssl_host_state_delegate.h"
 #include "oxide_url_request_context.h"
 #include "oxide_url_request_delegated_job_factory.h"
-#include "oxide_user_script_master.h"
+#include "oxide_user_agent_settings.h"
 
 namespace oxide {
 
 namespace {
 
-base::LazyInstance<std::vector<BrowserContext *> > g_all_contexts;
+base::LazyInstance<std::set<BrowserContext *> > g_all_contexts;
 
 // Cache was used for the default blockfile backend (CACHE_BACKEND_BLOCKFILE),
 // Cache2 is used since the switch to the simple backend (CACHE_BACKEND_SIMPLE).
@@ -97,13 +95,6 @@ const char kDataScheme[] = "data";
 const char kFileScheme[] = "file";
 const char kFtpScheme[] = "ftp";
 
-const char kBrowserContextKey[] = "oxide_browser_context_data";
-
-const char kDefaultAcceptLanguage[] = "en-us,en";
-
-const char kDevtoolsDefaultServerIp[] = "127.0.0.1";
-const int kBackLog = 1;
-
 void CleanupOldCacheDir(const base::FilePath& path) {
   if (!base::DirectoryExists(path)) {
     return;
@@ -131,66 +122,26 @@ void CleanupOldCacheDir(const base::FilePath& path) {
 
 } // namespace
 
-class TCPServerSocketFactory :
-    public content::DevToolsHttpHandler::ServerSocketFactory {
- public:
-  TCPServerSocketFactory(const std::string& address, int port)
-      : address_(address),
-        port_(port) {}
-
- private:
-  scoped_ptr<net::ServerSocket> CreateForHttpServer() override {
-    scoped_ptr<net::TCPServerSocket> socket(
-        new net::TCPServerSocket(nullptr, net::NetLog::Source()));
-    if (socket->ListenWithAddressAndPort(address_, port_, kBackLog) != net::OK) {
-      return scoped_ptr<net::ServerSocket>();
-    }
-
-    return socket.Pass();
-  }
-
-  scoped_ptr<net::ServerSocket> CreateForTethering(
-      std::string* out_name) override {
-    // Not supported
-    return scoped_ptr<net::ServerSocket>();
-  }
-
-  std::string address_;
-  int port_;
-
-  DISALLOW_COPY_AND_ASSIGN(TCPServerSocketFactory);
-};
-
-class ResourceContextData : public base::SupportsUserData::Data {
- public:
-  ResourceContextData(BrowserContextIOData* context) :
-      context_(context) {}
-  virtual ~ResourceContextData() {}
-
-  BrowserContextIOData* get() const { return context_; }
-
- private:
-  BrowserContextIOData* context_;
-};
-
-class MainURLRequestContextGetter final : public URLRequestContextGetter {
+class MainURLRequestContextGetter : public URLRequestContextGetter {
  public:
   MainURLRequestContextGetter(
       BrowserContextIOData* context,
       content::ProtocolHandlerMap* protocol_handlers,
       content::URLRequestInterceptorScopedVector request_interceptors) :
       context_(context),
-      request_interceptors_(request_interceptors.Pass()) {
+      request_interceptors_(std::move(request_interceptors)) {
     std::swap(protocol_handlers_, *protocol_handlers);
   }
 
-  net::URLRequestContext* GetURLRequestContext() final {
+ private:
+  // URLRequestContextGetter implementation
+  net::URLRequestContext* GetURLRequestContext() override {
     DCHECK_CURRENTLY_ON(content::BrowserThread::IO);
 
     if (!url_request_context_) {
       DCHECK(context_);
       url_request_context_ = context_->CreateMainRequestContext(
-          protocol_handlers_, request_interceptors_.Pass())->AsWeakPtr();
+          protocol_handlers_, std::move(request_interceptors_))->AsWeakPtr();
       context_ = nullptr;
     }
 
@@ -205,68 +156,44 @@ class MainURLRequestContextGetter final : public URLRequestContextGetter {
   base::WeakPtr<URLRequestContext> url_request_context_;
 };
 
-class ResourceContext final : public content::ResourceContext {
+class ResourceContext : public content::ResourceContext {
  public:
-  ResourceContext() :
-      request_context_(nullptr) {}
+  ResourceContext(BrowserContextIOData* io_data)
+      : io_data_(io_data),
+        request_context_(nullptr) {}
 
-  net::HostResolver* GetHostResolver() final {
+ private:
+  friend class BrowserContextIOData; // for setting request_context_ and
+                                     // accessing io_data_
+
+  // content::ResourceContext implementation
+  net::HostResolver* GetHostResolver() override {
     return IOThread::instance()->globals()->host_resolver();
   }
 
-  net::URLRequestContext* GetRequestContext() final {
+  net::URLRequestContext* GetRequestContext() override {
     CHECK(request_context_);
     return request_context_;
   }
 
- private:
-  friend class BrowserContextIOData;
-
+  BrowserContextIOData* io_data_;
   net::URLRequestContext* request_context_;
 
   DISALLOW_COPY_AND_ASSIGN(ResourceContext);
 };
 
-struct BrowserContextSharedData {
-  BrowserContextSharedData(BrowserContext* context,
-                           const BrowserContext::Params& params)
-      : ref_count(0),
-        in_dtor(false),
-        product(base::StringPrintf("Chrome/%s", CHROME_VERSION_STRING)),
-        user_agent_string_is_default(true),
-        user_script_master(context),
-        devtools_enabled(params.devtools_enabled),
-        devtools_port(params.devtools_port),
-        devtools_ip(params.devtools_ip) {}
-
-  mutable int ref_count;
-  mutable bool in_dtor;
-
-  std::string product;
-  bool user_agent_string_is_default;
-  UserScriptMaster user_script_master;
-
-  scoped_ptr<content::DevToolsHttpHandler> devtools_http_handler;
-  bool devtools_enabled;
-  int devtools_port;
-  std::string devtools_ip;
-};
-
 struct BrowserContextSharedIOData {
-  BrowserContextSharedIOData(const BrowserContext::Params& params)
+  BrowserContextSharedIOData(BrowserContextIOData* context,
+                             const BrowserContext::Params& params)
       : path(params.path),
         cache_path(params.cache_path),
         max_cache_size_hint(params.max_cache_size_hint),
         cookie_policy(net::StaticCookiePolicy::ALLOW_ALL_COOKIES),
         session_cookie_mode(params.session_cookie_mode),
         popup_blocker_enabled(true),
-        host_mapping_rules(params.host_mapping_rules) {
-
-   accept_langs = dgettext(OXIDE_GETTEXT_DOMAIN, "AcceptLanguage");
-   if (accept_langs == "AcceptLanguage") {
-     accept_langs = kDefaultAcceptLanguage;
-   }
-  }
+        host_mapping_rules(params.host_mapping_rules),
+        user_agent_settings(new UserAgentSettingsIOData(context)),
+        do_not_track(false) {}
 
   mutable base::Lock lock;
 
@@ -274,15 +201,16 @@ struct BrowserContextSharedIOData {
   base::FilePath cache_path;
   int max_cache_size_hint;
 
-  std::string user_agent_string;
-  std::string accept_langs;
-
   net::StaticCookiePolicy::Type cookie_policy;
   content::CookieStoreConfig::SessionCookieMode session_cookie_mode;
   bool popup_blocker_enabled;
 
   std::vector<std::string> host_mapping_rules;
 
+  scoped_ptr<UserAgentSettingsIOData> user_agent_settings;
+
+  bool do_not_track;
+
   scoped_refptr<BrowserContextDelegate> delegate;
 };
 
@@ -304,23 +232,24 @@ void BrowserContextDelegateTraits::Destruct(const BrowserContextDelegate* x) {
 class BrowserContextIODataImpl : public BrowserContextIOData {
  public:
   BrowserContextIODataImpl(const BrowserContext::Params& params)
-      : data_(params) {}
+      : data_(this, params) {}
 
-  BrowserContextSharedIOData& GetSharedData() final {
+  BrowserContextSharedIOData& GetSharedData() override {
     return data_;
   }
-  const BrowserContextSharedIOData& GetSharedData() const final {
+  const BrowserContextSharedIOData& GetSharedData() const override {
     return data_;
   }
 
  private:
-  content::CookieStoreConfig::SessionCookieMode GetSessionCookieMode() const final {
+  content::CookieStoreConfig::SessionCookieMode
+  GetSessionCookieMode() const override {
     return GetPath().empty() ?
         content::CookieStoreConfig::EPHEMERAL_SESSION_COOKIES :
         data_.session_cookie_mode;
   }
  
-  bool IsOffTheRecord() const final {
+  bool IsOffTheRecord() const override {
     return false;
   }
 
@@ -333,18 +262,19 @@ class OTRBrowserContextIODataImpl : public BrowserContextIOData {
       : original_io_data_(original) {}
 
  private:
-  content::CookieStoreConfig::SessionCookieMode GetSessionCookieMode() const final {
+  content::CookieStoreConfig::SessionCookieMode
+  GetSessionCookieMode() const override {
     return content::CookieStoreConfig::EPHEMERAL_SESSION_COOKIES;
   }
 
-  bool IsOffTheRecord() const final {
+  bool IsOffTheRecord() const override {
     return true;
   }
 
-  BrowserContextSharedIOData& GetSharedData() final {
+  BrowserContextSharedIOData& GetSharedData() override {
     return original_io_data_->GetSharedData();
   }
-  const BrowserContextSharedIOData& GetSharedData() const final {
+  const BrowserContextSharedIOData& GetSharedData() const override {
     return original_io_data_->GetSharedData();
   }
 
@@ -364,11 +294,10 @@ void BrowserContextIOData::Init() {
                                  nullptr, nullptr));
 }
 
-BrowserContextIOData::BrowserContextIOData() :
-    resource_context_(new ResourceContext()) {
-  resource_context_->SetUserData(kBrowserContextKey,
-                                 new ResourceContextData(this));
-}
+BrowserContextIOData::BrowserContextIOData()
+    : resource_context_(new ResourceContext(this)),
+      temporary_saved_permission_context_(
+        new TemporarySavedPermissionContext()) {}
 
 BrowserContextIOData::~BrowserContextIOData() {
   DCHECK_CURRENTLY_ON(content::BrowserThread::IO);
@@ -377,8 +306,7 @@ BrowserContextIOData::~BrowserContextIOData() {
 // static
 BrowserContextIOData* BrowserContextIOData::FromResourceContext(
     content::ResourceContext* resource_context) {
-  return static_cast<ResourceContextData *>(
-      resource_context->GetUserData(kBrowserContextKey))->get();
+  return static_cast<ResourceContext*>(resource_context)->io_data_;
 }
 
 scoped_refptr<BrowserContextDelegate> BrowserContextIOData::GetDelegate() {
@@ -419,16 +347,10 @@ int BrowserContextIOData::GetMaxCacheSizeHint() const {
   return max_cache_size_hint;
 }
 
-std::string BrowserContextIOData::GetAcceptLangs() const {
+bool BrowserContextIOData::GetDoNotTrack() const {
   const BrowserContextSharedIOData& data = GetSharedData();
   base::AutoLock lock(data.lock);
-  return data.accept_langs;
-}
-
-std::string BrowserContextIOData::GetUserAgent() const {
-  const BrowserContextSharedIOData& data = GetSharedData();
-  base::AutoLock lock(data.lock);
-  return data.user_agent_string;
+  return data.do_not_track;
 }
 
 URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
@@ -498,17 +420,17 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
       FROM_HERE,
       base::Bind(&CleanupOldCacheDir, GetCachePath().Append(kCacheDirname)));
 
-  net::HttpCache::BackendFactory* cache_backend = nullptr;
+  scoped_ptr<net::HttpCache::BackendFactory> cache_backend;
   if (IsOffTheRecord() || GetCachePath().empty()) {
     cache_backend = net::HttpCache::DefaultBackend::InMemory(0);
   } else {
-    cache_backend = new net::HttpCache::DefaultBackend(
+    cache_backend.reset(new net::HttpCache::DefaultBackend(
           net::DISK_CACHE,
           net::CACHE_BACKEND_SIMPLE,
           GetCachePath().Append(kCacheDirname2),
           GetMaxCacheSizeHint() * 1024 * 1024, // MB -> bytes
           content::BrowserThread::GetMessageLoopProxyForThread(
-              content::BrowserThread::CACHE));
+              content::BrowserThread::CACHE)));
   }
 
   net::HttpNetworkSession::Params session_params;
@@ -527,11 +449,16 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
   session_params.net_log = context->net_log();
   session_params.host_mapping_rules = host_mapping_rules_.get();
 
+  http_network_session_ =
+      make_scoped_ptr(new net::HttpNetworkSession(session_params));
+
   {
     // Calls QuickStreamFactory constructor which uses base::CPU
     base::ThreadRestrictions::ScopedAllowIO allow_io;
     storage->set_http_transaction_factory(
-        new net::HttpCache(session_params, cache_backend));
+        make_scoped_ptr(new net::HttpCache(http_network_session_.get(),
+                                           std::move(cache_backend),
+                                           true)));
   }
 
   scoped_ptr<net::URLRequestJobFactoryImpl> job_factory(
@@ -541,29 +468,32 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
   for (content::ProtocolHandlerMap::iterator it = protocol_handlers.begin();
        it != protocol_handlers.end();
        ++it) {
-    set_protocol = job_factory->SetProtocolHandler(it->first,
-                                                   it->second.release());
+    set_protocol =
+        job_factory->SetProtocolHandler(it->first,
+                                        make_scoped_ptr(it->second.release()));
     DCHECK(set_protocol);
   }
   protocol_handlers.clear();
 
   set_protocol = job_factory->SetProtocolHandler(
       oxide::kFileScheme,
-      new net::FileProtocolHandler(
+      make_scoped_ptr(new net::FileProtocolHandler(
         content::BrowserThread::GetMessageLoopProxyForThread(
-          content::BrowserThread::FILE)));
+          content::BrowserThread::FILE))));
   DCHECK(set_protocol);
   set_protocol = job_factory->SetProtocolHandler(
-      oxide::kDataScheme, new net::DataProtocolHandler());
+      oxide::kDataScheme,
+      make_scoped_ptr(new net::DataProtocolHandler()));
   DCHECK(set_protocol);
 
   set_protocol = job_factory->SetProtocolHandler(
       oxide::kFtpScheme,
-      new net::FtpProtocolHandler(ftp_transaction_factory_.get()));
+      make_scoped_ptr(new net::FtpProtocolHandler(
+        ftp_transaction_factory_.get())));
   DCHECK(set_protocol);
 
   scoped_ptr<net::URLRequestJobFactory> top_job_factory(
-      new URLRequestDelegatedJobFactory(job_factory.Pass(),
+      new URLRequestDelegatedJobFactory(std::move(job_factory),
                                         this));
 
   for (content::URLRequestInterceptorScopedVector::reverse_iterator it =
@@ -571,11 +501,11 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
        it != request_interceptors.rend();
        ++it) {
     top_job_factory.reset(new net::URLRequestInterceptingJobFactory(
-        top_job_factory.Pass(), make_scoped_ptr(*it)));
+        std::move(top_job_factory), make_scoped_ptr(*it)));
   }
   request_interceptors.weak_clear();
 
-  storage->set_job_factory(top_job_factory.release());
+  storage->set_job_factory(std::move(top_job_factory));
 
   resource_context_->request_context_ = context;
   return main_request_context_.get();
@@ -588,16 +518,6 @@ content::ResourceContext* BrowserContextIOData::GetResourceContext() {
 bool BrowserContextIOData::CanAccessCookies(const GURL& url,
                                             const GURL& first_party_url,
                                             bool write) {
-  scoped_refptr<BrowserContextDelegate> delegate(GetDelegate());
-  if (delegate.get()) {
-    StoragePermission res =
-        delegate->CanAccessStorage(url, first_party_url, write,
-                                   STORAGE_TYPE_COOKIES);
-    if (res != STORAGE_PERMISSION_UNDEFINED) {
-      return res == STORAGE_PERMISSION_ALLOW;
-    }
-  }
-
   net::StaticCookiePolicy policy(GetCookiePolicy());
   if (write) {
     return policy.CanSetCookie(url, first_party_url) == net::OK;
@@ -606,6 +526,15 @@ bool BrowserContextIOData::CanAccessCookies(const GURL& url,
   return policy.CanGetCookies(url, first_party_url) == net::OK;
 }
 
+TemporarySavedPermissionContext*
+BrowserContextIOData::GetTemporarySavedPermissionContext() const {
+  return temporary_saved_permission_context_.get();
+}
+
+UserAgentSettingsIOData* BrowserContextIOData::GetUserAgentSettings() const {
+  return GetSharedData().user_agent_settings.get();
+}
+
 class BrowserContextImpl;
 
 class OTRBrowserContextImpl : public BrowserContext {
@@ -613,112 +542,78 @@ class OTRBrowserContextImpl : public BrowserContext {
   OTRBrowserContextImpl(BrowserContextImpl* original,
                         BrowserContextIODataImpl* original_io_data);
 
- private:
-  friend class BrowserContextImpl; // For the destructor
-
-  virtual ~OTRBrowserContextImpl() {}
-
-  BrowserContext* GetOffTheRecordContext() final {
-    return this;
+  base::WeakPtr<OTRBrowserContextImpl> GetWeakPtr() {
+    return weak_ptr_factory_.GetWeakPtr();
   }
-  BrowserContext* GetOriginalContext() final;
 
-  BrowserContextSharedData& GetSharedData() final;
-  const BrowserContextSharedData& GetSharedData() const final;
+ private:
+  ~OTRBrowserContextImpl() override;
 
-  bool HasOffTheRecordContext() const final { return true; }
+  scoped_refptr<BrowserContext> GetOffTheRecordContext() override {
+    return make_scoped_refptr(this);
+  }
+  BrowserContext* GetOriginalContext() const override;
 
-  BrowserContextImpl* original_context_;
+  bool HasOffTheRecordContext() const override { return true; }
+
+  scoped_refptr<BrowserContextImpl> original_context_;
+
+  base::WeakPtrFactory<OTRBrowserContextImpl> weak_ptr_factory_;
 };
 
 class BrowserContextImpl : public BrowserContext {
  public:
   BrowserContextImpl(const BrowserContext::Params& params);
 
-  BrowserContextSharedData& GetSharedData() final {
-    return data_;
-  }
-  const BrowserContextSharedData& GetSharedData() const final {
-    return data_;
-  }
-
  private:
-  virtual ~BrowserContextImpl();
+  ~BrowserContextImpl() override;
 
-  BrowserContext* GetOffTheRecordContext() final;
+  scoped_refptr<BrowserContext> GetOffTheRecordContext() override;
 
-  BrowserContext* GetOriginalContext() final {
-    return this;
+  BrowserContext* GetOriginalContext() const override {
+    return const_cast<BrowserContextImpl*>(this);
   }
 
-  bool HasOffTheRecordContext() const final {
+  bool HasOffTheRecordContext() const override {
     return otr_context_ != nullptr;
   }
 
-  BrowserContextSharedData data_;
-  OTRBrowserContextImpl* otr_context_;
+  base::WeakPtr<OTRBrowserContextImpl> otr_context_;
 };
 
-BrowserContextSharedData& OTRBrowserContextImpl::GetSharedData() {
-  return original_context_->GetSharedData();
-}
+OTRBrowserContextImpl::~OTRBrowserContextImpl() {}
 
-const BrowserContextSharedData&
-OTRBrowserContextImpl::GetSharedData() const {
-  return original_context_->GetSharedData();
-}
-
-BrowserContext* OTRBrowserContextImpl::GetOriginalContext() {
-  return original_context_;
+BrowserContext* OTRBrowserContextImpl::GetOriginalContext() const {
+  return original_context_.get();
 }
 
 OTRBrowserContextImpl::OTRBrowserContextImpl(
     BrowserContextImpl* original,
     BrowserContextIODataImpl* original_io_data)
     : BrowserContext(new OTRBrowserContextIODataImpl(original_io_data)),
-      original_context_(original) {}
-
-BrowserContextImpl::~BrowserContextImpl() {
-  if (otr_context_) {
-    delete otr_context_;
-    otr_context_ = nullptr;
-  }
+      original_context_(original),
+      weak_ptr_factory_(this) {
+  BrowserContextDependencyManager::GetInstance()
+      ->CreateBrowserContextServices(this);
 }
 
-BrowserContext* BrowserContextImpl::GetOffTheRecordContext() {
+BrowserContextImpl::~BrowserContextImpl() {
+  CHECK(!otr_context_);
+}
+
+scoped_refptr<BrowserContext> BrowserContextImpl::GetOffTheRecordContext() {
   if (!otr_context_) {
-    otr_context_ = new OTRBrowserContextImpl(
+    OTRBrowserContextImpl* context = new OTRBrowserContextImpl(
         this,
         static_cast<BrowserContextIODataImpl *>(io_data()));
+    otr_context_ = context->GetWeakPtr();
   }
 
-  return otr_context_;
+  return make_scoped_refptr(otr_context_.get());
 }
 
 BrowserContextImpl::BrowserContextImpl(const BrowserContext::Params& params)
-    : BrowserContext(new BrowserContextIODataImpl(params)),
-      data_(this, params),
-      otr_context_(nullptr) {
-  io_data()->GetSharedData().user_agent_string =
-      content::BuildUserAgentFromProduct(data_.product);
-
-  if (data_.devtools_enabled &&
-      data_.devtools_port < 65535 &&
-      data_.devtools_port > 1024) {
-    net::IPAddressNumber unused;
-    std::string ip =
-        net::ParseIPLiteralToNumber(data_.devtools_ip, &unused) ?
-          data_.devtools_ip : kDevtoolsDefaultServerIp;
-
-    scoped_ptr<content::DevToolsHttpHandler::ServerSocketFactory> factory(
-        new TCPServerSocketFactory(ip, data_.devtools_port));
-    data_.devtools_http_handler.reset(
-        content::DevToolsHttpHandler::Start(factory.Pass(),
-                                            std::string(),
-                                            new DevtoolsHttpHandlerDelegate(),
-                                            base::FilePath()));
-  }
-
+    : BrowserContext(new BrowserContextIODataImpl(params)) {
   if (!GetPath().empty()) {
     base::FilePath gpu_cache = GetPath().Append(FILE_PATH_LITERAL("GPUCache"));
     content::BrowserThread::PostTask(
@@ -726,6 +621,13 @@ BrowserContextImpl::BrowserContextImpl(const BrowserContext::Params& params)
         FROM_HERE,
         base::Bind(&CleanupOldCacheDir, gpu_cache));
   }
+
+  BrowserContextDependencyManager::GetInstance()
+      ->CreateBrowserContextServices(this);
+}
+
+void BrowserContextTraits::Destruct(const BrowserContext* x) {
+  BrowserContextDestroyer::DestroyContext(const_cast<BrowserContext*>(x));
 }
 
 scoped_ptr<content::ZoomLevelDelegate> BrowserContext::CreateZoomLevelDelegate(
@@ -775,8 +677,8 @@ BrowserContext::GetMediaRequestContextForStoragePartition(
 }
 
 content::DownloadManagerDelegate*
-    BrowserContext::GetDownloadManagerDelegate() {
-  return nullptr;
+BrowserContext::GetDownloadManagerDelegate() {
+  return DownloadManagerDelegate::Get(this);
 }
 
 content::BrowserPluginGuestManager* BrowserContext::GetGuestManager() {
@@ -801,12 +703,17 @@ content::SSLHostStateDelegate* BrowserContext::GetSSLHostStateDelegate() {
 
 content::PermissionManager* BrowserContext::GetPermissionManager() {
   if (!permission_manager_) {
-    permission_manager_.reset(new PermissionManager());
+    permission_manager_.reset(new PermissionManager(this));
   }
 
   return permission_manager_.get();
 }
 
+content::BackgroundSyncController*
+BrowserContext::GetBackgroundSyncController() {
+  return nullptr;
+}
+
 void BrowserContext::AddObserver(BrowserContextObserver* observer) {
   DCHECK(CalledOnValidThread());
   observers_.AddObserver(observer);
@@ -817,18 +724,13 @@ void BrowserContext::RemoveObserver(BrowserContextObserver* observer) {
   observers_.RemoveObserver(observer);
 }
 
-// static
-void BrowserContext::Delete(const BrowserContext* context) {
-  delete const_cast<BrowserContext *>(context)->GetOriginalContext();
-}
-
 BrowserContext::BrowserContext(BrowserContextIOData* io_data) :
     io_data_(io_data) {
   CHECK(BrowserProcessMain::GetInstance()->IsRunning()) <<
       "The main browser process components must be started before " <<
       "creating a context";
 
-  g_all_contexts.Get().push_back(this);
+  g_all_contexts.Get().insert(this);
 
   // Make sure that the cookie store is properly created
   io_data->Init();
@@ -843,14 +745,10 @@ BrowserContext::~BrowserContext() {
                     observers_,
                     OnBrowserContextDestruction());
 
-  std::vector<BrowserContext *>::iterator it;
-  for (std::vector<BrowserContext *>::iterator it = g_all_contexts.Get().begin();
-       it != g_all_contexts.Get().end(); ++it) {
-    if (*it == this) {
-      g_all_contexts.Get().erase(it);
-      break;
-    }
-  }
+  g_all_contexts.Get().erase(this);
+
+  BrowserContextDependencyManager::GetInstance()
+      ->DestroyBrowserContextServices(this);
 
   // Schedule io_data_ to be destroyed on the IO thread
   content::BrowserThread::DeleteSoon(content::BrowserThread::IO,
@@ -864,29 +762,20 @@ scoped_refptr<BrowserContext> BrowserContext::Create(const Params& params) {
   return context;
 }
 
+// static
+void BrowserContext::ForEach(const BrowserContextCallback& callback) {
+  for (auto context : g_all_contexts.Get()) {
+    callback.Run(context);
+  }
+}
+
 // static
 void BrowserContext::AssertNoContextsExist() {
   CHECK_EQ(g_all_contexts.Get().size(), static_cast<size_t>(0));
 }
 
-void BrowserContext::AddRef() const {
-  DCHECK(CalledOnValidThread());
-
-  const BrowserContextSharedData& data = GetSharedData();
-  DCHECK(!data.in_dtor);
-  data.ref_count++;
-}
-
-void BrowserContext::Release() const {
-  DCHECK(CalledOnValidThread());
-
-  const BrowserContextSharedData& data = GetSharedData();
-  DCHECK(!data.in_dtor);
-  DCHECK(data.ref_count > 0);
-  if (--data.ref_count == 0) {
-    data.in_dtor = true;
-    Delete(this);
-  }
+BrowserContextID BrowserContext::GetID() const {
+  return reinterpret_cast<BrowserContextID>(this);
 }
 
 net::URLRequestContextGetter* BrowserContext::CreateRequestContext(
@@ -898,7 +787,7 @@ net::URLRequestContextGetter* BrowserContext::CreateRequestContext(
   main_request_context_getter_ =
       new MainURLRequestContextGetter(io_data(),
                                       protocol_handlers,
-                                      request_interceptors.Pass());
+                                      std::move(request_interceptors));
 
   return main_request_context_getter_.get();
 }
@@ -925,7 +814,7 @@ bool BrowserContext::IsSameContext(BrowserContext* other) const {
   DCHECK(CalledOnValidThread());
   return other->GetOriginalContext() == this ||
          (other->HasOffTheRecordContext() &&
-          other->GetOffTheRecordContext() == this);
+          other->GetOffTheRecordContext().get() == this);
 }
 
 base::FilePath BrowserContext::GetPath() const {
@@ -943,63 +832,6 @@ int BrowserContext::GetMaxCacheSizeHint() const {
   return io_data()->GetMaxCacheSizeHint();
 }
 
-std::string BrowserContext::GetAcceptLangs() const {
-  DCHECK(CalledOnValidThread());
-  return io_data()->GetAcceptLangs();
-}
-
-void BrowserContext::SetAcceptLangs(const std::string& langs) {
-  DCHECK(CalledOnValidThread());
-
-  BrowserContextSharedIOData& data = io_data()->GetSharedData();
-  base::AutoLock lock(data.lock);
-  data.accept_langs = langs;
-}
-
-std::string BrowserContext::GetProduct() const {
-  DCHECK(CalledOnValidThread());
-  return GetSharedData().product;
-}
-
-void BrowserContext::SetProduct(const std::string& product) {
-  DCHECK(CalledOnValidThread());
-
-  BrowserContextSharedData& data = GetSharedData();
-  data.product = product.empty() ?
-      base::StringPrintf("Chrome/%s", CHROME_VERSION_STRING) : product;
-  if (data.user_agent_string_is_default) {
-    SetUserAgent(std::string());
-  }
-}
-
-std::string BrowserContext::GetUserAgent() const {
-  DCHECK(CalledOnValidThread());
-  return io_data()->GetUserAgent();
-}
-
-void BrowserContext::SetUserAgent(const std::string& user_agent) {
-  DCHECK(CalledOnValidThread());
-
-  {
-    BrowserContextSharedIOData& data = io_data()->GetSharedData();
-    base::AutoLock lock(data.lock);
-    data.user_agent_string = user_agent.empty() ?
-        content::BuildUserAgentFromProduct(GetSharedData().product) :
-        user_agent;
-    GetSharedData().user_agent_string_is_default = user_agent.empty();
-  }
-
-  for (content::RenderProcessHost::iterator it =
-          content::RenderProcessHost::AllHostsIterator();
-       !it.IsAtEnd(); it.Advance()) {
-    content::RenderProcessHost* host = it.GetCurrentValue();
-    if (IsSameContext(
-            BrowserContext::FromContent(host->GetBrowserContext()))) {
-      host->Send(new OxideMsg_SetUserAgent(GetUserAgent()));
-    }
-  }
-}
-
 net::StaticCookiePolicy::Type BrowserContext::GetCookiePolicy() const {
   DCHECK(CalledOnValidThread());
   return io_data()->GetCookiePolicy();
@@ -1039,31 +871,11 @@ void BrowserContext::SetIsPopupBlockerEnabled(bool enabled) {
   }
 }
 
-bool BrowserContext::GetDevtoolsEnabled() const {
-  DCHECK(CalledOnValidThread());
-  return GetSharedData().devtools_enabled;
-}
-
-int BrowserContext::GetDevtoolsPort() const {
-  DCHECK(CalledOnValidThread());
-  return GetSharedData().devtools_port;
-}
-
-std::string BrowserContext::GetDevtoolsBindIp() const {
-  DCHECK(CalledOnValidThread());
-  return GetSharedData().devtools_ip;
-}
-
 const std::vector<std::string>& BrowserContext::GetHostMappingRules() const {
   DCHECK(CalledOnValidThread());
   return io_data()->GetSharedData().host_mapping_rules;
 }
 
-UserScriptMaster& BrowserContext::UserScriptManager() {
-  DCHECK(CalledOnValidThread());
-  return GetSharedData().user_script_master;
-}
-
 content::ResourceContext* BrowserContext::GetResourceContext() {
   DCHECK(CalledOnValidThread());
   return io_data()->GetResourceContext();
@@ -1074,4 +886,33 @@ scoped_refptr<net::CookieStore> BrowserContext::GetCookieStore() {
   return io_data()->cookie_store_;
 }
 
+
+TemporarySavedPermissionContext*
+BrowserContext::GetTemporarySavedPermissionContext() const {
+  DCHECK(CalledOnValidThread());
+  return io_data()->GetTemporarySavedPermissionContext();
+}
+
+bool BrowserContext::GetDoNotTrack() const {
+  DCHECK(CalledOnValidThread());
+  return io_data()->GetSharedData().do_not_track;
+}
+
+void BrowserContext::SetDoNotTrack(bool dnt) {
+  DCHECK(CalledOnValidThread());
+
+  BrowserContextSharedIOData& data = io_data()->GetSharedData();
+  base::AutoLock lock(data.lock);
+  data.do_not_track = dnt;
+
+  FOR_EACH_OBSERVER(BrowserContextObserver,
+                    GetOriginalContext()->observers_,
+                    NotifyDoNotTrackChanged());
+  if (HasOffTheRecordContext()) {
+    FOR_EACH_OBSERVER(BrowserContextObserver,
+                      GetOffTheRecordContext()->observers_,
+                      NotifyDoNotTrackChanged());
+  }
+}
+
 } // namespace oxide
