@@ -20,9 +20,8 @@
 
 #include <string>
 
-#include "base/basictypes.h"
-#include "base/compiler_specific.h"
 #include "base/files/file_path.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/observer_list.h"
@@ -42,6 +41,7 @@ namespace net {
 class CookieMonster;
 class FtpNetworkLayer;
 class HostMappingRules;
+class HttpNetworkSession;
 class HttpServerProperties;
 class HttpUserAgentSettings;
 class SSLConfigService;
@@ -52,6 +52,9 @@ class TransportSecurityState;
 
 namespace oxide {
 
+class BrowserContext;
+typedef uintptr_t BrowserContextID;
+
 class BrowserContextDelegate;
 class BrowserContextImpl;
 class BrowserContextObserver;
@@ -61,9 +64,10 @@ class GeolocationPermissionContext;
 class PermissionManager;
 class ResourceContext;
 class SSLHostStateDelegate;
+class TemporarySavedPermissionContext;
 class URLRequestContext;
 class URLRequestContextGetter;
-class UserScriptMaster;
+class UserAgentSettingsIOData;
 
 class BrowserContextIOData {
  public:
@@ -83,8 +87,7 @@ class BrowserContextIOData {
   base::FilePath GetCachePath() const;
   int GetMaxCacheSizeHint() const;
 
-  std::string GetAcceptLangs() const;
-  std::string GetUserAgent() const;
+  bool GetDoNotTrack() const;
 
   virtual bool IsOffTheRecord() const = 0;
 
@@ -98,6 +101,12 @@ class BrowserContextIOData {
                         const GURL& first_party_url,
                         bool write);
 
+  // XXX: This will be going away
+  // (see the comment in oxide_temporary_saved_permission_context.h)
+  TemporarySavedPermissionContext* GetTemporarySavedPermissionContext() const;
+
+  UserAgentSettingsIOData* GetUserAgentSettings() const;
+
  protected:
   friend class BrowserContextImpl; // For GetSharedData()
 
@@ -120,40 +129,47 @@ class BrowserContextIOData {
   scoped_ptr<net::TransportSecurityState> transport_security_state_;
   scoped_ptr<net::TransportSecurityPersister> transport_security_persister_;
 
+  scoped_ptr<net::HttpNetworkSession> http_network_session_;
+
   scoped_ptr<URLRequestContext> main_request_context_;
   scoped_ptr<ResourceContext> resource_context_;
   scoped_refptr<net::CookieStore> cookie_store_;
 
   scoped_ptr<net::HostMappingRules> host_mapping_rules_;
+
+  scoped_ptr<TemporarySavedPermissionContext>
+      temporary_saved_permission_context_;
 };
 
-class BrowserContext : public content::BrowserContext,
-                       public base::NonThreadSafe {
+class BrowserContext;
+
+struct BrowserContextTraits {
+  static void Destruct(const BrowserContext* x);
+};
+
+// This class holds the context needed for a browsing session. It lives on
+// and must only be accessed on the UI thread - note that it uses a thread-safe
+// refcount only so that we can override the delete behaviour
+class BrowserContext
+    : public content::BrowserContext,
+      public base::RefCountedThreadSafe<BrowserContext, BrowserContextTraits>,
+      public base::NonThreadSafe {
  public:
 
   struct Params {
     Params(const base::FilePath& path,
            const base::FilePath& cache_path,
            int max_cache_size_hint,
-           content::CookieStoreConfig::SessionCookieMode session_cookie_mode,
-           bool devtools_enabled,
-           int devtools_port,
-           const std::string& devtools_ip)
-        : path(path),
-          cache_path(cache_path),
-          max_cache_size_hint(max_cache_size_hint),
-          session_cookie_mode(session_cookie_mode),
-          devtools_enabled(devtools_enabled),
-          devtools_port(devtools_port),
-          devtools_ip(devtools_ip) {}
+           content::CookieStoreConfig::SessionCookieMode session_cookie_mode)
+           : path(path),
+             cache_path(cache_path),
+             max_cache_size_hint(max_cache_size_hint),
+             session_cookie_mode(session_cookie_mode) {}
 
     base::FilePath path;
     base::FilePath cache_path;
     int max_cache_size_hint;
     content::CookieStoreConfig::SessionCookieMode session_cookie_mode;
-    bool devtools_enabled;
-    int devtools_port;
-    std::string devtools_ip;
     std::vector<std::string> host_mapping_rules;
   };
 
@@ -164,13 +180,13 @@ class BrowserContext : public content::BrowserContext,
 
   static scoped_refptr<BrowserContext> Create(const Params& params);
 
+  typedef base::Callback<void(BrowserContext*)> BrowserContextCallback;
+  static void ForEach(const BrowserContextCallback& callback);
+
   // Aborts if there are any live contexts
   static void AssertNoContextsExist();
 
-  // We don't use base::RefCounted here because BrowserContext always
-  // comes in pairs, and we want a shared reference count between each half
-  void AddRef() const;
-  void Release() const;
+  BrowserContextID GetID() const;
 
   net::URLRequestContextGetter* CreateRequestContext(
       content::ProtocolHandlerMap* protocol_handlers,
@@ -179,26 +195,18 @@ class BrowserContext : public content::BrowserContext,
   BrowserContextDelegate* GetDelegate() const;
   void SetDelegate(BrowserContextDelegate* delegate);
 
-  virtual BrowserContext* GetOffTheRecordContext() = 0;
-  virtual BrowserContext* GetOriginalContext() = 0;
+  virtual scoped_refptr<BrowserContext> GetOffTheRecordContext() = 0;
+  virtual BrowserContext* GetOriginalContext() const = 0;
+  virtual bool HasOffTheRecordContext() const = 0;
 
-  bool IsOffTheRecord() const final;
+  bool IsOffTheRecord() const override; // from content::BrowserContext
 
   bool IsSameContext(BrowserContext* other) const;
 
-  base::FilePath GetPath() const final;
+  base::FilePath GetPath() const override; // from content::BrowserContext
   base::FilePath GetCachePath() const;
   int GetMaxCacheSizeHint() const;
 
-  std::string GetAcceptLangs() const;
-  void SetAcceptLangs(const std::string& langs);
-
-  std::string GetProduct() const;
-  void SetProduct(const std::string& product);
-
-  std::string GetUserAgent() const;
-  void SetUserAgent(const std::string& user_agent);
-
   net::StaticCookiePolicy::Type GetCookiePolicy() const;
   void SetCookiePolicy(net::StaticCookiePolicy::Type policy);
 
@@ -207,69 +215,62 @@ class BrowserContext : public content::BrowserContext,
   bool IsPopupBlockerEnabled() const;
   void SetIsPopupBlockerEnabled(bool enabled);
 
-  bool GetDevtoolsEnabled() const;
-  int GetDevtoolsPort() const;
-  std::string GetDevtoolsBindIp() const;
-
   const std::vector<std::string>& GetHostMappingRules() const;
 
-  UserScriptMaster& UserScriptManager();
+  bool GetDoNotTrack() const;
+  void SetDoNotTrack(bool dnt);
 
-  content::ResourceContext* GetResourceContext() final;
+  // from content::BrowserContext
+  content::ResourceContext* GetResourceContext() override;
 
   scoped_refptr<net::CookieStore> GetCookieStore();
 
+  // XXX: This will be going away
+  // (see the comment in oxide_temporary_saved_permission_context.h)
+  TemporarySavedPermissionContext* GetTemporarySavedPermissionContext() const;
+
  protected:
+  friend class BrowserContextDestroyer; // for destructor
+
   BrowserContext(BrowserContextIOData* io_data);
   virtual ~BrowserContext();
 
   BrowserContextIOData* io_data() const { return io_data_; }
 
-  virtual BrowserContextSharedData& GetSharedData() = 0;
-  virtual const BrowserContextSharedData& GetSharedData() const = 0;
-
  private:
   friend class BrowserContextObserver; // for {Add,Remove}Observer
 
+  // content::BrowserContext implementation
   scoped_ptr<content::ZoomLevelDelegate> CreateZoomLevelDelegate(
-      const base::FilePath& partition_path) final;
-
-  net::URLRequestContextGetter* GetRequestContext() final;
+      const base::FilePath& partition_path) override;
+  net::URLRequestContextGetter* GetRequestContext() override;
   net::URLRequestContextGetter* GetRequestContextForRenderProcess(
-      int renderer_child_id) final;
-
-  net::URLRequestContextGetter* GetMediaRequestContext() final;
+      int renderer_child_id) override;
+  net::URLRequestContextGetter* GetMediaRequestContext() override;
   net::URLRequestContextGetter* GetMediaRequestContextForRenderProcess(
-      int renderer_child_id) final;
-
-  net::URLRequestContextGetter*
-      GetMediaRequestContextForStoragePartition(
-          const base::FilePath& partition_path,
-          bool in_memory) final;
-
-  content::DownloadManagerDelegate* GetDownloadManagerDelegate() final;
-
-  content::BrowserPluginGuestManager* GetGuestManager() final;
-  storage::SpecialStoragePolicy* GetSpecialStoragePolicy() final;
-  content::PushMessagingService* GetPushMessagingService() final;
-  content::SSLHostStateDelegate* GetSSLHostStateDelegate() final;
-  content::PermissionManager* GetPermissionManager() final;
+      int renderer_child_id) override;
+  net::URLRequestContextGetter* GetMediaRequestContextForStoragePartition(
+      const base::FilePath& partition_path,
+      bool in_memory) override;
+  content::DownloadManagerDelegate* GetDownloadManagerDelegate() override;
+  content::BrowserPluginGuestManager* GetGuestManager() override;
+  storage::SpecialStoragePolicy* GetSpecialStoragePolicy() override;
+  content::PushMessagingService* GetPushMessagingService() override;
+  content::SSLHostStateDelegate* GetSSLHostStateDelegate() override;
+  content::PermissionManager* GetPermissionManager() override;
+  content::BackgroundSyncController* GetBackgroundSyncController() override;
 
   void AddObserver(BrowserContextObserver* observer);
   void RemoveObserver(BrowserContextObserver* observer);
 
-  static void Delete(const BrowserContext* context);
-
-  virtual bool HasOffTheRecordContext() const = 0;
-
   BrowserContextIOData* io_data_;
   scoped_refptr<URLRequestContextGetter> main_request_context_getter_;
-  ObserverList<BrowserContextObserver> observers_;
+  base::ObserverList<BrowserContextObserver> observers_;
 
   scoped_ptr<SSLHostStateDelegate> ssl_host_state_delegate_;
   scoped_ptr<PermissionManager> permission_manager_;
 
-  DISALLOW_IMPLICIT_CONSTRUCTORS(BrowserContext);
+  DISALLOW_COPY_AND_ASSIGN(BrowserContext);
 };
 
 } // namespace oxide
