@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
 #include <dlfcn.h>
 #include <signal.h>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/at_exit.h"
@@ -47,14 +48,15 @@
 #include "content/public/common/main_function_params.h"
 #include "content/renderer/in_process_renderer_thread.h"
 #include "content/utility/in_process_utility_thread.h"
-#if defined(USE_NSS)
+#if defined(USE_NSS_CERTS)
 #include "crypto/nss_util.h"
 #endif
 #ifdef V8_USE_EXTERNAL_STARTUP_DATA
-#include "gin/public/isolate_holder.h"
+#include "gin/v8_initializer.h"
 #endif
 #include "gpu/command_buffer/service/gpu_switches.h"
 #include "ipc/ipc_descriptors.h"
+#include "media/base/media_switches.h"
 #include "ui/base/ui_base_paths.h"
 #include "ui/base/ui_base_switches.h"
 #include "ui/gl/gl_switches.h"
@@ -64,9 +66,10 @@
 #include "shared/app/oxide_platform_delegate.h"
 #include "shared/common/oxide_constants.h"
 #include "shared/common/oxide_content_client.h"
+#include "shared/common/oxide_form_factor.h"
 
 #include "oxide_browser_context.h"
-#include "oxide_form_factor.h"
+#include "oxide_form_factor_detection.h"
 #include "oxide_message_pump.h"
 #include "oxide_web_contents_unloader.h"
 
@@ -97,12 +100,7 @@ class BrowserProcessMainImpl : public BrowserProcessMain {
   BrowserProcessMainImpl();
   virtual ~BrowserProcessMainImpl();
 
-  void Start(scoped_ptr<PlatformDelegate> delegate,
-#if defined(USE_NSS)
-             const base::FilePath& nss_db_path,
-#endif
-             gfx::GLImplementation gl_impl,
-             ProcessModel process_model) final;
+  void Start(StartParams params) final;
   void Shutdown() final;
 
   bool IsRunning() const final {
@@ -234,15 +232,18 @@ void InitializeCommandLine(const base::FilePath& subprocess_path,
   // We don't want the GPU shader cache (see https://launchpad.net/bugs/1430478)
   command_line->AppendSwitch(switches::kDisableGpuShaderDiskCache);
 
+  // We can't start the GPU thread before
+  // BrowserMainParts::PreMainMessageLoopRun, as we set the share context there.
+  // We can't set it earlier, because that depends on GpuDataManager which is
+  // initialized in BrowserMainLoop::PreCreateThreads, after the call in to
+  // BrowserMainParts::PreCreateThreads.
+  // Without this flag, the GPU thread gets started in
+  // BrowserMainLoop::BrowserThreadsStarted
+  command_line->AppendSwitch(switches::kDisableGpuEarlyInit);
+
   command_line->AppendSwitch(switches::kUIPrioritizeInGpuProcess);
   command_line->AppendSwitch(switches::kEnableSmoothScrolling);
 
-  // Remove this when we implement a selection API (see bug #1324292)
-  command_line->AppendSwitch(switches::kDisableTouchEditing);
-
-  // See https://launchpad.net/bugs/1442413
-  command_line->AppendSwitch("disable-mojo-channel");
-
   if (gl_impl == gfx::kGLImplementationNone ||
       IsEnvironmentOptionEnabled("DISABLE_GPU")) {
     command_line->AppendSwitch(switches::kDisableGpu);
@@ -305,9 +306,6 @@ void InitializeCommandLine(const base::FilePath& subprocess_path,
   if (IsEnvironmentOptionEnabled("ALLOW_SANDBOX_DEBUGGING")) {
     command_line->AppendSwitch(switches::kAllowSandboxDebugging);
   }
-  if (IsEnvironmentOptionEnabled("EXPERIMENTAL_ENABLE_GTALK_PLUGIN")) {
-    command_line->AppendSwitch(switches::kEnableGoogleTalkPlugin);
-  }
 
   if (IsEnvironmentOptionEnabled("ENABLE_MEDIA_HUB_AUDIO")) {
     command_line->AppendSwitch(switches::kEnableMediaHubAudio);
@@ -321,39 +319,24 @@ void InitializeCommandLine(const base::FilePath& subprocess_path,
       command_line->AppendSwitch(switches::kEnableMediaHubAudio);
     }
   }
+
+  if (IsEnvironmentOptionEnabled("TESTING_MODE")) {
+    command_line->AppendSwitch(switches::kUseFakeDeviceForMediaStream);
+  }
 }
 
 void AddFormFactorSpecificCommandLineArguments() {
+  if (GetFormFactorHint() == FORM_FACTOR_DESKTOP) {
+    return;
+  }
+
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
-
-  FormFactor form_factor = GetFormFactorHint();
-
-  if (form_factor != FORM_FACTOR_DESKTOP) {
-    command_line->AppendSwitch(switches::kEnableViewport);
-    command_line->AppendSwitch(switches::kEnableViewportMeta);
-    command_line->AppendSwitch(switches::kMainFrameResizesAreOrientationChanges);
-    command_line->AppendSwitch(switches::kEnablePinch);
-    // Note, overlay scrollbars do not work properly on desktop yet
-    // see https://launchpad.net/bugs/1426567
-    command_line->AppendSwitch(switches::kEnableOverlayScrollbar);
-    command_line->AppendSwitch(switches::kLimitMaxDecodedImageBytes);
-  }
-
-  const char* form_factor_string = nullptr;
-  switch (form_factor) {
-    case FORM_FACTOR_DESKTOP:
-      form_factor_string = switches::kFormFactorDesktop;
-      break;
-    case FORM_FACTOR_TABLET:
-      form_factor_string = switches::kFormFactorTablet;
-      break;
-    case FORM_FACTOR_PHONE:
-      form_factor_string = switches::kFormFactorPhone;
-      break;
-    default:
-      NOTREACHED();
-  }
-  command_line->AppendSwitchASCII(switches::kFormFactor, form_factor_string);
+  command_line->AppendSwitch(switches::kEnableViewport);
+  command_line->AppendSwitch(switches::kMainFrameResizesAreOrientationChanges);
+  command_line->AppendSwitch(switches::kEnablePinch);
+  // Note, overlay scrollbars do not work properly on desktop yet
+  // see https://launchpad.net/bugs/1426567
+  command_line->AppendSwitch(switches::kEnableOverlayScrollbar);
 }
 
 bool IsUnsupportedProcessModel(ProcessModel process_model) {
@@ -368,8 +351,39 @@ bool IsUnsupportedProcessModel(ProcessModel process_model) {
   }
 }
 
+const char* GetFormFactorHintCommandLine(FormFactor form_factor) {
+  switch (form_factor) {
+    case FORM_FACTOR_DESKTOP:
+      return switches::kFormFactorDesktop;
+    case FORM_FACTOR_TABLET:
+      return switches::kFormFactorTablet;
+    case FORM_FACTOR_PHONE:
+      return switches::kFormFactorPhone;
+  }
+
+  NOTREACHED();
+  return nullptr;
 }
 
+}
+
+BrowserProcessMain::StartParams::StartParams(
+    scoped_ptr<PlatformDelegate> delegate)
+    : delegate(std::move(delegate)),
+      gl_implementation(gfx::kGLImplementationNone),
+      process_model(PROCESS_MODEL_MULTI_PROCESS) {}
+
+BrowserProcessMain::StartParams::~StartParams() {}
+
+BrowserProcessMain::StartParams::StartParams(StartParams&& other)
+    : delegate(std::move(other.delegate)),
+#if defined(USE_NSS_CERTS)
+      nss_db_path(std::move(other.nss_db_path)),
+#endif
+      gl_implementation(std::move(other.gl_implementation)),
+      process_model(std::move(other.process_model)),
+      primary_screen_size_dip(std::move(other.primary_screen_size_dip)) {}
+
 BrowserProcessMainImpl::BrowserProcessMainImpl()
     : state_(STATE_NOT_STARTED),
       process_model_(PROCESS_MODEL_MULTI_PROCESS) {}
@@ -379,25 +393,20 @@ BrowserProcessMainImpl::~BrowserProcessMainImpl() {
       "BrowserProcessMain::Shutdown() should be called before process exit";
 }
 
-void BrowserProcessMainImpl::Start(scoped_ptr<PlatformDelegate> delegate,
-#if defined(USE_NSS)
-                                   const base::FilePath& nss_db_path,
-#endif
-                                   gfx::GLImplementation gl_impl,
-                                   ProcessModel process_model) {
+void BrowserProcessMainImpl::Start(StartParams params) {
   CHECK_EQ(state_, STATE_NOT_STARTED) <<
       "Browser components cannot be started more than once";
-  CHECK(delegate) << "No PlatformDelegate provided";
+  CHECK(params.delegate) << "No PlatformDelegate provided";
 
-  platform_delegate_ = delegate.Pass();
+  platform_delegate_ = std::move(params.delegate);
   main_delegate_.reset(new ContentMainDelegate(platform_delegate_.get()));
 
-  if (IsUnsupportedProcessModel(process_model)) {
+  if (IsUnsupportedProcessModel(params.process_model)) {
     LOG(WARNING) <<
         "Using an unsupported process model. This may affect stability and "
         "security. Use at your own risk!";
   }
-  process_model_ = process_model;
+  process_model_ = params.process_model;
 
   state_ = STATE_STARTED;
 
@@ -410,7 +419,15 @@ void BrowserProcessMainImpl::Start(scoped_ptr<PlatformDelegate> delegate,
   exit_manager_.reset(new base::AtExitManager());
 
   base::FilePath subprocess_exe = GetSubprocessPath();
-  InitializeCommandLine(subprocess_exe, process_model_, gl_impl);
+  InitializeCommandLine(subprocess_exe, process_model_,
+                        params.gl_implementation);
+
+  FormFactor form_factor =
+      DetectFormFactorHint(params.primary_screen_size_dip);
+  base::CommandLine::ForCurrentProcess()
+      ->AppendSwitchASCII(switches::kFormFactor,
+                          GetFormFactorHintCommandLine(form_factor));
+  AddFormFactorSpecificCommandLineArguments();
 
   // We need to override FILE_EXE in the browser process to the path of the
   // renderer, as various bits of Chrome use this to find other resources
@@ -427,13 +444,11 @@ void BrowserProcessMainImpl::Start(scoped_ptr<PlatformDelegate> delegate,
       base::CommandLine::ForCurrentProcess()->HasSwitch(
         switches::kSingleProcess));
 
-  AddFormFactorSpecificCommandLineArguments();
-
-#if defined(USE_NSS)
-  if (!nss_db_path.empty()) {
+#if defined(USE_NSS_CERTS)
+  if (!params.nss_db_path.empty()) {
     // Used for testing
     PathService::OverrideAndCreateIfNeeded(crypto::DIR_NSSDB,
-                                           nss_db_path,
+                                           params.nss_db_path,
                                            false, true);
   }
   crypto::EarlySetupForNSSInit();
@@ -445,7 +460,8 @@ void BrowserProcessMainImpl::Start(scoped_ptr<PlatformDelegate> delegate,
 
   CHECK(base::i18n::InitializeICU()) << "Failed to initialize ICU";
 #if defined(V8_USE_EXTERNAL_STARTUP_DATA)
-  CHECK(gin::IsolateHolder::LoadV8Snapshot());
+  gin::V8Initializer::LoadV8Snapshot();
+  gin::V8Initializer::LoadV8Natives();
 #endif  // V8_USE_EXTERNAL_STARTUP_DATA
 
   main_delegate_->PreSandboxStartup();
@@ -476,7 +492,7 @@ void BrowserProcessMainImpl::Shutdown() {
   }
   state_ = STATE_SHUTTING_DOWN;
 
-  MessageLoopForUI::current()->Stop();
+  MessagePump::Get()->Stop();
 
   WebContentsUnloader::GetInstance()->WaitForPendingUnloadsToFinish();
 
