@@ -18,6 +18,7 @@
 #include "oxide_content_browser_client.h"
 
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/command_line.h"
@@ -26,6 +27,7 @@
 #include "base/memory/scoped_ptr.h"
 #include "content/public/browser/certificate_request_result_type.h"
 #include "content/public/browser/geolocation_provider.h"
+#include "content/public/browser/location_provider.h"
 #include "content/public/browser/permission_type.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
@@ -34,40 +36,39 @@
 #include "content/public/common/web_preferences.h"
 
 #include "shared/browser/compositor/oxide_compositor_utils.h"
+#include "shared/browser/media/oxide_media_capture_devices_dispatcher.h"
+#include "shared/browser/notifications/oxide_platform_notification_service.h"
+#include "shared/browser/ssl/oxide_certificate_error_dispatcher.h"
 #include "shared/common/oxide_constants.h"
 #include "shared/common/oxide_content_client.h"
-#include "shared/common/oxide_messages.h"
+#include "shared/common/oxide_form_factor.h"
 
 #include "oxide_access_token_store.h"
-#include "oxide_android_properties.h"
 #include "oxide_browser_context.h"
-#include "oxide_browser_context_anchor.h"
 #include "oxide_browser_main_parts.h"
 #include "oxide_browser_platform_integration.h"
 #include "oxide_browser_process_main.h"
-#include "oxide_devtools_manager_delegate.h"
-#include "oxide_form_factor.h"
 #include "oxide_quota_permission_context.h"
+#include "oxide_render_message_filter.h"
 #include "oxide_resource_dispatcher_host_delegate.h"
-#include "oxide_script_message_dispatcher_browser.h"
-#include "oxide_user_agent_override_provider.h"
+#include "oxide_user_agent_settings.h"
 #include "oxide_web_preferences.h"
 #include "oxide_web_view.h"
 #include "oxide_web_view_contents_helper.h"
 
+#if defined(ENABLE_HYBRIS)
+#include "oxide_hybris_utils.h"
+#endif
+
 #if defined(ENABLE_PLUGINS)
 #include "content/public/browser/browser_ppapi_host.h"
 #include "ppapi/host/ppapi_host.h"
 
-#include "oxide_pepper_host_factory_browser.h"
+#include "pepper/oxide_pepper_host_factory_browser.h"
 #endif
 
 namespace oxide {
 
-ContentBrowserClient::ContentBrowserClient() {}
-
-ContentBrowserClient::~ContentBrowserClient() {}
-
 content::BrowserMainParts* ContentBrowserClient::CreateBrowserMainParts(
     const content::MainFunctionParams& parameters) {
   return new BrowserMainParts();
@@ -75,12 +76,7 @@ content::BrowserMainParts* ContentBrowserClient::CreateBrowserMainParts(
 
 void ContentBrowserClient::RenderProcessWillLaunch(
     content::RenderProcessHost* host) {
-  BrowserContextAnchor::GetInstance()->RenderProcessWillLaunch(host);
-
-  host->Send(new OxideMsg_SetUserAgent(
-      BrowserContext::FromContent(host->GetBrowserContext())->GetUserAgent()));
-  host->AddFilter(new ScriptMessageDispatcherBrowser(host));
-  host->AddFilter(new UserAgentOverrideProvider(host));
+  host->AddFilter(new RenderMessageFilter(host));
 }
 
 net::URLRequestContextGetter* ContentBrowserClient::CreateRequestContext(
@@ -89,7 +85,7 @@ net::URLRequestContextGetter* ContentBrowserClient::CreateRequestContext(
     content::URLRequestInterceptorScopedVector request_interceptors) {
   return BrowserContext::FromContent(
       browser_context)->CreateRequestContext(protocol_handlers,
-                                             request_interceptors.Pass());
+                                             std::move(request_interceptors));
 }
 
 net::URLRequestContextGetter*
@@ -107,7 +103,7 @@ ContentBrowserClient::CreateRequestContextForStoragePartition(
 
 std::string ContentBrowserClient::GetAcceptLangs(
     content::BrowserContext* browser_context) {
-  return BrowserContext::FromContent(browser_context)->GetAcceptLangs();
+  return UserAgentSettings::Get(browser_context)->GetAcceptLangs();
 }
 
 void ContentBrowserClient::AppendExtraCommandLineSwitches(
@@ -115,10 +111,8 @@ void ContentBrowserClient::AppendExtraCommandLineSwitches(
     int child_process_id) {
   // This can be called on the UI or IO thread
   static const char* const kSwitchNames[] = {
-    switches::kEnableGoogleTalkPlugin,
-    switches::kFormFactor,
-    switches::kLimitMaxDecodedImageBytes,
     switches::kEnableMediaHubAudio,
+    switches::kFormFactor,
     switches::kMediaHubFixedSessionDomains
   };
   command_line->CopySwitchesFrom(*base::CommandLine::ForCurrentProcess(),
@@ -158,7 +152,7 @@ bool ContentBrowserClient::AllowSetCookie(const GURL& url,
                                           content::ResourceContext* context,
                                           int render_process_id,
                                           int render_frame_id,
-                                          net::CookieOptions* options) {
+                                          const net::CookieOptions& options) {
   return BrowserContextIOData::FromResourceContext(
       context)->CanAccessCookies(url, first_party, true);
 }
@@ -168,8 +162,8 @@ content::QuotaPermissionContext* ContentBrowserClient::CreateQuotaPermissionCont
 }
 
 void ContentBrowserClient::AllowCertificateError(
-    int render_process_id,
-    int render_frame_id,
+    content::WebContents* contents,
+    bool is_main_frame,
     int cert_error,
     const net::SSLInfo& ssl_info,
     const GURL& request_url,
@@ -179,25 +173,25 @@ void ContentBrowserClient::AllowCertificateError(
     bool expired_previous_decision,
     const base::Callback<void(bool)>& callback,
     content::CertificateRequestResultType* result) {
-  DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
+  CertificateErrorDispatcher::AllowCertificateError(contents,
+                                                    is_main_frame,
+                                                    cert_error,
+                                                    ssl_info,
+                                                    request_url,
+                                                    resource_type,
+                                                    overridable,
+                                                    strict_enforcement,
+                                                    callback,
+                                                    result);
+}
 
-  content::RenderFrameHost* rfh = content::RenderFrameHost::FromID(
-      render_process_id, render_frame_id);
-  if (!rfh) {
-    *result = content::CERTIFICATE_REQUEST_RESULT_TYPE_CANCEL;
-    return;
-  }
+content::MediaObserver* ContentBrowserClient::GetMediaObserver() {
+  return MediaCaptureDevicesDispatcher::GetInstance();
+}
 
-  WebView* webview = WebView::FromRenderFrameHost(rfh);
-  if (!webview) {
-    *result = content::CERTIFICATE_REQUEST_RESULT_TYPE_CANCEL;
-    return;
-  }
-
-  webview->AllowCertificateError(rfh, cert_error, ssl_info, request_url,
-                                 resource_type, overridable,
-                                 strict_enforcement, callback,
-                                 result);
+content::PlatformNotificationService*
+ContentBrowserClient::GetPlatformNotificationService() {
+  return PlatformNotificationService::GetInstance();
 }
 
 bool ContentBrowserClient::CanCreateWindow(
@@ -213,7 +207,8 @@ bool ContentBrowserClient::CanCreateWindow(
     bool opener_suppressed,
     content::ResourceContext* context,
     int render_process_id,
-    int opener_id,
+    int opener_render_view_id,
+    int opener_render_frame_id,
     bool* no_javascript_access) {
   *no_javascript_access = false;
 
@@ -241,7 +236,13 @@ void ContentBrowserClient::OverrideWebkitPrefs(
   WebViewContentsHelper* contents_helper =
       WebViewContentsHelper::FromRenderViewHost(render_view_host);
 
-  WebPreferences* web_prefs = contents_helper->GetWebPreferences();
+  WebPreferences* web_prefs = nullptr;
+  if (contents_helper) {
+    // If RVH is for an InterstitialPage, we can't map to WebContents
+    // XXX: If we ever expose transient pages in the public API, we should find
+    //  a way around this, so that transient pages get the same preferences
+    web_prefs = contents_helper->GetWebPreferences();
+  }
   if (!web_prefs) {
     web_prefs = WebPreferences::GetFallback();
   }
@@ -253,13 +254,16 @@ void ContentBrowserClient::OverrideWebkitPrefs(
   prefs->device_supports_touch = platform_integration_->IsTouchSupported();
 
   prefs->javascript_can_open_windows_automatically =
-      !contents_helper->GetBrowserContext()->IsPopupBlockerEnabled();
+      !BrowserContext::FromContent(
+        render_view_host->GetProcess()->GetBrowserContext())
+        ->IsPopupBlockerEnabled();
 
   FormFactor form_factor = GetFormFactorHint();
   if (form_factor == FORM_FACTOR_TABLET || form_factor == FORM_FACTOR_PHONE) {
     prefs->shrinks_standalone_images_to_fit = false;
     prefs->default_minimum_page_scale_factor = 0.25f;
     prefs->default_maximum_page_scale_factor = 5.f;
+    prefs->viewport_meta_enabled = true;
   }
 
   prefs->supports_multiple_windows = false;
@@ -271,12 +275,7 @@ void ContentBrowserClient::OverrideWebkitPrefs(
 
 content::LocationProvider*
 ContentBrowserClient::OverrideSystemLocationProvider() {
-  return platform_integration_->CreateLocationProvider();
-}
-
-content::DevToolsManagerDelegate*
-ContentBrowserClient::GetDevToolsManagerDelegate() {
-  return new DevToolsManagerDelegate();
+  return platform_integration_->CreateLocationProvider().release();
 }
 
 void ContentBrowserClient::DidCreatePpapiPlugin(content::BrowserPpapiHost* host) {
@@ -286,27 +285,33 @@ void ContentBrowserClient::DidCreatePpapiPlugin(content::BrowserPpapiHost* host)
 #endif
 }
 
-void ContentBrowserClient::SetPlatformIntegration(
-    BrowserPlatformIntegration* integration) {
-  CHECK(integration && !platform_integration_);
-  platform_integration_.reset(integration);
-}
-
 gpu::GpuControlList::OsType
 ContentBrowserClient::GetOsTypeOverrideForGpuDataManager(
     std::string* os_version) {
-  if (!AndroidProperties::GetInstance()->Available()) {
+#if defined(ENABLE_HYBRIS)
+  if (!HybrisUtils::IsUsingAndroidEGL()) {
     // Use the platform defaults in this case
     return gpu::GpuControlList::kOsAny;
   }
 
-  *os_version = AndroidProperties::GetInstance()->GetOSVersion();
+  *os_version = HybrisUtils::GetDeviceProperties().os_version;
   return gpu::GpuControlList::kOsAndroid;
+#else
+  return gpu::GpuControlList::kOsAny;
+#endif
 }
 
 std::string
 ContentBrowserClient::GetApplicationLocale() {
-  return platform_integration_->GetApplicationLocale();
+  return application_locale_;
 }
 
+ContentBrowserClient::ContentBrowserClient(
+    const std::string& application_locale,
+    BrowserPlatformIntegration* integration)
+    : application_locale_(application_locale),
+      platform_integration_(integration) {}
+
+ContentBrowserClient::~ContentBrowserClient() {}
+
 } // namespace oxide
