@@ -24,7 +24,12 @@
 //   found in the LICENSE file.
 //
 //  gpu/config/gpu_info_collector_linux.cc
-//   Copyright 2014 The Chromium Authors. All rights reserved.
+//   Copyright (c) 2014 The Chromium Authors. All rights reserved.
+//   Use of this source code is governed by a BSD-style license that can be
+//   found in the LICENSE file.
+//
+//  gpu/config/gpu_info_collector.cc
+//   Copyright (c) 2012 The Chromium Authors. All rights reserved.
 //   Use of this source code is governed by a BSD-style license that can be
 //   found in the LICENSE file.
 
@@ -46,58 +51,27 @@
 #include "third_party/libXNVCtrl/NVCtrl.h"
 #include "third_party/libXNVCtrl/NVCtrlLib.h"
 #include "ui/gfx/x/x11_types.h"
-#include "ui/gl/gl_bindings.h"
-#include "ui/gl/gl_context.h"
 #include "ui/gl/gl_implementation.h"
 
 #include "library_loaders/libpci.h"
 
 #include "shared/port/gpu_config/gpu_info_collector_oxide_linux.h"
 
-#include "oxide_android_properties.h"
+#if defined(ENABLE_HYBRIS)
+#include "base/native_library.h"
+#include "base/strings/string_number_conversions.h"
+#include "base/strings/stringprintf.h"
+#include "ui/gl/egl_util.h"
+#include "ui/gl/gl_bindings.h"
+
+#include "oxide_hybris_utils.h"
+#endif
 
 namespace oxide {
 
 namespace {
 
-class ScopedRestoreNonOwnedEGLContext {
- public:
-  ScopedRestoreNonOwnedEGLContext();
-  ~ScopedRestoreNonOwnedEGLContext();
-
- private:
-  EGLContext context_;
-  EGLDisplay display_;
-  EGLSurface draw_surface_;
-  EGLSurface read_surface_;
-};
-
-ScopedRestoreNonOwnedEGLContext::ScopedRestoreNonOwnedEGLContext()
-  : context_(EGL_NO_CONTEXT),
-    display_(EGL_NO_DISPLAY),
-    draw_surface_(EGL_NO_SURFACE),
-    read_surface_(EGL_NO_SURFACE) {
-  // This should only used to restore a context that is not created or owned by
-  // Chromium native code, but created by Android system itself.
-  DCHECK(!gfx::GLContext::GetCurrent());
-
-  context_ = eglGetCurrentContext();
-  display_ = eglGetCurrentDisplay();
-  draw_surface_ = eglGetCurrentSurface(EGL_DRAW);
-  read_surface_ = eglGetCurrentSurface(EGL_READ);
-}
-
-ScopedRestoreNonOwnedEGLContext::~ScopedRestoreNonOwnedEGLContext() {
-  if (context_ == EGL_NO_CONTEXT || display_ == EGL_NO_DISPLAY ||
-      draw_surface_ == EGL_NO_SURFACE || read_surface_ == EGL_NO_SURFACE) {
-    return;
-  }
-
-  if (!eglMakeCurrent(display_, draw_surface_, read_surface_, context_)) {
-    LOG(WARNING) << "Failed to restore EGL context";
-  }
-}
-
+#if defined(ENABLE_HYBRIS)
 std::string GetDriverVersionFromString(const std::string& version_string) {
   // Extract driver version from the second number in a string like:
   // "OpenGL ES 2.0 V@6.0 AU@ (CL@2946718)"
@@ -121,24 +95,228 @@ std::string GetDriverVersionFromString(const std::string& version_string) {
   } else {
     sub_string = version_string.substr(begin);
   }
-  std::vector<std::string> pieces;
-  base::SplitString(sub_string, '.', &pieces);
+  std::vector<std::string> pieces = base::SplitString(
+      sub_string, ".", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
   if (pieces.size() < 2) {
     return "0";
   }
   return pieces[0] + "." + pieces[1];
 }
 
+std::pair<std::string, size_t> GetVersionFromString(
+    const std::string& version_string,
+    size_t begin = 0) {
+  begin = version_string.find_first_of("0123456789", begin);
+  if (begin == std::string::npos) {
+    return std::make_pair("", std::string::npos);
+  }
+
+  size_t end = version_string.find_first_not_of("01234567890.", begin);
+  std::string sub_string;
+  if (end != std::string::npos) {
+    sub_string = version_string.substr(begin, end - begin);
+  } else {
+    sub_string = version_string.substr(begin);
+  }
+  std::vector<std::string> pieces = base::SplitString(
+      sub_string, ".", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
+  if (pieces.size() >= 2) {
+    return std::make_pair(pieces[0] + "." + pieces[1], end);
+  } else {
+    return std::make_pair("", end);
+  }
+}
+
+gpu::CollectInfoResult CollectDriverInfo(gpu::GPUInfo* gpu_info) {
+  // Go through the process of loading GL libs and initializing an EGL
+  // context so that we can get GL vendor/version/renderer strings.
+  base::NativeLibrary gles_library, egl_library;
+  base::NativeLibraryLoadError error;
+  gles_library =
+      base::LoadNativeLibrary(base::FilePath("libGLESv2.so.2"), &error);
+  if (!gles_library) {
+    LOG(FATAL) << "Failed to load libGLESv2.so.2";
+  }
+
+  egl_library = base::LoadNativeLibrary(base::FilePath("libEGL.so.1"), &error);
+  if (!egl_library) {
+    LOG(FATAL) << "Failed to load libEGL.so.1";
+  }
+
+  typedef void* (*eglGetProcAddressProc)(const char* name);
+
+  auto eglGetProcAddressFn = reinterpret_cast<eglGetProcAddressProc>(
+      base::GetFunctionPointerFromNativeLibrary(egl_library,
+                                                "eglGetProcAddress"));
+  if (!eglGetProcAddressFn) {
+    LOG(FATAL) << "eglGetProcAddress not found.";
+  }
+
+  auto get_func = [eglGetProcAddressFn, gles_library, egl_library](
+      const char* name) {
+    void *proc;
+    proc = base::GetFunctionPointerFromNativeLibrary(egl_library, name);
+    if (proc) {
+      return proc;
+    }
+    proc = base::GetFunctionPointerFromNativeLibrary(gles_library, name);
+    if (proc) {
+      return proc;
+    }
+    proc = eglGetProcAddressFn(name);
+    if (proc) {
+      return proc;
+    }
+    LOG(FATAL) << "Failed to look up " << name;
+    return (void *)nullptr;
+  };
+
+#define LOOKUP_FUNC(x) auto x##Fn = reinterpret_cast<gfx::x##Proc>(get_func(#x))
+
+  LOOKUP_FUNC(eglGetError);
+  LOOKUP_FUNC(eglQueryString);
+  LOOKUP_FUNC(eglGetCurrentContext);
+  LOOKUP_FUNC(eglGetCurrentDisplay);
+  LOOKUP_FUNC(eglGetCurrentSurface);
+  LOOKUP_FUNC(eglGetDisplay);
+  LOOKUP_FUNC(eglInitialize);
+  LOOKUP_FUNC(eglChooseConfig);
+  LOOKUP_FUNC(eglCreateContext);
+  LOOKUP_FUNC(eglCreatePbufferSurface);
+  LOOKUP_FUNC(eglMakeCurrent);
+  LOOKUP_FUNC(eglDestroySurface);
+  LOOKUP_FUNC(eglDestroyContext);
+
+  LOOKUP_FUNC(glGetString);
+  LOOKUP_FUNC(glGetIntegerv);
+
+#undef LOOKUP_FUNC
+
+  EGLDisplay curr_display = eglGetCurrentDisplayFn();
+  EGLContext curr_context = eglGetCurrentContextFn();
+  EGLSurface curr_draw_surface = eglGetCurrentSurfaceFn(EGL_DRAW);
+  EGLSurface curr_read_surface = eglGetCurrentSurfaceFn(EGL_READ);
+
+  EGLDisplay temp_display = EGL_NO_DISPLAY;
+  EGLContext temp_context = EGL_NO_CONTEXT;
+  EGLSurface temp_surface = EGL_NO_SURFACE;
+
+  const EGLint kConfigAttribs[] = {
+      EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
+      EGL_SURFACE_TYPE, EGL_PBUFFER_BIT,
+      EGL_NONE};
+  const EGLint kContextAttribs[] = {
+      EGL_CONTEXT_OPENGL_RESET_NOTIFICATION_STRATEGY_EXT,
+          EGL_LOSE_CONTEXT_ON_RESET_EXT,
+      EGL_CONTEXT_CLIENT_VERSION, 2,
+      EGL_NONE};
+  const EGLint kSurfaceAttribs[] = {
+      EGL_WIDTH, 1,
+      EGL_HEIGHT, 1,
+      EGL_NONE};
+
+  EGLint major, minor;
+
+  EGLConfig config;
+  EGLint num_configs;
+
+  auto errorstr = [eglGetErrorFn]() {
+    uint32_t err = eglGetErrorFn();
+    return base::StringPrintf("%s (%x)", ui::GetEGLErrorString(err), err);
+  };
+
+  temp_display = eglGetDisplayFn(EGL_DEFAULT_DISPLAY);
+
+  if (temp_display == EGL_NO_DISPLAY) {
+    LOG(FATAL) << "failed to get display. " << errorstr();
+  }
+
+  eglInitializeFn(temp_display, &major, &minor);
+
+  bool egl_create_context_robustness_supported =
+      strstr(reinterpret_cast<const char*>(
+                 eglQueryStringFn(temp_display, EGL_EXTENSIONS)),
+             "EGL_EXT_create_context_robustness") != NULL;
+
+  if (!eglChooseConfigFn(temp_display, kConfigAttribs, &config, 1,
+                         &num_configs)) {
+    LOG(FATAL) << "failed to choose an egl config. " << errorstr();
+  }
+
+  temp_context = eglCreateContextFn(
+      temp_display, config, EGL_NO_CONTEXT,
+      kContextAttribs + (egl_create_context_robustness_supported ? 0 : 2));
+  if (temp_context == EGL_NO_CONTEXT) {
+    LOG(FATAL)
+        << "failed to create a temporary context for fetching driver strings. "
+        << errorstr();
+  }
+
+  temp_surface =
+      eglCreatePbufferSurfaceFn(temp_display, config, kSurfaceAttribs);
+
+  if (temp_surface == EGL_NO_SURFACE) {
+    eglDestroyContextFn(temp_display, temp_context);
+    LOG(FATAL)
+        << "failed to create a pbuffer surface for fetching driver strings. "
+        << errorstr();
+  }
+
+  eglMakeCurrentFn(temp_display, temp_surface, temp_surface, temp_context);
+
+  gpu_info->gl_vendor = reinterpret_cast<const char*>(glGetStringFn(GL_VENDOR));
+  gpu_info->gl_version =
+      reinterpret_cast<const char*>(glGetStringFn(GL_VERSION));
+  gpu_info->gl_renderer =
+      reinterpret_cast<const char*>(glGetStringFn(GL_RENDERER));
+  gpu_info->gl_extensions =
+      reinterpret_cast<const char*>(glGetStringFn(GL_EXTENSIONS));
+
+  GLint max_samples = 0;
+  glGetIntegervFn(GL_MAX_SAMPLES, &max_samples);
+  gpu_info->max_msaa_samples = base::IntToString(max_samples);
+
+  bool supports_robustness =
+      gpu_info->gl_extensions.find("GL_EXT_robustness") != std::string::npos ||
+      gpu_info->gl_extensions.find("GL_KHR_robustness") != std::string::npos ||
+      gpu_info->gl_extensions.find("GL_ARB_robustness") != std::string::npos;
+
+  if (supports_robustness) {
+    glGetIntegervFn(
+        GL_RESET_NOTIFICATION_STRATEGY_ARB,
+        reinterpret_cast<GLint*>(&gpu_info->gl_reset_notification_strategy));
+  }
+
+  std::string glsl_version_string =
+      reinterpret_cast<const char*>(glGetStringFn(GL_SHADING_LANGUAGE_VERSION));
+
+  std::string glsl_version = GetVersionFromString(glsl_version_string).first;
+  gpu_info->pixel_shader_version = glsl_version;
+  gpu_info->vertex_shader_version = glsl_version;
+
+  if (curr_display != EGL_NO_DISPLAY &&
+      curr_context != EGL_NO_CONTEXT) {
+    eglMakeCurrentFn(curr_display, curr_draw_surface, curr_read_surface,
+                     curr_context);
+  } else {
+    eglMakeCurrentFn(temp_display, EGL_NO_SURFACE, EGL_NO_SURFACE,
+                     EGL_NO_CONTEXT);
+  }
+
+  eglDestroySurfaceFn(temp_display, temp_surface);
+  eglDestroyContextFn(temp_display, temp_context);
+
+  return gpu::kCollectInfoSuccess;
+}
+
 gpu::CollectInfoResult CollectBasicGraphicsInfoAndroid(
     gpu::GPUInfo* gpu_info) {
   gpu_info->can_lose_context = false;
 
-  gpu_info->machine_model_name = AndroidProperties::GetInstance()->GetModel();
+  gpu_info->machine_model_name = HybrisUtils::GetDeviceProperties().model;
 
   // Create a short-lived context on the UI thread to collect the GL strings.
-  // Make sure we restore the existing context if there is one.
-  ScopedRestoreNonOwnedEGLContext restore_context;
-  gpu::CollectInfoResult result = CollectGraphicsInfoGL(gpu_info);
+  gpu::CollectInfoResult result = CollectDriverInfo(gpu_info);
   gpu_info->basic_info_state = result;
   gpu_info->context_info_state = result;
 
@@ -159,6 +337,8 @@ gpu::CollectInfoResult CollectDriverInfoGLAndroid(gpu::GPUInfo* gpu_info) {
   return gpu::kCollectInfoSuccess;
 }
 
+#endif
+
 // Scan /etc/ati/amdpcsdb.default for "ReleaseVersion".
 // Return empty string on failing.
 std::string CollectDriverVersionATI() {
@@ -175,7 +355,9 @@ std::string CollectDriverVersionATI() {
   base::StringTokenizer t(contents, "\r\n");
   while (t.GetNext()) {
     std::string line = t.token();
-    if (StartsWithASCII(line, "ReleaseVersion=", true)) {
+    if (base::StartsWith(line,
+                         "ReleaseVersion=",
+                         base::CompareCase::SENSITIVE)) {
       size_t begin = line.find_first_of("0123456789");
       if (begin != std::string::npos) {
         size_t end = line.find_first_not_of("0123456789.", begin);
@@ -226,9 +408,9 @@ bool IsPciSupported() {
   return (base::PathExists(pci_path) || base::PathExists(pcie_path));
 }
 
-const uint32 kVendorIDIntel = 0x8086;
-const uint32 kVendorIDNVidia = 0x10de;
-const uint32 kVendorIDAMD = 0x1002;
+const uint32_t kVendorIDIntel = 0x8086;
+const uint32_t kVendorIDNVidia = 0x10de;
+const uint32_t kVendorIDAMD = 0x1002;
 
 gpu::CollectInfoResult CollectPCIVideoCardInfo(gpu::GPUInfo* gpu_info) {
   DCHECK(gpu_info);
@@ -368,11 +550,12 @@ gpu::CollectInfoResult CollectBasicGraphicsInfoLinux(gpu::GPUInfo* gpu_info) {
 
 gpu::CollectInfoResult CollectDriverInfoGLLinux(gpu::GPUInfo* gpu_info) {
   std::string gl_version = gpu_info->gl_version;
-  if (StartsWithASCII(gl_version, "OpenGL ES", true)) {
+  if (base::StartsWith(gl_version, "OpenGL ES", base::CompareCase::SENSITIVE)) {
     gl_version = gl_version.substr(10);
   }
-  std::vector<std::string> pieces;
-  base::SplitStringAlongWhitespace(gl_version, &pieces);
+  std::vector<std::string> pieces = base::SplitString(
+      gl_version, base::kWhitespaceASCII, base::KEEP_WHITESPACE,
+      base::SPLIT_WANT_ALL);
   // In linux, the gl version string might be in the format of
   //   GLVersion DriverVendor DriverVersion
   if (pieces.size() < 3) {
@@ -401,8 +584,8 @@ class GpuInfoCollectorLinux : public gpu::GpuInfoCollectorOxideLinux {
   GpuInfoCollectorLinux() {}
   ~GpuInfoCollectorLinux() override {}
 
-  gpu::CollectInfoResult CollectGpuID(uint32* vendor_id,
-                                      uint32* device_id) override;
+  gpu::CollectInfoResult CollectGpuID(uint32_t* vendor_id,
+                                      uint32_t* device_id) override;
   gpu::CollectInfoResult CollectContextGraphicsInfo(
       gpu::GPUInfo* gpu_info) override;
   gpu::CollectInfoResult CollectBasicGraphicsInfo(
@@ -413,14 +596,17 @@ class GpuInfoCollectorLinux : public gpu::GpuInfoCollectorOxideLinux {
   DISALLOW_COPY_AND_ASSIGN(GpuInfoCollectorLinux);
 };
 
-gpu::CollectInfoResult GpuInfoCollectorLinux::CollectGpuID(uint32* vendor_id,
-                                                           uint32* device_id) {
+gpu::CollectInfoResult GpuInfoCollectorLinux::CollectGpuID(
+    uint32_t* vendor_id,
+    uint32_t* device_id) {
   *vendor_id = 0;
   *device_id = 0;
 
-  if (AndroidProperties::GetInstance()->Available()) {
+#if defined(ENABLE_HYBRIS)
+  if (HybrisUtils::IsUsingAndroidEGL()) {
     return gpu::kCollectInfoNonFatalFailure;
   }
+#endif
 
   gpu::GPUInfo gpu_info;
   gpu::CollectInfoResult result = CollectPCIVideoCardInfo(&gpu_info);
@@ -434,27 +620,33 @@ gpu::CollectInfoResult GpuInfoCollectorLinux::CollectGpuID(uint32* vendor_id,
 
 gpu::CollectInfoResult GpuInfoCollectorLinux::CollectContextGraphicsInfo(
     gpu::GPUInfo* gpu_info) {
-  if (AndroidProperties::GetInstance()->Available()) {
+#if defined(ENABLE_HYBRIS)
+  if (HybrisUtils::IsUsingAndroidEGL()) {
     return CollectContextGraphicsInfoAndroid(gpu_info);
   }
+#endif
 
   return CollectContextGraphicsInfoLinux(gpu_info);
 }
 
 gpu::CollectInfoResult GpuInfoCollectorLinux::CollectBasicGraphicsInfo(
     gpu::GPUInfo* gpu_info) {
-  if (AndroidProperties::GetInstance()->Available()) {
+#if defined(ENABLE_HYBRIS)
+  if (HybrisUtils::IsUsingAndroidEGL()) {
     return CollectBasicGraphicsInfoAndroid(gpu_info);
   }
+#endif
 
   return CollectBasicGraphicsInfoLinux(gpu_info);
 }
 
 gpu::CollectInfoResult GpuInfoCollectorLinux::CollectDriverInfoGL(
     gpu::GPUInfo* gpu_info) {
-  if (AndroidProperties::GetInstance()->Available()) {
+#if defined(ENABLE_HYBRIS)
+  if (HybrisUtils::IsUsingAndroidEGL()) {
     return CollectDriverInfoGLAndroid(gpu_info);
   }
+#endif
 
   return CollectDriverInfoGLLinux(gpu_info);
 }
