@@ -17,6 +17,8 @@
 
 #include "oxide_io_thread.h"
 
+#include <utility>
+
 #include "base/logging.h"
 #include "base/threading/worker_pool.h"
 #include "content/public/browser/browser_thread.h"
@@ -30,8 +32,8 @@
 #include "net/http/http_server_properties_impl.h"
 #include "net/http/transport_security_state.h"
 #include "net/log/net_log.h"
-#if defined(USE_NSS)
-#include "net/ocsp/nss_ocsp.h"
+#if defined(USE_NSS_CERTS)
+#include "net/cert_net/nss_ocsp.h"
 #endif
 #include "net/proxy/proxy_service.h"
 #include "net/ssl/channel_id_service.h"
@@ -118,6 +120,8 @@ void IOThread::InitSystemRequestContext() {
       content::BrowserThread::IO,
       FROM_HERE,
       base::Bind(&IOThread::InitSystemRequestContextOnIOThread,
+                 // |this| is only deleted after the IO thread has stopped
+                 // processing events
                  base::Unretained(this)));
 }
 
@@ -128,7 +132,7 @@ void IOThread::InitSystemRequestContextOnIOThread() {
   URLRequestContext* context = globals()->system_request_context_.get();
   net::URLRequestContextStorage* storage = context->storage();
 
-#if defined(USE_NSS)
+#if defined(USE_NSS_CERTS)
   net::SetURLRequestContextForNSSHttpIO(context);
 #endif
 
@@ -141,7 +145,8 @@ void IOThread::InitSystemRequestContextOnIOThread() {
       scoped_ptr<net::HttpServerProperties>(
         new net::HttpServerPropertiesImpl()));
   storage->set_cookie_store(new net::CookieMonster(nullptr, nullptr));
-  storage->set_transport_security_state(new net::TransportSecurityState());
+  storage->set_transport_security_state(
+      make_scoped_ptr(new net::TransportSecurityState()));
 
   net::HttpNetworkSession::Params session_params;
   session_params.host_resolver = context->host_resolver();
@@ -158,9 +163,11 @@ void IOThread::InitSystemRequestContextOnIOThread() {
   session_params.net_log = context->net_log();
 
   storage->set_http_transaction_factory(
-      new net::HttpNetworkLayer(new net::HttpNetworkSession(session_params)));
+      make_scoped_ptr(new net::HttpNetworkLayer(
+        new net::HttpNetworkSession(session_params))));
 
-  storage->set_job_factory(new net::URLRequestJobFactoryImpl());
+  storage->set_job_factory(
+      make_scoped_ptr(new net::URLRequestJobFactoryImpl()));
 }
 
 void IOThread::Init() {
@@ -168,12 +175,8 @@ void IOThread::Init() {
 
   BrowserPlatformIntegration::GetInstance()->BrowserThreadInit(
       content::BrowserThread::IO);
-}
 
-void IOThread::InitAsync() {
-  DCHECK_CURRENTLY_ON(content::BrowserThread::IO);
-
-#if defined(USE_NSS)
+#if defined(USE_NSS_CERTS)
   net::SetMessageLoopForNSSHttpIO();
 #endif
 
@@ -183,21 +186,21 @@ void IOThread::InitAsync() {
   globals()->host_resolver_ =
       net::HostResolver::CreateDefaultResolver(
         IOThread::instance()->net_log());
-  globals()->cert_verifier_.reset(net::CertVerifier::CreateDefault());
-  globals()->http_auth_handler_factory_.reset(
+  globals()->cert_verifier_ = net::CertVerifier::CreateDefault();
+  globals()->http_auth_handler_factory_ =
       net::HttpAuthHandlerFactory::CreateDefault(
-        globals()->host_resolver_.get()));
+        globals()->host_resolver_.get());
 
-  net::ProxyConfigService* proxy_config_service =
+  scoped_ptr<net::ProxyConfigService> proxy_config_service =
       net::ProxyService::CreateSystemProxyConfigService(
           content::BrowserThread::GetMessageLoopProxyForThread(
               content::BrowserThread::IO),
           content::BrowserThread::GetMessageLoopProxyForThread(
               content::BrowserThread::FILE));
 
-  globals()->proxy_service_.reset(
+  globals()->proxy_service_ =
       net::ProxyService::CreateUsingSystemProxyResolver(
-        proxy_config_service, 4, IOThread::instance()->net_log()));
+        std::move(proxy_config_service), 4, IOThread::instance()->net_log());
 
   globals()->throttler_manager_.reset(new net::URLRequestThrottlerManager());
 
@@ -205,13 +208,15 @@ void IOThread::InitAsync() {
       content::BrowserThread::UI,
       FROM_HERE,
       base::Bind(&IOThread::InitSystemRequestContext,
+                 // |this| is only deleted after we've stopped processing
+                 // events on the UI thread
                  base::Unretained(this)));
 }
 
 void IOThread::CleanUp() {
   DCHECK_CURRENTLY_ON(content::BrowserThread::IO);
 
-#if defined(USE_NSS)
+#if defined(USE_NSS_CERTS)
   net::ShutdownNSSHttpIO();
   net::SetURLRequestContextForNSSHttpIO(nullptr);
 #endif
