@@ -18,14 +18,15 @@
 #ifndef _OXIDE_SHARED_BROWSER_IO_THREAD_H_
 #define _OXIDE_SHARED_BROWSER_IO_THREAD_H_
 
-#include "base/basictypes.h"
-#include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/threading/non_thread_safe.h"
 #include "content/public/browser/browser_thread_delegate.h"
 
+namespace base {
 template <typename T> struct DefaultSingletonTraits;
+}
 
 namespace net {
 
@@ -93,7 +94,6 @@ class IOThread final : public content::BrowserThreadDelegate {
 
   // Called on the IO thread
   void Init() final;
-  void InitAsync() final;
   void CleanUp() final;
 
   scoped_ptr<net::NetLog> net_log_;
