@@ -18,8 +18,7 @@
 #ifndef _OXIDE_SHARED_BROWSER_NETWORK_DELEGATE_H_
 #define _OXIDE_SHARED_BROWSER_NETWORK_DELEGATE_H_
 
-#include "base/basictypes.h"
-#include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "net/base/network_delegate.h"
 
 namespace oxide {
@@ -65,7 +64,11 @@ class NetworkDelegate final : public net::NetworkDelegate {
 
   void OnResponseStarted(net::URLRequest* request) final;
 
-  void OnRawBytesRead(const net::URLRequest& request, int bytes_read) final;
+  void OnNetworkBytesReceived(net::URLRequest* request,
+                              int64_t bytes_received) final;
+
+  void OnNetworkBytesSent(net::URLRequest* request,
+                          int64_t bytes_sent) final;
 
   void OnCompleted(net::URLRequest* request, bool started) final;
 
@@ -89,13 +92,12 @@ class NetworkDelegate final : public net::NetworkDelegate {
   bool OnCanAccessFile(const net::URLRequest& request,
                        const base::FilePath& path) const final;
 
-  bool OnCanThrottleRequest(const net::URLRequest& request) const final;
-
   bool OnCanEnablePrivacyMode(
       const GURL& url,
       const GURL& first_party_for_cookies) const final;
 
-  bool OnFirstPartyOnlyCookieExperimentEnabled() const final;
+  bool OnAreExperimentalCookieFeaturesEnabled() const final;
+  bool OnAreStrictSecureCookiesEnabled() const final;
 
   bool OnCancelURLRequestWithPolicyViolatingReferrerHeader(
       const net::URLRequest& request,
