@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013-2014 Canonical Ltd.
+// Copyright (C) 2013-2016 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -17,71 +17,138 @@
 
 #include "oxide_render_widget_host_view.h"
 
+#include <utility>
+
 #include "base/bind.h"
+#include "base/command_line.h"
 #include "base/logging.h"
 #include "base/memory/scoped_vector.h"
 #include "cc/layers/delegated_frame_provider.h"
 #include "cc/layers/delegated_renderer_layer.h"
+#include "cc/layers/layer_settings.h"
 #include "cc/output/compositor_frame.h"
 #include "cc/output/compositor_frame_ack.h"
 #include "cc/output/delegated_frame_data.h"
+#include "cc/output/viewport_selection_bound.h"
 #include "cc/quads/render_pass.h"
+#include "content/browser/renderer_host/render_widget_host_delegate.h"
 #include "content/browser/renderer_host/render_widget_host_impl.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/render_process_host.h"
+#include "content/public/browser/render_view_host.h"
+#include "content/public/common/content_switches.h"
 #include "third_party/WebKit/public/platform/WebCursorInfo.h"
+#include "third_party/WebKit/public/platform/WebGestureDevice.h"
+#include "third_party/WebKit/public/web/WebInputEvent.h"
+#include "ui/base/touch/selection_bound.h"
+#include "ui/events/gesture_detection/motion_event.h"
+#include "ui/gfx/geometry/point_conversions.h"
 #include "ui/gfx/geometry/rect_conversions.h"
 #include "ui/gfx/geometry/size_conversions.h"
+#include "ui/touch_selection/touch_selection_controller.h"
 
 #include "shared/browser/compositor/oxide_compositor.h"
-#include "shared/browser/compositor/oxide_compositor_utils.h"
+#include "shared/browser/input/oxide_input_method_context.h"
 
 #include "oxide_browser_platform_integration.h"
 #include "oxide_browser_process_main.h"
+#include "oxide_event_utils.h"
 #include "oxide_renderer_frame_evictor.h"
-#include "oxide_render_widget_host_view_delegate.h"
+#include "oxide_render_widget_host_view_container.h"
 
 namespace oxide {
 
+namespace {
+
+const float kMobileViewportWidthEpsilon = 0.15f;
+
+bool ShouldSendPinchGesture() {
+  static bool pinch_allowed =
+      base::CommandLine::ForCurrentProcess()->HasSwitch(
+        switches::kEnableViewport) ||
+      base::CommandLine::ForCurrentProcess()->HasSwitch(
+        switches::kEnablePinch);
+  return pinch_allowed;
+}
+
+bool HasFixedPageScale(const cc::CompositorFrameMetadata& frame_metadata) {
+  return frame_metadata.min_page_scale_factor ==
+         frame_metadata.max_page_scale_factor;
+}
+
+bool HasMobileViewport(const cc::CompositorFrameMetadata& frame_metadata) {
+  float window_width_dip =
+      frame_metadata.page_scale_factor *
+      frame_metadata.scrollable_viewport_size.width();
+  float content_width_css = frame_metadata.root_layer_size.width();
+  return content_width_css <= window_width_dip + kMobileViewportWidthEpsilon;
+}
+
+ui::SelectionBound::Type ConvertSelectionBoundType(
+    cc::SelectionBoundType type) {
+  switch (type) {
+    case cc::SELECTION_BOUND_LEFT:
+      return ui::SelectionBound::LEFT;
+    case cc::SELECTION_BOUND_RIGHT:
+      return ui::SelectionBound::RIGHT;
+    case cc::SELECTION_BOUND_CENTER:
+      return ui::SelectionBound::CENTER;
+    case cc::SELECTION_BOUND_EMPTY:
+      return ui::SelectionBound::EMPTY;
+  }
+  NOTREACHED() << "Unknown selection bound type";
+  return ui::SelectionBound::EMPTY;
+}
+
+// Copied from content/browser/renderer_host/input/ui_touch_selection_helper.cc,
+// because that helper is not part of content’s public API.
+ui::SelectionBound ConvertSelectionBound(
+    const cc::ViewportSelectionBound& bound) {
+  ui::SelectionBound ui_bound;
+  ui_bound.set_type(ConvertSelectionBoundType(bound.type));
+  ui_bound.set_visible(bound.visible);
+  if (ui_bound.type() != ui::SelectionBound::EMPTY)
+    ui_bound.SetEdge(bound.edge_top, bound.edge_bottom);
+  return ui_bound;
+}
+
+} // namespace
+
 void RenderWidgetHostView::OnTextInputStateChanged(
     ui::TextInputType type,
     bool show_ime_if_needed) {
-  if (type != current_text_input_type_ ||
-      show_ime_if_needed != show_ime_if_needed_) {
-    current_text_input_type_ = type;
-    show_ime_if_needed_ = show_ime_if_needed;
-
-    if (delegate_) {
-      delegate_->TextInputStateChanged(current_text_input_type_,
-                                       show_ime_if_needed_);
-    }
-  }
+  ime_bridge_.TextInputStateChanged(type, show_ime_if_needed);
 }
 
 void RenderWidgetHostView::OnSelectionBoundsChanged(
     const gfx::Rect& anchor_rect,
     const gfx::Rect& focus_rect,
     bool is_anchor_first) {
-  caret_rect_ = gfx::UnionRects(anchor_rect, focus_rect);
+  gfx::Rect caret_rect = gfx::UnionRects(anchor_rect, focus_rect);
+
+  size_t selection_cursor_position = 0;
+  size_t selection_anchor_position = 0;
 
   if (selection_range_.IsValid()) {
     if (is_anchor_first) {
-      selection_cursor_position_ =
+      selection_cursor_position =
           selection_range_.GetMax() - selection_text_offset_;
-      selection_anchor_position_ =
+      selection_anchor_position =
           selection_range_.GetMin() - selection_text_offset_;
     } else {
-      selection_cursor_position_ =
+      selection_cursor_position =
           selection_range_.GetMin() - selection_text_offset_;
-      selection_anchor_position_ =
+      selection_anchor_position =
           selection_range_.GetMax() - selection_text_offset_;
     }
   }
 
-  if (delegate_) {
-    delegate_->SelectionBoundsChanged(caret_rect_,
-                                      selection_cursor_position_,
-                                      selection_anchor_position_);
+  ime_bridge_.SelectionBoundsChanged(caret_rect,
+                                     selection_cursor_position,
+                                     selection_anchor_position);
+
+  if (container_) {
+    container_->EditingCapabilitiesChanged();
   }
 }
 
@@ -101,17 +168,17 @@ void RenderWidgetHostView::SelectionChanged(const base::string16& text,
 
   content::RenderWidgetHostViewBase::SelectionChanged(text, offset, range);
 
-  if (delegate_) {
-    delegate_->SelectionChanged();
+  if (ime_bridge_.context()) {
+    ime_bridge_.context()->SelectionChanged();
   }
 }
 
 gfx::Size RenderWidgetHostView::GetPhysicalBackingSize() const {
-  if (!delegate_) {
+  if (!container_) {
     return gfx::Size();
   }
 
-  return delegate_->GetViewSizePix();
+  return container_->GetViewSizePix();
 }
 
 bool RenderWidgetHostView::DoTopControlsShrinkBlinkSize() const {
@@ -119,22 +186,23 @@ bool RenderWidgetHostView::DoTopControlsShrinkBlinkSize() const {
 }
 
 float RenderWidgetHostView::GetTopControlsHeight() const {
-  if (!delegate_) {
+  if (!container_) {
     return 0.0f;
   }
 
-  return delegate_->GetLocationBarHeightDip();
+  if (container_->IsFullscreen()) {
+    return 0.0f;
+  }
+
+  return container_->GetLocationBarHeightDip();
 }
 
 void RenderWidgetHostView::FocusedNodeChanged(bool is_editable_node) {
-  focused_node_is_editable_ = is_editable_node;
-  if (delegate_) {
-    delegate_->FocusedNodeChanged(is_editable_node);
-  }
+  ime_bridge_.FocusedNodeChanged(is_editable_node);
 }
 
 void RenderWidgetHostView::OnSwapCompositorFrame(
-    uint32 output_surface_id,
+    uint32_t output_surface_id,
     scoped_ptr<cc::CompositorFrame> frame) {
   if (!frame->delegated_frame_data) {
     DLOG(ERROR) << "Non delegated renderer path is not supported";
@@ -143,7 +211,7 @@ void RenderWidgetHostView::OnSwapCompositorFrame(
   }
 
   scoped_ptr<cc::DelegatedFrameData> frame_data =
-      frame->delegated_frame_data.Pass();
+      std::move(frame->delegated_frame_data);
 
   if (frame_data->render_pass_list.empty()) {
     DLOG(ERROR) << "Invalid delegated frame";
@@ -151,9 +219,6 @@ void RenderWidgetHostView::OnSwapCompositorFrame(
     return;
   }
 
-  Compositor* compositor = delegate_ ? delegate_->GetCompositor() : nullptr;
-  CompositorLock lock(compositor);
-
   if (output_surface_id != last_output_surface_id_) {
     resource_collection_->SetClient(nullptr);
     if (resource_collection_->LoseAllResources()) {
@@ -175,7 +240,7 @@ void RenderWidgetHostView::OnSwapCompositorFrame(
   ack_callbacks_.push(ack_callback);
 
   float device_scale_factor = frame->metadata.device_scale_factor;
-  cc::RenderPass* root_pass = frame_data->render_pass_list.back();
+  cc::RenderPass* root_pass = frame_data->render_pass_list.back().get();
 
   gfx::Size frame_size = root_pass->output_rect.size();
   gfx::Size frame_size_dip = gfx::Size(
@@ -183,7 +248,8 @@ void RenderWidgetHostView::OnSwapCompositorFrame(
       std::lround(frame_size.height() / device_scale_factor));
 
   gfx::Rect damage_rect_dip = gfx::ToEnclosingRect(
-      gfx::ScaleRect(root_pass->damage_rect, 1.0f / device_scale_factor));
+      gfx::ScaleRect(gfx::RectF(root_pass->damage_rect),
+                     1.0f / device_scale_factor));
 
   if (frame_size.IsEmpty()) {
     DestroyDelegatedContent();
@@ -192,27 +258,31 @@ void RenderWidgetHostView::OnSwapCompositorFrame(
         frame_size != frame_provider_->frame_size() ||
         frame_size_dip != last_frame_size_dip_) {
       DetachLayer();
+
       frame_provider_ = new cc::DelegatedFrameProvider(resource_collection_,
-                                                       frame_data.Pass());
-      layer_ = cc::DelegatedRendererLayer::Create(frame_provider_);
+                                                       std::move(frame_data));
+      layer_ = cc::DelegatedRendererLayer::Create(cc::LayerSettings(),
+                                                  frame_provider_);
+      layer_->SetIsDrawable(true);
+      layer_->SetContentsOpaque(true);
+      layer_->SetBounds(frame_size_dip);
+      layer_->SetHideLayerAndSubtree(!is_showing_);
+
       AttachLayer();
     } else {
-      frame_provider_->SetFrameData(frame_data.Pass());
+      frame_provider_->SetFrameData(std::move(frame_data));
     }
   }
 
   last_frame_size_dip_ = frame_size_dip;
 
   if (layer_.get()) {
-    layer_->SetIsDrawable(true);
-    layer_->SetContentsOpaque(true);
-    layer_->SetBounds(frame_size_dip);
     layer_->SetNeedsDisplayRect(damage_rect_dip);
   }
 
   if (frame_is_evicted_) {
     frame_is_evicted_ = false;
-    RendererFrameEvictor::GetInstance()->AddFrame(this, IsShowing());
+    RendererFrameEvictor::GetInstance()->AddFrame(this, is_showing_);
   }
 
   compositor_frame_metadata_ = frame->metadata;
@@ -225,9 +295,34 @@ void RenderWidgetHostView::OnSwapCompositorFrame(
     host_->WasResized();
   }
 
+  bool has_mobile_viewport = HasMobileViewport(compositor_frame_metadata_);
+  bool has_fixed_page_scale = HasFixedPageScale(compositor_frame_metadata_);
+  gesture_provider_->SetDoubleTapSupportForPageEnabled(
+      !has_fixed_page_scale && !has_mobile_viewport);
+
+  Compositor* compositor = container_ ? container_->GetCompositor() : nullptr;
   if (!compositor || !compositor->IsActive()) {
     RunAckCallbacks();
   }
+
+  const cc::ViewportSelection& selection = compositor_frame_metadata_.selection;
+  selection_controller_->OnSelectionEditable(selection.is_editable);
+  selection_controller_->OnSelectionEmpty(selection.is_empty_text_form_control);
+  selection_controller_->OnSelectionBoundsChanged(
+      ConvertSelectionBound(selection.start),
+      ConvertSelectionBound(selection.end));
+}
+
+void RenderWidgetHostView::ClearCompositorFrame() {
+  DestroyDelegatedContent();
+}
+
+void RenderWidgetHostView::ProcessAckedTouchEvent(
+    const content::TouchEventWithLatencyInfo& touch,
+    content::InputEventAckState ack_result) {
+  gesture_provider_->OnTouchEventAck(
+      touch.event.uniqueTouchEventId,
+      ack_result == content::INPUT_EVENT_ACK_STATE_CONSUMED);
 }
 
 void RenderWidgetHostView::InitAsPopup(
@@ -245,12 +340,17 @@ void RenderWidgetHostView::MovePluginWindows(
     const std::vector<content::WebPluginGeometry>& moves) {}
 
 void RenderWidgetHostView::UpdateCursor(const content::WebCursor& cursor) {
-  if (cursor.IsEqual(current_cursor_)) {
+  if (cursor.IsEqual(web_cursor_)) {
     return;
   }
 
-  current_cursor_ = cursor;
-  UpdateCursorOnWebView();
+  web_cursor_ = cursor;
+
+  if (is_loading_) {
+    return;
+  }
+
+  UpdateCurrentCursor();
 }
 
 void RenderWidgetHostView::SetIsLoading(bool is_loading) {
@@ -259,20 +359,15 @@ void RenderWidgetHostView::SetIsLoading(bool is_loading) {
   }
 
   is_loading_ = is_loading;
-  UpdateCursorOnWebView();
+  UpdateCurrentCursor();
 }
 
-void RenderWidgetHostView::TextInputTypeChanged(ui::TextInputType type,
-                                                ui::TextInputMode mode,
-                                                bool can_compose_inline,
-                                                int flags) {}
-
 void RenderWidgetHostView::ImeCancelComposition() {
-  if (!delegate_) {
+  if (!ime_bridge_.context()) {
     return;
   }
 
-  delegate_->ImeCancelComposition();
+  ime_bridge_.context()->CancelComposition();
 }
 
 void RenderWidgetHostView::RenderProcessGone(base::TerminationStatus status,
@@ -290,17 +385,17 @@ void RenderWidgetHostView::SetTooltipText(const base::string16& tooltip_text) {}
 void RenderWidgetHostView::CopyFromCompositingSurface(
     const gfx::Rect& src_subrect,
     const gfx::Size& dst_size,
-    content::ReadbackRequestCallback& callback,
+    const content::ReadbackRequestCallback& callback,
     const SkColorType color_type) {
-  callback.Run(SkBitmap(), content::READBACK_NOT_SUPPORTED);
+  callback.Run(SkBitmap(), content::READBACK_FAILED);
 }
 
 void RenderWidgetHostView::CopyFromCompositingSurfaceToVideoFrame(
     const gfx::Rect& src_subrect,
     const scoped_refptr<media::VideoFrame>& target,
-    const base::Callback<void(bool)>& callback) {
+    const base::Callback<void(const gfx::Rect&, bool)>& callback) {
   NOTIMPLEMENTED();
-  callback.Run(false);
+  callback.Run(gfx::Rect(), false);
 }
 
 bool RenderWidgetHostView::CanCopyToVideoFrame() const {
@@ -313,50 +408,43 @@ bool RenderWidgetHostView::HasAcceleratedSurface(
 }
 
 void RenderWidgetHostView::GetScreenInfo(blink::WebScreenInfo* result) {
-  if (!delegate_) {
+  if (!container_) {
     *result =
         BrowserPlatformIntegration::GetInstance()->GetDefaultScreenInfo();
     return;
   }
 
-  *result = delegate_->GetScreenInfo();
+  *result = container_->GetScreenInfo();
+}
+
+bool RenderWidgetHostView::GetScreenColorProfile(
+    std::vector<char>* color_profile) {
+  DCHECK(color_profile->empty());
+  NOTREACHED();
+  return false;
 }
 
 gfx::Rect RenderWidgetHostView::GetBoundsInRootWindow() {
   return GetViewBounds();
 }
 
-gfx::GLSurfaceHandle RenderWidgetHostView::GetCompositingSurface() {
-  if (shared_surface_handle_.is_null()) {
-    shared_surface_handle_ =
-        CompositorUtils::GetInstance()->GetSharedSurfaceHandle();
-  }
-
-  return shared_surface_handle_;
-}
-
 void RenderWidgetHostView::ShowDisambiguationPopup(
     const gfx::Rect& rect_pixels,
     const SkBitmap& zoomed_bitmap) {}
 
-void RenderWidgetHostView::ProcessAckedTouchEvent(
-    const content::TouchEventWithLatencyInfo& touch,
-    content::InputEventAckState ack_result) {
-  if (!delegate_) {
-    return;
-  }
+void RenderWidgetHostView::LockCompositingSurface() {
+  NOTIMPLEMENTED();
+}
 
-  delegate_->ProcessAckedTouchEvent(
-      ack_result == content::INPUT_EVENT_ACK_STATE_CONSUMED);
+void RenderWidgetHostView::UnlockCompositingSurface() {
+  NOTIMPLEMENTED();
 }
 
 void RenderWidgetHostView::ImeCompositionRangeChanged(
     const gfx::Range& range,
     const std::vector<gfx::Rect>& character_bounds) {}
 
-void RenderWidgetHostView::InitAsChild(gfx::NativeView parent_view) {
-  NOTREACHED() << "InitAsChild() isn't used. Please use Init() instead";
-}
+void RenderWidgetHostView::InitAsChild(gfx::NativeView parent_view) {}
 
 gfx::Vector2dF RenderWidgetHostView::GetLastScrollOffset() const {
   NOTREACHED();
@@ -376,64 +464,28 @@ gfx::NativeViewAccessible RenderWidgetHostView::GetNativeViewAccessible() {
 }
 
 bool RenderWidgetHostView::HasFocus() const {
-  if (!delegate_) {
+  if (!container_) {
     return false;
   }
 
-  return delegate_->HasFocus();
+  return container_->HasFocus(this);
 }
 
 bool RenderWidgetHostView::IsSurfaceAvailableForCopy() const {
   return true;
 }
 
-void RenderWidgetHostView::Show() {
-  if (is_showing_) {
-    DCHECK(delegate_);
-    return;
-  }
-
-  if (!delegate_) {
-    return;
-  }
-
-  is_showing_ = true;
-
-  if (!frame_is_evicted_) {
-    RendererFrameEvictor::GetInstance()->LockFrame(this);
-  }
-
-  host_->WasShown(ui::LatencyInfo());
-}
-
-void RenderWidgetHostView::Hide() {
-  if (!is_showing_) {
-    return;
-  }
-
-  is_showing_ = false;
-
-  if (!frame_is_evicted_) {
-    RendererFrameEvictor::GetInstance()->UnlockFrame(this);
-  }
-
-  host_->WasHidden();
-
-  RunAckCallbacks();
-}
-
 bool RenderWidgetHostView::IsShowing() {
-  DCHECK(!is_showing_ || delegate_);
-  return is_showing_;
+  return is_showing_ && container_ && container_->IsVisible();
 }
 
 gfx::Rect RenderWidgetHostView::GetViewBounds() const {
   gfx::Rect bounds;
 
-  if (!delegate_) {
+  if (!container_) {
     bounds = gfx::Rect(last_size_);
   } else {
-    bounds = delegate_->GetViewBoundsDip();
+    bounds = container_->GetViewBoundsDip();
   }
 
   if (DoTopControlsShrinkBlinkSize()) {
@@ -449,34 +501,154 @@ bool RenderWidgetHostView::LockMouse() {
 
 void RenderWidgetHostView::UnlockMouse() {}
 
+void RenderWidgetHostView::CompositorDidCommit() {
+  RunAckCallbacks();
+}
+
+void RenderWidgetHostView::OnGestureEvent(
+    const blink::WebGestureEvent& event) {
+  if (!host_) {
+    return;
+  }
+
+  if ((event.type == blink::WebInputEvent::GesturePinchBegin ||
+       event.type == blink::WebInputEvent::GesturePinchUpdate ||
+       event.type == blink::WebInputEvent::GesturePinchEnd) &&
+      !ShouldSendPinchGesture()) {
+    return;
+  }
+
+  if (HandleGestureForTouchSelection(event)) {
+    return;
+  }
+
+  if (event.type == blink::WebInputEvent::GestureTapDown) {
+    // Webkit does not stop a fling-scroll on tap-down. So explicitly send an
+    // event to stop any in-progress flings.
+    blink::WebGestureEvent fling_cancel = event;
+    fling_cancel.type = blink::WebInputEvent::GestureFlingCancel;
+    fling_cancel.sourceDevice = blink::WebGestureDeviceTouchpad;
+    host_->ForwardGestureEvent(fling_cancel);
+  }
+
+  if (event.type == blink::WebInputEvent::Undefined) {
+    return;
+  }
+
+  host_->ForwardGestureEvent(event);
+}
+
+bool RenderWidgetHostView::HandleGestureForTouchSelection(
+    const blink::WebGestureEvent& event) const {
+  switch (event.type) {
+    case blink::WebInputEvent::GestureLongPress: {
+      base::TimeTicks event_time = base::TimeTicks() +
+          base::TimeDelta::FromSecondsD(event.timeStampSeconds);
+      gfx::PointF location(event.x, event.y);
+      if (selection_controller_->WillHandleLongPressEvent(
+              event_time, location)) {
+        return true;
+      }
+      break;
+    }
+    case blink::WebInputEvent::GestureTap: {
+      gfx::PointF location(event.x, event.y);
+      if (selection_controller_->WillHandleTapEvent(
+              location, event.data.tap.tapCount)) {
+        return true;
+      }
+      break;
+    }
+    case blink::WebInputEvent::GestureScrollBegin:
+      // XXX: currently commented out because when doing a pinch-to-zoom
+      // gesture, we don’t always get the corresponding GestureScrollEnd event,
+      // so selection handles would remain hidden.
+      //selection_controller()->SetTemporarilyHidden(true);
+      break;
+    case blink::WebInputEvent::GestureScrollEnd:
+      // XXX: see above
+      //selection_controller()->SetTemporarilyHidden(false);
+      break;
+    default:
+      break;
+  }
+  return false;
+}
+
+void RenderWidgetHostView::EvictCurrentFrame() {
+  frame_is_evicted_ = true;
+  DestroyDelegatedContent();
+}
+
 void RenderWidgetHostView::UnusedResourcesAreAvailable() {
   if (ack_callbacks_.empty()) {
     SendReturnedDelegatedResources();
   }
 }
 
-void RenderWidgetHostView::EvictCurrentFrame() {
-  frame_is_evicted_ = true;
-  DestroyDelegatedContent();
+bool RenderWidgetHostView::SupportsAnimation() const {
+  return false;
+}
 
-  if (delegate_) {
-    delegate_->EvictCurrentFrame();
+void RenderWidgetHostView::SetNeedsAnimate() {
+  NOTREACHED();
+}
+
+void RenderWidgetHostView::MoveCaret(const gfx::PointF& position) {
+  content::RenderWidgetHostImpl* rwhi =
+      content::RenderWidgetHostImpl::From(host_);
+  rwhi->MoveCaret(gfx::ToRoundedPoint(position));
+}
+
+void RenderWidgetHostView::MoveRangeSelectionExtent(const gfx::PointF& extent) {
+  content::RenderWidgetHostImpl* rwhi =
+      content::RenderWidgetHostImpl::From(host_);
+  content::RenderWidgetHostDelegate* host_delegate = rwhi->delegate();
+  if (host_delegate) {
+    host_delegate->MoveRangeSelectionExtent(gfx::ToRoundedPoint(extent));
   }
 }
 
-void RenderWidgetHostView::UpdateCursorOnWebView() {
-  if (!delegate_) {
-    return;
+void RenderWidgetHostView::SelectBetweenCoordinates(const gfx::PointF& base,
+                                                    const gfx::PointF& extent) {
+  content::RenderWidgetHostImpl* rwhi =
+      content::RenderWidgetHostImpl::From(host_);
+  content::RenderWidgetHostDelegate* host_delegate = rwhi->delegate();
+  if (host_delegate) {
+    host_delegate->SelectRange(gfx::ToRoundedPoint(base),
+                               gfx::ToRoundedPoint(extent));
+  }
+}
+
+void RenderWidgetHostView::OnSelectionEvent(ui::SelectionEventType event) {
+  if (container_) {
+    container_->TouchSelectionChanged();
+  }
+}
+
+scoped_ptr<ui::TouchHandleDrawable> RenderWidgetHostView::CreateDrawable() {
+  if (!container_) {
+    return nullptr;
   }
 
+  return scoped_ptr<ui::TouchHandleDrawable>(
+      container_->CreateTouchHandleDrawable());
+}
+
+void RenderWidgetHostView::UpdateCurrentCursor() {
   if (is_loading_) {
     content::WebCursor::CursorInfo busy_cursor_info(
         blink::WebCursorInfo::TypeWait);
-    content::WebCursor busy_cursor(busy_cursor_info);
-    delegate_->UpdateCursor(busy_cursor);
+    current_cursor_.InitFromCursorInfo(busy_cursor_info);
   } else {
-    delegate_->UpdateCursor(current_cursor_);
+    current_cursor_ = web_cursor_;
   }
+
+  if (!container_) {
+    return;
+  }
+
+  container_->CursorChanged();
 }
 
 void RenderWidgetHostView::DestroyDelegatedContent() {
@@ -485,7 +657,7 @@ void RenderWidgetHostView::DestroyDelegatedContent() {
   layer_ = nullptr;
 }
 
-void RenderWidgetHostView::SendDelegatedFrameAck(uint32 surface_id) {
+void RenderWidgetHostView::SendDelegatedFrameAck(uint32_t surface_id) {
   cc::CompositorFrameAck ack;
   resource_collection_->TakeUnusedResourcesForChildCompositor(&ack.resources);
 
@@ -517,96 +689,119 @@ void RenderWidgetHostView::RunAckCallbacks() {
 }
 
 void RenderWidgetHostView::AttachLayer() {
-  if (!delegate_) {
+  if (!container_) {
     return;
   }
   if (!layer_.get()) {
     return;
   }
 
-  delegate_->GetCompositor()->SetRootLayer(layer_.get());
+  container_->AttachLayer(layer_);
 }
 
 void RenderWidgetHostView::DetachLayer() {
-  if (!delegate_) {
+  if (!container_) {
     return;
   }
   if (!layer_.get()) {
     return;
   }
 
-  delegate_->GetCompositor()->SetRootLayer(scoped_refptr<cc::Layer>());
+  container_->DetachLayer(layer_);
 }
 
-RenderWidgetHostView::RenderWidgetHostView(content::RenderWidgetHost* host) :
-    host_(content::RenderWidgetHostImpl::From(host)),
-    delegate_(nullptr),
-    resource_collection_(new cc::DelegatedFrameResourceCollection()),
-    last_output_surface_id_(0),
-    frame_is_evicted_(true),
-    selection_cursor_position_(0),
-    selection_anchor_position_(0),
-    current_text_input_type_(ui::TEXT_INPUT_TYPE_NONE),
-    show_ime_if_needed_(false),
-    focused_node_is_editable_(false),
-    is_loading_(false),
-    is_showing_(false),
-    top_controls_shrink_blink_size_(false) {
+RenderWidgetHostView::RenderWidgetHostView(
+    content::RenderWidgetHostImpl* host)
+    : host_(host),
+      container_(nullptr),
+      resource_collection_(new cc::DelegatedFrameResourceCollection()),
+      last_output_surface_id_(0),
+      frame_is_evicted_(true),
+      ime_bridge_(this),
+      is_loading_(false),
+      is_showing_(!host->is_hidden()),
+      top_controls_shrink_blink_size_(false),
+      gesture_provider_(GestureProvider::Create(this)) {
   CHECK(host_) << "Implementation didn't supply a RenderWidgetHost";
 
   resource_collection_->SetClient(this);
   host_->SetView(this);
+
+  gesture_provider_->SetDoubleTapSupportForPageEnabled(false);
+
+  ui::TouchSelectionController::Config tsc_config;
+  // default values from ui/events/gesture_detection/gesture_configuration.cc
+  tsc_config.max_tap_duration = base::TimeDelta::FromMilliseconds(150);
+  tsc_config.tap_slop = 15;
+  tsc_config.enable_adaptive_handle_orientation = false;
+  tsc_config.show_on_tap_for_empty_editable = true;
+  tsc_config.enable_longpress_drag_selection = false;
+  selection_controller_.reset(
+      new ui::TouchSelectionController(this, tsc_config));
 }
 
 RenderWidgetHostView::~RenderWidgetHostView() {
   resource_collection_->SetClient(nullptr);
-  SetDelegate(nullptr);
+  SetContainer(nullptr);
 }
 
-void RenderWidgetHostView::CompositorDidCommit() {
-  RunAckCallbacks();
-}
-
-void RenderWidgetHostView::SetDelegate(
-    RenderWidgetHostViewDelegate* delegate) {
-  if (delegate == delegate_) {
+void RenderWidgetHostView::SetContainer(
+    RenderWidgetHostViewContainer* container) {
+  if (container == container_) {
     return;
   }
 
   DetachLayer();
-  delegate_ = delegate;
+  container_ = container;
   AttachLayer();
 
-  if (delegate_) {
-    DCHECK(host_) <<
-        "Shouldn't be attaching to a delegate when we're already destroyed";
-    host_->SendScreenRects();
-    host_->WasResized();
+  CompositorObserver::Observe(
+      container_ ? container_->GetCompositor() : nullptr);
 
-    if (delegate_->IsVisible()) {
-      Show();
-    } else {
-      Hide();
-    }
-
-    UpdateCursorOnWebView();
-    delegate_->TextInputStateChanged(current_text_input_type_,
-                                     show_ime_if_needed_);
-    delegate_->FocusedNodeChanged(focused_node_is_editable_);
-    delegate_->SelectionBoundsChanged(caret_rect_,
-                                      selection_cursor_position_,
-                                      selection_anchor_position_);
-    delegate_->SelectionChanged();
-  } else if (host_) {
-    Hide();
+  if (!host_) {
+    return;
   }
+
+  host_->SendScreenRects();
+  host_->WasResized();
 }
 
 void RenderWidgetHostView::Blur() {
-  host_->SetInputMethodActive(false);
-
   host_->SetActive(false);
   host_->Blur();
+
+  selection_controller_->HideAndDisallowShowingAutomatically();
+}
+
+void RenderWidgetHostView::HandleTouchEvent(const ui::MotionEvent& event) {
+  if (selection_controller_->WillHandleTouchEvent(event)) {
+    return;
+  }
+
+  auto rv = gesture_provider_->OnTouchEvent(event);
+  if (!rv.succeeded) {
+    return;
+  }
+
+  if (!host_) {
+    gesture_provider_->OnTouchEventAck(event.GetUniqueEventId(), false);
+    return;
+  }
+
+  host_->ForwardTouchEventWithLatencyInfo(
+      MakeWebTouchEvent(event, rv.did_generate_scroll),
+      ui::LatencyInfo());
+}
+
+void RenderWidgetHostView::ResetGestureDetection() {
+  const ui::MotionEvent* current_down_event =
+      gesture_provider_->GetCurrentDownEvent();
+  if (current_down_event) {
+    scoped_ptr<ui::MotionEvent> cancel_event = current_down_event->Cancel();
+    HandleTouchEvent(*cancel_event);
+  }
+
+  gesture_provider_->ResetDetection();
 }
 
 content::RenderWidgetHost* RenderWidgetHostView::GetRenderWidgetHost() const {
@@ -624,9 +819,50 @@ void RenderWidgetHostView::SetBounds(const gfx::Rect& rect) {
 void RenderWidgetHostView::Focus() {
   host_->Focus();
   host_->SetActive(true);
+}
 
-  // XXX: Should we have a run-time check to see if this is required?
-  host_->SetInputMethodActive(true);
+void RenderWidgetHostView::Show() {
+  if (is_showing_) {
+    return;
+  }
+  is_showing_ = true;
+
+  if (layer_.get()) {
+    layer_->SetHideLayerAndSubtree(false);
+  }
+
+  if (!frame_is_evicted_) {
+    RendererFrameEvictor::GetInstance()->LockFrame(this);
+  }
+
+  if (!host_ || !host_->is_hidden()) {
+    return;
+  }
+
+  host_->WasShown(ui::LatencyInfo());
+}
+
+void RenderWidgetHostView::Hide() {
+  if (!is_showing_) {
+    return;
+  }
+  is_showing_ = false;
+
+  if (layer_.get()) {
+    layer_->SetHideLayerAndSubtree(true);
+  }
+
+  if (!frame_is_evicted_) {
+    RendererFrameEvictor::GetInstance()->UnlockFrame(this);
+  }
+
+  RunAckCallbacks();
+
+  if (!host_ || host_->is_hidden()) {
+    return;
+  }
+
+  host_->WasHidden();
 }
 
 } // namespace oxide
