@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013-2014 Canonical Ltd.
+// Copyright (C) 2013-2016 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -21,19 +21,20 @@
 #include <queue>
 #include <string>
 
-#include "base/basictypes.h"
 #include "base/callback.h"
-#include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "cc/layers/delegated_frame_resource_collection.h"
 #include "cc/output/compositor_frame_metadata.h"
 #include "content/common/cursors/webcursor.h"
-#include "ui/base/ime/text_input_type.h"
-#include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/size.h"
+#include "ui/touch_selection/touch_selection_controller.h"
 
+#include "shared/browser/compositor/oxide_compositor_observer.h"
+#include "shared/browser/input/oxide_ime_bridge_impl.h"
+#include "shared/browser/oxide_gesture_provider.h"
 #include "shared/browser/oxide_renderer_frame_evictor_client.h"
 #include "shared/port/content/browser/render_widget_host_view_oxide.h"
 
@@ -46,41 +47,62 @@ namespace content {
 class RenderWidgetHostImpl;
 }
 
+namespace ui {
+class MotionEvent;
+class TouchHandleDrawable;
+class TouchSelectionController;
+}
+
 namespace oxide {
 
-class RenderWidgetHostViewDelegate;
-class WebView;
+class RenderWidgetHostViewContainer;
 
 class RenderWidgetHostView final :
     public content::RenderWidgetHostViewOxide,
+    public CompositorObserver,
+    public GestureProviderClient,
     public RendererFrameEvictorClient,
     public cc::DelegatedFrameResourceCollectionClient,
+    public ui::TouchSelectionControllerClient,
     public base::SupportsWeakPtr<RenderWidgetHostView> {
  public:
-  RenderWidgetHostView(content::RenderWidgetHost* render_widget_host);
+  RenderWidgetHostView(content::RenderWidgetHostImpl* render_widget_host);
   ~RenderWidgetHostView();
 
-  content::RenderWidgetHostImpl* host() const { return host_; }
+  void SetContainer(RenderWidgetHostViewContainer* container);
 
-  void CompositorDidCommit();
-  void SetDelegate(RenderWidgetHostViewDelegate* delegate);
+  ImeBridgeImpl* ime_bridge() { return &ime_bridge_; }
 
   const base::string16& selection_text() const {
     return selection_text_;
   }
 
+  const gfx::Range& selection_range() const {
+    return selection_range_;
+  }
+
   const cc::CompositorFrameMetadata& compositor_frame_metadata() const {
     return compositor_frame_metadata_;
   }
 
-  // content::RenderWidgetHostViewBase implementation
-  void Blur() final;
+  const content::WebCursor& current_cursor() const { return current_cursor_; }
+
+  void HandleTouchEvent(const ui::MotionEvent& event);
+  void ResetGestureDetection();
+
+  void Blur();
 
   // content::RenderWidgetHostView implementation
   content::RenderWidgetHost* GetRenderWidgetHost() const final;
   void SetSize(const gfx::Size& size) final;
   void SetBounds(const gfx::Rect& rect) final;
   void Focus() final;
+  void Show() final;
+  void Hide() final;
+
+  ui::TouchSelectionController* selection_controller() const {
+    return selection_controller_.get();
+  }
 
  private:
   // content::RenderWidgetHostViewOxide implementation
@@ -98,8 +120,11 @@ class RenderWidgetHostView final :
   bool DoTopControlsShrinkBlinkSize() const final;
   float GetTopControlsHeight() const final;
   void FocusedNodeChanged(bool is_editable_node) final;
-  void OnSwapCompositorFrame(uint32 output_surface_id,
+  void OnSwapCompositorFrame(uint32_t output_surface_id,
                              scoped_ptr<cc::CompositorFrame> frame) final;
+  void ClearCompositorFrame() final;
+  void ProcessAckedTouchEvent(const content::TouchEventWithLatencyInfo& touch,
+                              content::InputEventAckState ack_result) final;
   void InitAsPopup(content::RenderWidgetHostView* parent_host_view,
                    const gfx::Rect& pos) final;
   void InitAsFullscreen(
@@ -108,10 +133,6 @@ class RenderWidgetHostView final :
       const std::vector<content::WebPluginGeometry>& moves) final;
   void UpdateCursor(const content::WebCursor& cursor) final;
   void SetIsLoading(bool is_loading) final;
-  void TextInputTypeChanged(ui::TextInputType type,
-                            ui::TextInputMode mode,
-                            bool can_compose_inline,
-                            int flags) final;
   void ImeCancelComposition() final;
   void RenderProcessGone(base::TerminationStatus status, int error_code) final;
   void Destroy() final;
@@ -119,21 +140,21 @@ class RenderWidgetHostView final :
   void CopyFromCompositingSurface(
       const gfx::Rect& src_subrect,
       const gfx::Size& dst_size,
-      content::ReadbackRequestCallback& callback,
+      const content::ReadbackRequestCallback& callback,
       const SkColorType color_type) final;
   void CopyFromCompositingSurfaceToVideoFrame(
       const gfx::Rect& src_subrect,
       const scoped_refptr<media::VideoFrame>& target,
-      const base::Callback<void(bool)>& callback) final;
+      const base::Callback<void(const gfx::Rect&, bool)>& callback) final;
   bool CanCopyToVideoFrame() const final;
   bool HasAcceleratedSurface(const gfx::Size& desired_size) final;
   void GetScreenInfo(blink::WebScreenInfo* results) final;
+  bool GetScreenColorProfile(std::vector<char>* color_profile) final;
   gfx::Rect GetBoundsInRootWindow() final;
-  gfx::GLSurfaceHandle GetCompositingSurface() final;
   void ShowDisambiguationPopup(const gfx::Rect& rect_pixels,
                                const SkBitmap& zoomed_bitmap) final;
-  void ProcessAckedTouchEvent(const content::TouchEventWithLatencyInfo& touch,
-                              content::InputEventAckState ack_result) final;
+  void LockCompositingSurface() final;
+  void UnlockCompositingSurface() final;
   void ImeCompositionRangeChanged(
       const gfx::Range& range,
       const std::vector<gfx::Rect>& character_bounds) final;
@@ -146,33 +167,49 @@ class RenderWidgetHostView final :
   gfx::NativeViewAccessible GetNativeViewAccessible() final;
   bool HasFocus() const final;
   bool IsSurfaceAvailableForCopy() const final;
-  void Show() final;
-  void Hide() final;
   bool IsShowing() final;
   gfx::Rect GetViewBounds() const final;
   bool LockMouse() final;
   void UnlockMouse() final;
 
-  // cc::DelegatedFrameResourceCollectionClient implementation
-  void UnusedResourcesAreAvailable() final;
+  // CompositorObserver implementation
+  void CompositorDidCommit() final;
+
+  // GestureProviderClient implementation
+  void OnGestureEvent(const blink::WebGestureEvent& event) final;
 
   // RendererFrameEvictorClient implemenetation
   void EvictCurrentFrame() final;
 
+  // cc::DelegatedFrameResourceCollectionClient implementation
+  void UnusedResourcesAreAvailable() final;
+
+  // ui::TouchSelectionControllerClient implementation
+  bool SupportsAnimation() const override;
+  void SetNeedsAnimate() override;
+  void MoveCaret(const gfx::PointF& position) override;
+  void MoveRangeSelectionExtent(const gfx::PointF& extent) override;
+  void SelectBetweenCoordinates(const gfx::PointF& base,
+                                const gfx::PointF& extent) override;
+  void OnSelectionEvent(ui::SelectionEventType event) override;
+  scoped_ptr<ui::TouchHandleDrawable> CreateDrawable() override;
+
   // ===================
 
-  void UpdateCursorOnWebView();
+  void UpdateCurrentCursor();
 
   void DestroyDelegatedContent();
-  void SendDelegatedFrameAck(uint32 surface_id);
+  void SendDelegatedFrameAck(uint32_t surface_id);
   void SendReturnedDelegatedResources();
   void RunAckCallbacks();
   void AttachLayer();
   void DetachLayer();
 
+  bool HandleGestureForTouchSelection(const blink::WebGestureEvent& event) const;
+
   content::RenderWidgetHostImpl* host_;
 
-  RenderWidgetHostViewDelegate* delegate_;
+  RenderWidgetHostViewContainer* container_;
 
   gfx::GLSurfaceHandle shared_surface_handle_;
 
@@ -181,7 +218,7 @@ class RenderWidgetHostView final :
   scoped_refptr<cc::DelegatedRendererLayer> layer_;
 
   // The output surface ID for the last frame from the renderer
-  uint32 last_output_surface_id_;
+  uint32_t last_output_surface_id_;
 
   std::queue<base::Closure> ack_callbacks_;
 
@@ -189,15 +226,10 @@ class RenderWidgetHostView final :
 
   bool frame_is_evicted_;
 
-  gfx::Rect caret_rect_;
-  size_t selection_cursor_position_;
-  size_t selection_anchor_position_;
-
-  ui::TextInputType current_text_input_type_;
-  bool show_ime_if_needed_;
-  bool focused_node_is_editable_;
+  ImeBridgeImpl ime_bridge_;
 
   bool is_loading_;
+  content::WebCursor web_cursor_;
   content::WebCursor current_cursor_;
 
   bool is_showing_;
@@ -207,6 +239,10 @@ class RenderWidgetHostView final :
 
   bool top_controls_shrink_blink_size_;
 
+  scoped_ptr<GestureProvider> gesture_provider_;
+
+  scoped_ptr<ui::TouchSelectionController> selection_controller_;
+
   DISALLOW_IMPLICIT_CONSTRUCTORS(RenderWidgetHostView);
 };
 
