@@ -18,11 +18,14 @@
 #ifndef _OXIDE_SHARED_BROWSER_RESOURCE_DISPATCHER_HOST_DELEGATE_H_
 #define _OXIDE_SHARED_BROWSER_RESOURCE_DISPATCHER_HOST_DELEGATE_H_
 
+#include "base/callback.h"
 #include "base/macros.h"
+#include "base/strings/string16.h"
 #include "content/public/browser/resource_dispatcher_host_delegate.h"
 
 namespace content {
 class ResourceContext;
+class ResourceThrottle;
 struct Referrer;
 }
 
@@ -33,12 +36,26 @@ class CookieStore;
 
 namespace oxide {
 
+class ResourceDispatcherHostLoginDelegate;
+
 class ResourceDispatcherHostDelegate
     : public content::ResourceDispatcherHostDelegate {
  public:
+
   ResourceDispatcherHostDelegate();
   ~ResourceDispatcherHostDelegate() override;
 
+  void DownloadStarting(
+      net::URLRequest* request,
+      content::ResourceContext* resource_context,
+      int child_id,
+      int route_id,
+      int request_id,
+      bool is_content_initiated,
+      bool must_download,
+      const std::string& suggested_filename,
+      ScopedVector<content::ResourceThrottle>* throttles) override;
+
  private:
   struct DownloadRequestParams;
 
@@ -51,7 +68,8 @@ class ResourceDispatcherHostDelegate
                                const std::string& mime_type,
                                int render_process_id,
                                int render_view_id,
-                               content::ResourceContext* resource_context);
+                               content::ResourceContext* resource_context,
+                               net::URLRequest* url_request);
 
   static void DispatchDownloadRequestWithCookies(
       const DownloadRequestParams& params,
@@ -67,20 +85,19 @@ class ResourceDispatcherHostDelegate
       content::AppCacheService* appcache_service,
       content::ResourceType resource_type,
       ScopedVector<content::ResourceThrottle>* throttles) override;
-  bool HandleExternalProtocol(const GURL& url,
-                              int child_id,
-                              int route_id) override;
-  bool ShouldDownloadUrl(const GURL& url,
-                         const GURL& first_party_url,
-                         bool is_content_initiated,
-                         const base::string16& suggested_name,
-                         const bool use_prompt,
-                         const content::Referrer& referrer,
-                         const std::string& mime_type,
-                         int render_process_id,
-                         int render_view_id,
-                         content::ResourceContext* resource_context) override;
+  bool HandleExternalProtocol(
+      const GURL& url,
+      int child_id,
+      const content::ResourceRequestInfo::WebContentsGetter& web_contents_getter,
+      bool is_main_frame,
+      ui::PageTransition page_transition,
+      bool has_user_gesture) override;
 
+  content::ResourceDispatcherHostLoginDelegate* CreateLoginDelegate(
+      net::AuthChallengeInfo* auth_info,
+      net::URLRequest* request) override;
+
+private:
   DISALLOW_COPY_AND_ASSIGN(ResourceDispatcherHostDelegate);
 };
 
