@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -21,15 +21,27 @@
 #include "content/browser/web_contents/web_contents_impl.h"
 #include "content/public/browser/web_contents.h"
 
+#include "shared/common/oxide_unowned_user_data.h"
+
 #include "oxide_render_widget_host_view.h"
-#include "oxide_web_view.h"
+#include "oxide_render_widget_host_view_container.h"
 
 namespace oxide {
 
-WebContentsView::WebContentsView(content::WebContents* web_contents) :
-    web_contents_(web_contents) {}
+namespace {
+int kUserDataKey;
+}
 
-WebContentsView::~WebContentsView() {}
+WebContentsView::WebContentsView(content::WebContents* web_contents)
+    : web_contents_(web_contents),
+      container_(nullptr) {
+  web_contents_->SetUserData(&kUserDataKey,
+                             new UnownedUserData<WebContentsView>(this));
+}
+
+WebContentsView::~WebContentsView() {
+  web_contents_->RemoveUserData(&kUserDataKey);
+}
 
 // static
 content::WebContentsViewOxide* WebContentsView::Create(
@@ -37,8 +49,21 @@ content::WebContentsViewOxide* WebContentsView::Create(
   return new WebContentsView(web_contents);
 }
 
-WebView* WebContentsView::GetWebView() const {
-  return WebView::FromWebContents(web_contents_);
+// static
+WebContentsView* WebContentsView::FromWebContents(
+    content::WebContents* contents) {
+  UnownedUserData<WebContentsView>* data =
+      static_cast<UnownedUserData<WebContentsView>*>(
+        contents->GetUserData(&kUserDataKey));
+  if (!data) {
+    return nullptr;
+  }
+
+  return data->get();
+}
+
+void WebContentsView::SetContainer(RenderWidgetHostViewContainer* container) {
+  container_ = container;
 }
 
 gfx::NativeView WebContentsView::GetNativeView() const {
@@ -54,7 +79,7 @@ gfx::NativeWindow WebContentsView::GetTopLevelNativeWindow() const {
 }
 
 void WebContentsView::GetContainerBounds(gfx::Rect* out) const {
-  *out = GetWebView()->GetViewBoundsDip();
+  *out = container_->GetViewBoundsDip();
 }
 
 void WebContentsView::SizeContents(const gfx::Size& size) {
@@ -79,13 +104,9 @@ void WebContentsView::SetInitialFocus() {
   NOTIMPLEMENTED();
 }
 
-void WebContentsView::StoreFocus() {
-  NOTIMPLEMENTED();
-}
+void WebContentsView::StoreFocus() {}
 
-void WebContentsView::RestoreFocus() {
-  NOTIMPLEMENTED();
-}
+void WebContentsView::RestoreFocus() {}
 
 content::DropData* WebContentsView::GetDropData() const {
   return nullptr;
@@ -107,26 +128,14 @@ void WebContentsView::CreateView(const gfx::Size& initial_size,
 content::RenderWidgetHostViewBase* WebContentsView::CreateViewForWidget(
     content::RenderWidgetHost* render_widget_host,
     bool is_guest_view_hack) {
-  RenderWidgetHostView* rwhv = new RenderWidgetHostView(render_widget_host);
-
-  WebView* view = GetWebView();
-  if (view) {
-    // As RWHV contains the plumbing from WebView::VisibilityChanged to
-    // RenderWidgetHostImpl::Was{Shown,Hidden}, RWHI::is_hidden could be
-    // out of date. This ensures that we sync RWHI::is_hidden with the
-    // real visibility of the webview - see https://launchpad.net/bugs/1322622
-    view->VisibilityChanged();
-
-    // Also sync focus state
-    view->FocusChanged();
-  }
-
-  return rwhv;
+  return new RenderWidgetHostView(
+      content::RenderWidgetHostImpl::From(render_widget_host));
 }
 
 content::RenderWidgetHostViewBase* WebContentsView::CreateViewForPopupWidget(
     content::RenderWidgetHost* render_widget_host) {
-  return nullptr;
+  return new RenderWidgetHostView(
+      content::RenderWidgetHostImpl::From(render_widget_host));
 }
 
 void WebContentsView::SetPageTitle(const base::string16& title) {}
@@ -137,6 +146,23 @@ void WebContentsView::RenderViewSwappedIn(content::RenderViewHost* host) {}
 
 void WebContentsView::SetOverscrollControllerEnabled(bool enabled) {}
 
+void WebContentsView::ShowContextMenu(
+    content::RenderFrameHost* render_frame_host,
+    const content::ContextMenuParams& params) {
+  container_->ShowContextMenu(render_frame_host, params);
+}
+
+void WebContentsView::StartDragging(
+    const content::DropData& drop_data,
+    blink::WebDragOperationsMask allowed_ops,
+    const gfx::ImageSkia& image,
+    const gfx::Vector2d& image_offset,
+    const content::DragEventSourceInfo& event_info) {
+  // TODO: Implement drag and drop support
+  //  see https://launchpad.net/bugs/1459830
+  web_contents_->SystemDragEnded();
+}
+
 void WebContentsView::ShowPopupMenu(
     content::RenderFrameHost* render_frame_host,
     const gfx::Rect& bounds,
@@ -146,13 +172,13 @@ void WebContentsView::ShowPopupMenu(
     const std::vector<content::MenuItem>& items,
     bool right_aligned,
     bool allow_multiple_selection) {
-  GetWebView()->ShowPopupMenu(render_frame_host,
-                              bounds, selected_item, items,
-                              allow_multiple_selection);
+  container_->ShowPopupMenu(render_frame_host,
+                            bounds, selected_item, items,
+                            allow_multiple_selection);
 }
 
 void WebContentsView::HidePopupMenu() {
-  GetWebView()->HidePopupMenu();
+  container_->HidePopupMenu();
 }
 
 } // namespace oxide
