@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,7 @@ class WebContents;
 
 namespace oxide {
 
-class WebView;
+class RenderWidgetHostViewContainer;
 
 class WebContentsView final : public content::WebContentsViewOxide {
  public:
@@ -35,7 +35,9 @@ class WebContentsView final : public content::WebContentsViewOxide {
   static content::WebContentsViewOxide* Create(
       content::WebContents* web_contents);
 
-  WebView* GetWebView() const;
+  static WebContentsView* FromWebContents(content::WebContents* contents);
+
+  void SetContainer(RenderWidgetHostViewContainer* container);
 
   // content::WebContentsView
   gfx::NativeView GetNativeView() const final;
@@ -71,6 +73,13 @@ class WebContentsView final : public content::WebContentsViewOxide {
   void SetOverscrollControllerEnabled(bool enabled) final;
 
   // content::RenderViewHostDelegateView
+  void ShowContextMenu(content::RenderFrameHost* render_frame_host,
+                       const content::ContextMenuParams& params) final;
+  void StartDragging(const content::DropData& drop_data,
+                     blink::WebDragOperationsMask allowed_ops,
+                     const gfx::ImageSkia& image,
+                     const gfx::Vector2d& image_offset,
+                     const content::DragEventSourceInfo& event_info) final;
   void ShowPopupMenu(content::RenderFrameHost* render_frame_host,
                      const gfx::Rect& bounds,
                      int item_height,
@@ -86,6 +95,8 @@ class WebContentsView final : public content::WebContentsViewOxide {
 
   content::WebContents* web_contents_;
 
+  RenderWidgetHostViewContainer* container_;
+
   DISALLOW_IMPLICIT_CONSTRUCTORS(WebContentsView);
 };
 
