@@ -20,8 +20,7 @@
 
 #include <string>
 
-#include "base/basictypes.h"
-#include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "base/observer_list.h"
 #include "base/strings/string16.h"
 
@@ -131,7 +130,7 @@ class WebPreferences {
 
   bool attributes_[ATTR_LAST];
 
-  ObserverList<WebPreferencesObserver> observers_;
+  base::ObserverList<WebPreferencesObserver> observers_;
 
   DISALLOW_COPY_AND_ASSIGN(WebPreferences);
 };
