@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2016 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -22,40 +22,36 @@
 #include <string>
 #include <vector>
 
-#include "base/compiler_specific.h"
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/strings/string16.h"
-#include "base/timer/timer.h"
 #include "cc/output/compositor_frame_metadata.h"
-#include "components/sessions/serialized_navigation_entry.h"
-#include "content/public/browser/certificate_request_result_type.h"
+#include "components/sessions/core/serialized_navigation_entry.h"
+#include "content/public/browser/navigation_controller.h"
 #include "content/public/browser/notification_observer.h"
 #include "content/public/browser/notification_registrar.h"
 #include "content/public/browser/web_contents_delegate.h"
 #include "content/public/browser/web_contents_observer.h"
 #include "content/public/common/javascript_message_type.h"
-#include "content/public/common/permission_status.mojom.h"
-#include "content/public/common/resource_type.h"
 #include "third_party/WebKit/public/platform/WebScreenInfo.h"
 #include "third_party/WebKit/public/platform/WebTopControlsState.h"
-#include "third_party/WebKit/public/web/WebCompositionUnderline.h"
-#include "ui/base/ime/text_input_type.h"
 #include "ui/gfx/geometry/point.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/size.h"
 
 #include "shared/browser/compositor/oxide_compositor_client.h"
-#include "shared/browser/oxide_certificate_error.h"
+#include "shared/browser/compositor/oxide_compositor_observer.h"
+#include "shared/browser/input/oxide_input_method_context_observer.h"
+#include "shared/browser/oxide_browser_platform_integration_observer.h"
 #include "shared/browser/oxide_content_types.h"
-#include "shared/browser/oxide_gesture_provider.h"
-#include "shared/browser/oxide_permission_request.h"
-#include "shared/browser/oxide_render_widget_host_view_delegate.h"
+#include "shared/browser/oxide_render_object_id.h"
+#include "shared/browser/oxide_render_widget_host_view_container.h"
 #include "shared/browser/oxide_script_message_target.h"
 #include "shared/browser/oxide_security_status.h"
 #include "shared/browser/oxide_security_types.h"
+#include "shared/browser/oxide_touch_event_state.h"
 #include "shared/browser/oxide_web_preferences_observer.h"
 #include "shared/common/oxide_message_enums.h"
 
@@ -66,18 +62,20 @@ class WebMouseEvent;
 class WebMouseWheelEvent;
 } // namespace blink
 
+namespace cc {
+class SolidColorLayer;
+}
+
 namespace content {
 
-struct MenuItem;
 class NativeWebKeyboardEvent;
 class NotificationRegistrar;
 struct OpenURLParams;
 class RenderFrameHost;
 class RenderViewHost;
-class RenderWidgetHostImpl;
+class RenderWidgetHost;
 class WebContents;
 class WebContentsImpl;
-class WebCursor;
 
 } // namespace content
 
@@ -86,13 +84,9 @@ class Range;
 class Size;
 }
 
-namespace net {
-class SSLInfo;
-}
-
 namespace ui {
-class GestureEvent;
 class TouchEvent;
+class TouchHandleDrawable;
 }
 
 namespace oxide {
@@ -102,11 +96,14 @@ class Compositor;
 class CompositorFrameHandle;
 class FilePicker;
 class JavaScriptDialog;
+class ResourceDispatcherHostLoginDelegate;
 class RenderWidgetHostView;
+class WebContextMenu;
 class WebFrame;
 class WebPopupMenu;
 class WebPreferences;
 class WebView;
+class WebViewClient;
 class WebViewContentsHelper;
 
 class WebViewIterator final {
@@ -126,37 +123,39 @@ class WebViewIterator final {
   Vector::iterator current_;
 };
 
-// This is the main webview class. Implementations should subclass
-// this. Note that this class will hold the main browser process
-// components alive
+// This is the main webview class. Implementations should customize this by
+// providing an implementation of WebViewClient
 class WebView : public ScriptMessageTarget,
+                private InputMethodContextObserver,
+                private CompositorObserver,
                 private CompositorClient,
                 private WebPreferencesObserver,
-                private GestureProviderClient,
                 private content::NotificationObserver,
-                private RenderWidgetHostViewDelegate,
+                private RenderWidgetHostViewContainer,
                 private content::WebContentsDelegate,
-                private content::WebContentsObserver {
+                private content::WebContentsObserver,
+                private BrowserPlatformIntegrationObserver {
  public:
-  virtual ~WebView();
-
-  // Maps to content::NavigationController::RestoreType
-  enum RestoreType {
-    RESTORE_CURRENT_SESSION,
-    RESTORE_LAST_SESSION_EXITED_CLEANLY,
-    RESTORE_LAST_SESSION_CRASHED,
-  };
 
   struct Params {
     Params();
     ~Params();
 
+    WebViewClient* client;
     BrowserContext* context;
-    scoped_ptr<content::WebContents> contents;
     bool incognito;
+    std::vector<sessions::SerializedNavigationEntry> restore_entries;
+    content::NavigationController::RestoreType restore_type;
+    int restore_index;
   };
 
-  virtual void Init(Params* params);
+  WebView(const Params& params);
+  WebView(scoped_ptr<content::WebContents> contents,
+          WebViewClient* client);
+
+  ~WebView() override;
+
+  WebViewClient* client() const { return client_; }
 
   static WebView* FromWebContents(const content::WebContents* web_contents);
   static WebView* FromRenderViewHost(content::RenderViewHost* rvh);
@@ -172,16 +171,15 @@ class WebView : public ScriptMessageTarget,
   void SetURL(const GURL& url);
 
   std::vector<sessions::SerializedNavigationEntry> GetState() const;
-  void SetState(content::NavigationController::RestoreType type,
-                std::vector<sessions::SerializedNavigationEntry> state,
-                int index);
 
-  void LoadData(const std::string& encodedData,
-                const std::string& mimeType,
-                const GURL& baseUrl);
+  void LoadData(const std::string& encoded_data,
+                const std::string& mime_type,
+                const GURL& base_url);
 
   std::string GetTitle() const;
 
+  const GURL& GetFaviconURL() const;
+
   bool CanGoBack() const;
   bool CanGoForward() const;
 
@@ -194,14 +192,13 @@ class WebView : public ScriptMessageTarget,
 
   bool IsLoading() const;
 
-  bool IsFullscreen() const;
-  void SetIsFullscreen(bool fullscreen);
+  bool FullscreenGranted() const;
+  void SetFullscreenGranted(bool fullscreen);
 
   void WasResized();
   void ScreenUpdated();
   void VisibilityChanged();
   void FocusChanged();
-  void InputPanelVisibilityChanged();
   void UpdateWebPreferences();
 
   BrowserContext* GetBrowserContext() const;
@@ -231,9 +228,9 @@ class WebView : public ScriptMessageTarget,
   gfx::Size GetCompositorFrameContentSizePix();
   gfx::Size GetCompositorFrameViewportSizePix();
 
-  int GetLocationBarOffsetPix();
-  int GetLocationBarContentOffsetPix();
-  float GetLocationBarContentOffsetDip();
+  int GetLocationBarOffsetPix() const;
+  int GetLocationBarContentOffsetPix() const;
+  float GetLocationBarContentOffsetDip() const;
 
   const SecurityStatus& security_status() const { return security_status_; }
 
@@ -261,137 +258,114 @@ class WebView : public ScriptMessageTarget,
 
   void PrepareToClose();
 
-  void ShowPopupMenu(content::RenderFrameHost* render_frame_host,
-                     const gfx::Rect& bounds,
-                     int selected_item,
-                     const std::vector<content::MenuItem>& items,
-                     bool allow_multiple_selection);
-  void HidePopupMenu();
-
-  void RequestGeolocationPermission(
-      const GURL& requesting_frame,
-      int bridge_id,
-      const base::Callback<void(content::PermissionStatus)>& callback);
-  void CancelGeolocationPermissionRequest(
-      const GURL& requesting_frame,
-      int bridge_id);
-
-  void AllowCertificateError(content::RenderFrameHost* rfh,
-                             int cert_error,
-                             const net::SSLInfo& ssl_info,
-                             const GURL& request_url,
-                             content::ResourceType resource_type,
-                             bool overridable,
-                             bool strict_enforcement,
-                             const base::Callback<void(bool)>& callback,
-                             content::CertificateRequestResultType* result);
-                             
   void HandleKeyEvent(const content::NativeWebKeyboardEvent& event);
   void HandleMouseEvent(const blink::WebMouseEvent& event);
   void HandleTouchEvent(const ui::TouchEvent& event);
   void HandleWheelEvent(const blink::WebMouseWheelEvent& event);
 
-  void ImeCommitText(const base::string16& text,
-                     const gfx::Range& replacement_range);
-  void ImeSetComposingText(
-      const base::string16& text,
-      const std::vector<blink::WebCompositionUnderline>& underlines,
-      const gfx::Range& selection_range);
-
   void DownloadRequested(
       const GURL& url,
-      const std::string& mimeType,
-      const bool shouldPrompt,
-      const base::string16& suggestedFilename,
+      const std::string& mime_type,
+      const bool should_prompt,
+      const base::string16& suggested_filename,
       const std::string& cookies,
-      const std::string& referrer);
+      const std::string& referrer,
+      const std::string& user_agent);
+
+  void HttpAuthenticationRequested(
+      ResourceDispatcherHostLoginDelegate* login_delegate);
 
   CompositorFrameHandle* GetCompositorFrameHandle() const;
   void DidCommitCompositorFrame();
 
-  // ============================
+  blink::WebScreenInfo GetScreenInfo() const;
+  gfx::Rect GetViewBoundsPix() const;
+  bool IsVisible() const;
+  bool HasFocus() const;
 
-  virtual blink::WebScreenInfo GetScreenInfo() const = 0;
-  virtual gfx::Rect GetViewBoundsPix() const = 0;
-  virtual bool IsVisible() const = 0;
-  virtual bool HasFocus() const = 0;
-  virtual bool IsInputPanelVisible() const;
-
-  virtual JavaScriptDialog* CreateJavaScriptDialog(
+  JavaScriptDialog* CreateJavaScriptDialog(
       content::JavaScriptMessageType javascript_message_type);
-  virtual JavaScriptDialog* CreateBeforeUnloadDialog();
+  JavaScriptDialog* CreateBeforeUnloadDialog();
 
-  virtual bool CanCreateWindows() const;
+  bool ShouldHandleNavigation(const GURL& url, bool has_user_gesture);
 
- protected:
-  WebView();
+  bool CanCreateWindows() const;
 
-  base::string16 GetSelectedText() const;
-  const base::string16& GetSelectionText() const;
+  const GURL& target_url() const { return target_url_; }
 
-  ui::TextInputType text_input_type_;
-  bool show_ime_if_needed_;
-  bool focused_node_is_editable_;
-
-  gfx::Rect caret_rect_;
-  size_t selection_cursor_position_;
-  size_t selection_anchor_position_;
+  int GetEditFlags() const;
 
  private:
+  WebView(WebViewClient* client);
+
+  void CommonInit(scoped_ptr<content::WebContents> contents);
+
   RenderWidgetHostView* GetRenderWidgetHostView() const;
   content::RenderViewHost* GetRenderViewHost() const;
-  content::RenderWidgetHostImpl* GetRenderWidgetHostImpl() const;
+  content::RenderWidgetHost* GetRenderWidgetHost() const;
 
   void DispatchLoadFailed(const GURL& validated_url,
                           int error_code,
-                          const base::string16& error_description);
+                          const base::string16& error_description,
+                          bool is_provisional_load = false);
 
   void OnDidBlockDisplayingInsecureContent();
   void OnDidBlockRunningInsecureContent();
 
   bool ShouldScrollFocusedEditableNodeIntoView();
-  void MaybeResetAutoScrollTimer();
-  void ScrollFocusedEditableNodeIntoView();
+  void MaybeScrollFocusedEditableNodeIntoView();
 
   float GetFrameMetadataScaleToPix();
 
   void InitializeTopControlsForHost(content::RenderViewHost* rvh,
                                     bool initial_host);
 
+  void DispatchPrepareToCloseResponse(bool proceed);
+
+  void RestartFindInPage();
+
+  void MaybeCancelFullscreenMode();
+
   // ScriptMessageTarget implementation
   virtual size_t GetScriptMessageHandlerCount() const override;
   virtual const ScriptMessageHandler* GetScriptMessageHandlerAt(
       size_t index) const override;
 
-  // CompositorClient implementation
+  // InputMethodContextObserver implementation
+  void InputPanelVisibilityChanged() override;
+
+  // CompositorObserver implementation
   void CompositorDidCommit() final;
-  void CompositorSwapFrame(uint32 surface_id,
-                           CompositorFrameHandle* frame) final;
+
+  // CompositorClient implementation
+  void CompositorSwapFrame(CompositorFrameHandle* handle) final;
 
   // WebPreferencesObserver implementation
   void WebPreferencesDestroyed() final;
 
-  // GestureProviderClient implementation
-  void OnGestureEvent(const blink::WebGestureEvent& event) final;
-
   // content::NotificationObserver implementation
   void Observe(int type,
                const content::NotificationSource& source,
                const content::NotificationDetails& details) final;
 
-  // RenderWidgetHostViewDelegate implementation
-  void EvictCurrentFrame() final;
-  void ProcessAckedTouchEvent(bool consumed) final;
-  void UpdateCursor(const content::WebCursor& cursor) final;
-  void TextInputStateChanged(ui::TextInputType type,
-                             bool show_ime_if_needed) final;
-  void FocusedNodeChanged(bool is_editable_node) final;
-  void ImeCancelComposition() final;
-  void SelectionBoundsChanged(const gfx::Rect& caret_rect,
-                              size_t selection_cursor_position,
-                              size_t selection_anchor_position) final;
-  void SelectionChanged() final;
+  // RenderWidgetHostViewContainer implementation
   Compositor* GetCompositor() const final;
+  void AttachLayer(scoped_refptr<cc::Layer> layer) final;
+  void DetachLayer(scoped_refptr<cc::Layer> layer) final;
+  void CursorChanged() final;
+  bool HasFocus(const RenderWidgetHostView* view) const final;
+  bool IsFullscreen() const final;
+  void ShowContextMenu(content::RenderFrameHost* render_frame_host,
+                       const content::ContextMenuParams& params) final;
+  void ShowPopupMenu(content::RenderFrameHost* render_frame_host,
+                     const gfx::Rect& bounds,
+                     int selected_item,
+                     const std::vector<content::MenuItem>& items,
+                     bool allow_multiple_selection) final;
+  void HidePopupMenu() final;
+  ui::TouchHandleDrawable* CreateTouchHandleDrawable() const final;
+  void TouchSelectionChanged() const final;
+  void EditingCapabilitiesChanged() final;
 
   // content::WebContentsDelegate implementation
   content::WebContents* OpenURLFromTab(content::WebContents* source,
@@ -403,8 +377,9 @@ class WebView : public ScriptMessageTarget,
       content::WebContents* source,
       int route_id,
       int main_frame_route_id,
+      int main_frame_widget_route_id,
       WindowContainerType window_container_type,
-      const base::string16& frame_name,
+      const std::string& frame_name,
       const GURL& target_url,
       const std::string& partition_id,
       content::SessionStorageNamespace* session_storage_namespace,
@@ -414,7 +389,7 @@ class WebView : public ScriptMessageTarget,
                            const content::NativeWebKeyboardEvent& event) final;
   void WebContentsCreated(content::WebContents* source,
                           int source_frame_id,
-                          const base::string16& frame_name,
+                          const std::string& frame_name,
                           const GURL& target_url,
                           content::WebContents* new_contents) final;
   void AddNewContents(content::WebContents* source,
@@ -425,10 +400,11 @@ class WebView : public ScriptMessageTarget,
                       bool* was_blocked) final;
   void LoadProgressChanged(content::WebContents* source, double progress) final;
   void CloseContents(content::WebContents* source) final;
+  void UpdateTargetURL(content::WebContents* source, const GURL& url) final;
   bool AddMessageToConsole(content::WebContents* source,
-               int32 level,
+               int32_t level,
                const base::string16& message,
-               int32 line_no,
+               int32_t line_no,
                const base::string16& source_id) final;
   void BeforeUnloadFired(content::WebContents* source,
                          bool proceed,
@@ -437,19 +413,42 @@ class WebView : public ScriptMessageTarget,
       content::WebContents* source) final;
   void RunFileChooser(content::WebContents* web_contents,
                       const content::FileChooserParams& params) final;
+  bool EmbedsFullscreenWidget() const final;
   void EnterFullscreenModeForTab(content::WebContents* source,
                                  const GURL& origin) final;
   void ExitFullscreenModeForTab(content::WebContents* source) final;
   bool IsFullscreenForTabOrPending(
       const content::WebContents* source) const final;
+  void FindReply(content::WebContents* source,
+                 int request_id,
+                 int number_of_matches,
+                 const gfx::Rect& selection_rect,
+                 int active_match_ordinal,
+                 bool final_update) final;
+  void RequestMediaAccessPermission(
+      content::WebContents* source,
+      const content::MediaStreamRequest& request,
+      const content::MediaResponseCallback& callback) final;
+  bool CheckMediaAccessPermission(content::WebContents* source,
+                                  const GURL& security_origin,
+                                  content::MediaStreamType type) final;
 
   // content::WebContentsObserver implementation
-  void RenderFrameCreated(content::RenderFrameHost* render_frame_host) final;
+  void RenderFrameForInterstitialPageCreated(
+      content::RenderFrameHost* render_frame_host) final;
+  void RenderViewReady() final;
   void RenderProcessGone(base::TerminationStatus status) final;
   void RenderViewHostChanged(content::RenderViewHost* old_host,
                              content::RenderViewHost* new_host) final;
-  void RenderFrameHostChanged(content::RenderFrameHost* old_host,
-                              content::RenderFrameHost* new_host) final;
+  void DidStartLoading() final;
+  void DidStopLoading() final;
+  void DidFinishLoad(content::RenderFrameHost* render_frame_host,
+                     const GURL& validated_url) final;
+  void DidFailLoad(content::RenderFrameHost* render_frame_host,
+                   const GURL& validated_url,
+                   int error_code,
+                   const base::string16& error_description,
+                   bool was_ignored_by_handler) final;
   void DidStartProvisionalLoadForFrame(
       content::RenderFrameHost* render_frame_host,
       const GURL& validated_url,
@@ -463,113 +462,28 @@ class WebView : public ScriptMessageTarget,
       content::RenderFrameHost* render_frame_host,
       const GURL& validated_url,
       int error_code,
-      const base::string16& error_description) final;
+      const base::string16& error_description,
+      bool was_ignored_by_handler) final;
   void DidNavigateMainFrame(
       const content::LoadCommittedDetails& details,
       const content::FrameNavigateParams& params) final;
-  void DidNavigateAnyFrame(
-      content::RenderFrameHost* render_frame_host,
-      const content::LoadCommittedDetails& details,
-      const content::FrameNavigateParams& params) final;
-  void DidFinishLoad(content::RenderFrameHost* render_frame_host,
-                     const GURL& validated_url) final;
-  void DidFailLoad(content::RenderFrameHost* render_frame_host,
-                   const GURL& validated_url,
-                   int error_code,
-                   const base::string16& error_description) final;
   void DidGetRedirectForResourceRequest(
       content::RenderFrameHost* render_frame_host,
       const content::ResourceRedirectDetails& details) final;
   void NavigationEntryCommitted(
       const content::LoadCommittedDetails& load_details) final;
-  void DidStartLoading() final;
-  void DidStopLoading() final;
-  void FrameDeleted(content::RenderFrameHost* render_frame_host) final;
   void TitleWasSet(content::NavigationEntry* entry, bool explicit_set) final;
-  void DidUpdateFaviconURL(
-      const std::vector<content::FaviconURL>& candidates) final;
+  void DidShowFullscreenWidget(int routing_id) final;
+  void DidDestroyFullscreenWidget(int routing_id) final;
+  void DidAttachInterstitialPage() final;
+  void DidDetachInterstitialPage() final;
   bool OnMessageReceived(const IPC::Message& msg,
                          content::RenderFrameHost* render_frame_host) final;
 
-  // Override in sub-classes
-  virtual void OnURLChanged();
-  virtual void OnTitleChanged();
-  virtual void OnIconChanged(const GURL& icon);
-  virtual void OnCommandsUpdated();
+  // BrowserPlatformIntegrationObserver implementation
+  void ClipboardDataChanged() final;
 
-  virtual void OnLoadingChanged();
-  virtual void OnLoadProgressChanged(double progress);
-
-  virtual void OnLoadStarted(const GURL& validated_url);
-  virtual void OnLoadRedirected(const GURL& url,
-                                const GURL& original_url);
-  virtual void OnLoadCommitted(const GURL& url,
-                               bool is_error_page);
-  virtual void OnLoadStopped(const GURL& validated_url);
-  virtual void OnLoadFailed(const GURL& validated_url,
-                            int error_code,
-                            const std::string& error_description);
-  virtual void OnLoadSucceeded(const GURL& validated_url);
-
-  virtual void OnNavigationEntryCommitted();
-  virtual void OnNavigationListPruned(bool from_front, int count);
-  virtual void OnNavigationEntryChanged(int index);
-
-  virtual bool OnAddMessageToConsole(int32 level,
-                                     const base::string16& message,
-                                     int32 line_no,
-                                     const base::string16& source_id);
-
-  virtual void OnToggleFullscreenMode(bool enter);
-
-  virtual void OnWebPreferencesDestroyed();
-
-  virtual void OnRequestGeolocationPermission(
-      scoped_ptr<SimplePermissionRequest> request);
-
-  virtual void OnUnhandledKeyboardEvent(
-      const content::NativeWebKeyboardEvent& event);
-
-  virtual void OnFrameMetadataUpdated(const cc::CompositorFrameMetadata& old);
-
-  virtual void OnDownloadRequested(
-      const GURL& url,
-      const std::string& mimeType,
-      const bool shouldPrompt,
-      const base::string16& suggestedFilename,
-      const std::string& cookies,
-      const std::string& referrer);
-
-  virtual bool ShouldHandleNavigation(const GURL& url,
-                                      WindowOpenDisposition disposition,
-                                      bool user_gesture);
-
-  virtual WebFrame* CreateWebFrame(
-      content::RenderFrameHost* render_frame_host);
-  virtual WebPopupMenu* CreatePopupMenu(content::RenderFrameHost* rfh);
-
-  virtual WebView* CreateNewWebView(const gfx::Rect& initial_pos,
-                                    WindowOpenDisposition disposition);
-
-  virtual FilePicker* CreateFilePicker(content::RenderViewHost* rvh);
-
-  virtual void OnSwapCompositorFrame() = 0;
-  virtual void OnEvictCurrentFrame();
-
-  virtual void OnTextInputStateChanged();
-  virtual void OnFocusedNodeChanged();
-  virtual void OnSelectionBoundsChanged();
-  virtual void OnImeCancelComposition();
-  virtual void OnSelectionChanged();
-
-  virtual void OnUpdateCursor(const content::WebCursor& cursor);
-
-  virtual void OnSecurityStatusChanged(const SecurityStatus& old);
-  virtual void OnCertificateError(scoped_ptr<CertificateError> error);
-  virtual void OnContentBlocked();
-
-  virtual void OnPrepareToCloseResponse(bool proceed);
-  virtual void OnCloseRequested();
+  WebViewClient* client_;
 
   struct WebContentsDeleter {
     void operator()(content::WebContents* contents);
@@ -581,36 +495,23 @@ class WebView : public ScriptMessageTarget,
   WebViewContentsHelper* web_contents_helper_;
 
   scoped_ptr<Compositor> compositor_;
+  scoped_refptr<cc::SolidColorLayer> root_layer_;
 
   scoped_refptr<CompositorFrameHandle> current_compositor_frame_;
   std::vector<scoped_refptr<CompositorFrameHandle> > previous_compositor_frames_;
-  std::queue<uint32> received_surface_ids_;
+  std::queue<uint32_t> received_surface_ids_;
 
-  scoped_ptr<GestureProvider> gesture_provider_;
-  bool in_swap_;
-
-  GURL initial_url_;
-  scoped_ptr<content::NavigationController::LoadURLParams> initial_data_;
-  content::NavigationController::RestoreType restore_type_;
-  std::vector<sessions::SerializedNavigationEntry> restore_state_;
-  int initial_index_;
+  gfx::Point global_mouse_position_;
+  TouchEventState touch_state_;
 
   content::NotificationRegistrar registrar_;
 
-  struct WebFrameDeleter {
-    void operator()(WebFrame* frame);
-  };
-  typedef scoped_ptr<WebFrame, WebFrameDeleter> WebFrameScopedPtr;
+  bool fullscreen_granted_;
+  bool fullscreen_requested_;
 
-  WebFrameScopedPtr root_frame_;
-
-  bool is_fullscreen_;
   base::WeakPtr<WebPopupMenu> active_popup_menu_;
   base::WeakPtr<FilePicker> active_file_picker_;
 
-  PermissionRequestManager permission_request_manager_;
-  CertificateErrorManager certificate_error_manager_;
-
   ContentType blocked_content_;
 
   cc::CompositorFrameMetadata pending_compositor_frame_metadata_;
@@ -618,25 +519,16 @@ class WebView : public ScriptMessageTarget,
 
   SecurityStatus security_status_;
 
-  // Usually we would scroll the focused editable node in to view after any
-  // resize if the input method is onscreen. However, this interacts badly
-  // with the browser header bar, which resizes the view when its visibility
-  // changes. To work around this, we don't scroll the focused node into
-  // view on a resize if it has already been scrolled once and the input
-  // method hasn't been hidden. This is reset if the input method goes
-  // offscreen or the focused node changes. To do this, we add a delay to
-  // ensure that we only do the scroll once any transitions are finished
-  // See https://bugs.launchpad.net/oxide/+bug/1301681/comments/3
-  //
-  // We should be able to get rid of this once we have a solution for
-  // https://launchpad.net/bugs/1370366
-  bool did_scroll_focused_editable_node_into_view_;
-  base::Timer auto_scroll_timer_;
-
   int location_bar_height_pix_;
   blink::WebTopControlsState location_bar_constraints_;
   bool location_bar_animated_;
 
+  RenderWidgetHostID interstitial_rwh_id_;
+
+  GURL target_url_;
+
+  int edit_flags_;
+
   base::WeakPtrFactory<WebView> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(WebView);
