@@ -42,21 +42,6 @@ WebViewContentsHelper::~WebViewContentsHelper() {
   }
 }
 
-void WebViewContentsHelper::Init() {
-  DCHECK(!FromWebContents(web_contents_));
-
-  web_contents_->SetUserData(kWebViewContentsHelperKey, this);
-
-  content::RendererPreferences* renderer_prefs =
-      web_contents_->GetMutableRendererPrefs();
-  renderer_prefs->browser_handles_non_local_top_level_requests = true;
-
-  content::RenderViewHost* rvh = web_contents_->GetRenderViewHost();
-  if (rvh) {
-    rvh->SyncRendererPrefs();
-  }
-}
-
 void WebViewContentsHelper::UpdateWebPreferences() {
   content::RenderViewHost* rvh = web_contents_->GetRenderViewHost();
   if (!rvh) {
@@ -74,27 +59,46 @@ void WebViewContentsHelper::WebPreferencesValueChanged() {
   UpdateWebPreferences();
 }
 
-WebViewContentsHelper::WebViewContentsHelper(content::WebContents* contents)
-    : BrowserContextObserver(
-          BrowserContext::FromContent(contents->GetBrowserContext())),
-      context_(BrowserContext::FromContent(contents->GetBrowserContext())),
-      web_contents_(contents),
-      owns_web_preferences_(false) {
-  Init();
+void WebViewContentsHelper::NotifyDoNotTrackChanged() {
+  content::RendererPreferences* renderer_prefs =
+      web_contents_->GetMutableRendererPrefs();
+  renderer_prefs->enable_do_not_track = context_->GetDoNotTrack();
+
+  // Send the new override string to the renderer.
+  content::RenderViewHost* rvh = web_contents_->GetRenderViewHost();
+  if (!rvh) {
+    return;
+  }
+  rvh->SyncRendererPrefs();
 }
 
 WebViewContentsHelper::WebViewContentsHelper(content::WebContents* contents,
-                                             WebViewContentsHelper* opener)
+                                             content::WebContents* opener)
     : BrowserContextObserver(
           BrowserContext::FromContent(contents->GetBrowserContext())),
       context_(BrowserContext::FromContent(contents->GetBrowserContext())),
       web_contents_(contents),
       owns_web_preferences_(false) {
-  Init();
+  DCHECK(!FromWebContents(web_contents_));
 
-  WebPreferencesObserver::Observe(opener->GetWebPreferences()->Clone());
-  owns_web_preferences_ = true;
-  UpdateWebPreferences();
+  web_contents_->SetUserData(kWebViewContentsHelperKey, this);
+
+  content::RendererPreferences* renderer_prefs =
+      web_contents_->GetMutableRendererPrefs();
+  renderer_prefs->enable_do_not_track = context_->GetDoNotTrack();
+
+  content::RenderViewHost* rvh = web_contents_->GetRenderViewHost();
+  if (rvh) {
+    rvh->SyncRendererPrefs();
+  }
+
+  if (opener) {
+    WebPreferencesObserver::Observe(
+        WebViewContentsHelper::FromWebContents(opener)
+          ->GetWebPreferences()->Clone());
+    owns_web_preferences_ = true;
+    UpdateWebPreferences();
+  }
 }
 
 // static
@@ -107,7 +111,13 @@ WebViewContentsHelper* WebViewContentsHelper::FromWebContents(
 // static
 WebViewContentsHelper* WebViewContentsHelper::FromRenderViewHost(
     content::RenderViewHost* rvh) {
-  return FromWebContents(content::WebContents::FromRenderViewHost(rvh));
+  content::WebContents* contents =
+      content::WebContents::FromRenderViewHost(rvh);
+  if (!contents) {
+    return nullptr;
+  }
+
+  return FromWebContents(contents);
 }
 
 content::WebContents* WebViewContentsHelper::GetWebContents() const {
