@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -25,20 +25,13 @@
 #include "url/gurl.h"
 
 #include "shared/common/oxide_message_enums.h"
-#include "shared/common/oxide_script_message_request.h"
+#include "shared/common/oxide_param_traits.h"
+#include "shared/common/oxide_script_message_params.h"
+#include "shared/common/oxide_user_agent_override_set.h"
 
-IPC_ENUM_TRAITS(OxideMsg_SendMessage_Type::Value)
-IPC_ENUM_TRAITS(oxide::ScriptMessageRequest::Error)
 IPC_ENUM_TRAITS(blink::WebTopControlsState)
-
-IPC_STRUCT_BEGIN(OxideMsg_SendMessage_Params)
-  IPC_STRUCT_MEMBER(std::string, context)
-  IPC_STRUCT_MEMBER(int, serial)
-  IPC_STRUCT_MEMBER(OxideMsg_SendMessage_Type::Value, type)
-  IPC_STRUCT_MEMBER(oxide::ScriptMessageRequest::Error, error)
-  IPC_STRUCT_MEMBER(std::string, msg_id)
-  IPC_STRUCT_MEMBER(std::string, payload)
-IPC_STRUCT_END()
+IPC_ENUM_TRAITS(oxide::ScriptMessageParams::Error)
+IPC_ENUM_TRAITS(oxide::ScriptMessageParams::Type)
 
 #define IPC_MESSAGE_START OxideMsgStart
 
@@ -48,11 +41,8 @@ IPC_MESSAGE_CONTROL1(OxideMsg_UpdateUserScripts,
 IPC_MESSAGE_CONTROL1(OxideMsg_SetUserAgent,
                      std::string)
 
-IPC_MESSAGE_ROUTED1(OxideHostMsg_SendMessage,
-                    OxideMsg_SendMessage_Params)
-
 IPC_MESSAGE_ROUTED1(OxideMsg_SendMessage,
-                    OxideMsg_SendMessage_Params)
+                    oxide::ScriptMessageParams)
 
 IPC_MESSAGE_ROUTED1(OxideMsg_SetAllowDisplayingInsecureContent,
                     bool)
@@ -65,13 +55,21 @@ IPC_MESSAGE_ROUTED3(OxideMsg_UpdateTopControlsState,
                     blink::WebTopControlsState,
                     bool)
 
+IPC_MESSAGE_CONTROL1(OxideMsg_UpdateUserAgentOverrides,
+                     std::vector<oxide::UserAgentOverrideSet::Entry>)
+
+IPC_MESSAGE_CONTROL1(OxideMsg_SetLegacyUserAgentOverrideEnabled,
+                     bool)
+
+IPC_MESSAGE_ROUTED1(OxideHostMsg_SendMessage,
+                    oxide::ScriptMessageParams)
+
 IPC_MESSAGE_ROUTED0(OxideHostMsg_DidBlockDisplayingInsecureContent)
 IPC_MESSAGE_ROUTED0(OxideHostMsg_DidBlockRunningInsecureContent)
 
-IPC_SYNC_MESSAGE_CONTROL1_2(OxideHostMsg_GetUserAgentOverride,
+IPC_SYNC_MESSAGE_CONTROL1_1(OxideHostMsg_GetUserAgentOverride,
                             GURL,
-                            std::string,
-                            bool)
+                            std::string)
 
 // Media
 IPC_ENUM_TRAITS(OxideHostMsg_MediaPlayer_Initialize_Type)
@@ -187,6 +185,11 @@ IPC_MESSAGE_ROUTED2(OxideHostMsg_MediaPlayer_SetVolume,
                     int /* player_id */,
                     double /* volume */)
 
+// Set the rate.
+IPC_MESSAGE_ROUTED2(OxideHostMsg_MediaPlayer_SetRate,
+                    int /* player_id */,
+                    double /* rate */)
+
 // Set the poster image.
 IPC_MESSAGE_ROUTED2(OxideHostMsg_MediaPlayer_SetPoster,
                     int /* player_id */,
