@@ -49,7 +49,7 @@
 #include "ui/gfx/image/image.h"
 // #include "webkit/renderer/compositor_bindings/web_layer_impl.h"
 
-static const uint32 kGLTextureExternalOES = 0x8D65;
+static const uint32_t kGLTextureExternalOES = 0x8D65;
 
 using blink::WebMediaPlayer;
 using blink::WebSize;
@@ -201,7 +201,7 @@ void WebMediaPlayer::pause() {
 }
 
 void WebMediaPlayer::seek(double seconds) {
-  NOTIMPLEMENTED();
+  player_manager_->Seek(player_id_, base::TimeDelta::FromSeconds(seconds));
 }
 
 bool WebMediaPlayer::supportsSave() const {
@@ -209,6 +209,13 @@ bool WebMediaPlayer::supportsSave() const {
 }
 
 void WebMediaPlayer::setRate(double rate) {
+  player_manager_->SetRate(player_id_, rate);
+}
+
+void WebMediaPlayer::setSinkId(
+      const blink::WebString& sinkId,
+      const blink::WebSecurityOrigin& origin,
+      blink::WebSetSinkIdCallbacks* callbacks) {
   NOTIMPLEMENTED();
 }
 
@@ -319,7 +326,8 @@ WebTimeRanges WebMediaPlayer::buffered() const {
 }
 
 WebTimeRanges WebMediaPlayer::seekable() const {
-  return buffered_;
+  const blink::WebTimeRange seekable_range(0.0, duration());
+  return blink::WebTimeRanges(&seekable_range, 1);
 }
 
 double WebMediaPlayer::maxTimeSeekable() const {
@@ -378,7 +386,7 @@ bool WebMediaPlayer::didPassCORSAccessCheck() const {
 }
 
 double WebMediaPlayer::mediaTimeForTimeValue(double timeValue) const {
-  return media::ConvertSecondsToTimestamp(timeValue).InSecondsF();
+  return base::TimeDelta::FromSecondsD(timeValue).InSecondsF();
 }
 
 unsigned WebMediaPlayer::decodedFrameCount() const {
@@ -653,7 +661,7 @@ void WebMediaPlayer::UpdatePlayingState(bool is_playing) {
 // Convert a WebString to ASCII, falling back on an empty string in the case
 // of a non-ASCII string.
 static std::string ToASCIIOrEmpty(const blink::WebString& string) {
-  return base::IsStringASCII(string) ? base::UTF16ToASCII(string)
+  return base::IsStringASCII(string) ? string.utf8()
                                      : std::string();
 }
 
@@ -673,15 +681,6 @@ static void EmeUMAHistogramEnumeration(const std::string& key_system,
       base::Histogram::kUmaTargetedHistogramFlag)->Add(sample);
 }
 
-static void EmeUMAHistogramCounts(const std::string& key_system,
-                                  const std::string& method,
-                                  int sample) {
-  // Use the same parameters as UMA_HISTOGRAM_COUNTS.
-  base::Histogram::FactoryGet(
-      kMediaEme + media::GetKeySystemNameForUMA(key_system) + "." + method,
-      1, 1000000, 50, base::Histogram::kUmaTargetedHistogramFlag)->Add(sample);
-}
-
 // Helper enum for reporting generateKeyRequest/addKey histograms.
 enum MediaKeyException {
   kUnknownResultId,
@@ -845,53 +844,6 @@ void WebMediaPlayer::setContentDecryptionModule(
   NOTIMPLEMENTED();
 }
 
-void WebMediaPlayer::OnKeyAdded(const std::string& session_id) {
-  EmeUMAHistogramCounts(current_key_system_, "KeyAdded", 1);
-
-  client_->keyAdded(
-      WebString::fromUTF8(media::GetPrefixedKeySystemName(current_key_system_)),
-      WebString::fromUTF8(session_id));
-}
-
-void WebMediaPlayer::OnKeyError(const std::string& session_id,
-                                       media::MediaKeys::KeyError error_code,
-                                       uint32 system_code) {
-  EmeUMAHistogramEnumeration(current_key_system_, "KeyError",
-                             error_code, media::MediaKeys::kMaxKeyError);
-
-  unsigned short short_system_code = 0;
-  if (system_code > std::numeric_limits<unsigned short>::max()) {
-    LOG(WARNING) << "system_code exceeds unsigned short limit.";
-    short_system_code = std::numeric_limits<unsigned short>::max();
-  } else {
-    short_system_code = static_cast<unsigned short>(system_code);
-  }
-
-  client_->keyError(
-      WebString::fromUTF8(media::GetPrefixedKeySystemName(current_key_system_)),
-      WebString::fromUTF8(session_id),
-      static_cast<blink::WebMediaPlayerClient::MediaKeyErrorCode>(error_code),
-      short_system_code);
-}
-
-void WebMediaPlayer::OnKeyMessage(const std::string& session_id,
-                                         const std::vector<uint8>& message,
-                                         const GURL& destination_url) {
-  DCHECK(destination_url.is_empty() || destination_url.is_valid());
-
-  client_->keyMessage(
-      WebString::fromUTF8(media::GetPrefixedKeySystemName(current_key_system_)),
-      WebString::fromUTF8(session_id),
-      message.empty() ? nullptr : &message[0],
-      message.size(),
-      destination_url);
-}
-
-void WebMediaPlayer::OnMediaSourceOpened(
-    blink::WebMediaSource* web_media_source) {
-  client_->mediaSourceOpened(web_media_source);
-}
-
 void WebMediaPlayer::enterFullscreen() {
   NOTIMPLEMENTED();
 }
