@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -20,26 +20,36 @@
 #include <string>
 
 #include "base/command_line.h"
+#include "base/strings/stringprintf.h"
 #include "cc/trees/layer_tree_settings.h"
-#include "content/public/common/url_utils.h"
+#include "content/public/common/url_constants.h"
+#include "content/public/common/user_agent.h"
 #include "content/public/renderer/render_frame.h"
 #include "content/public/renderer/render_thread.h"
 #include "content/public/renderer/render_view.h"
+#include "net/base/net_module.h"
+#include "third_party/WebKit/public/platform/WebURLResponse.h"
 #include "third_party/WebKit/public/web/WebRuntimeFeatures.h"
 #include "third_party/WebKit/public/web/WebSettings.h"
 #include "third_party/WebKit/public/web/WebView.h"
 #include "ui/native_theme/native_theme_switches.h"
 
+#include "shared/common/chrome_version.h"
 #include "shared/common/oxide_constants.h"
-#include "shared/common/oxide_messages.h"
+#include "shared/common/oxide_form_factor.h"
+#include "shared/common/oxide_net_resource_provider.h"
 
-#include "oxide_render_process_observer.h"
+#include "oxide_renderer_user_agent_settings.h"
 #include "oxide_script_message_dispatcher_renderer.h"
 #include "oxide_top_controls_handler.h"
 #include "oxide_user_script_scheduler.h"
 #include "oxide_user_script_slave.h"
 #include "oxide_web_content_settings_client.h"
 
+#if defined(ENABLE_PLUGINS)
+#include "pepper/oxide_pepper_render_frame_observer.h"
+#endif
+
 #if defined(ENABLE_MEDIAHUB)
 #include "media/oxide_renderer_media_player_manager.h"
 #include "media/oxide_web_media_player.h"
@@ -47,23 +57,33 @@
 
 namespace oxide {
 
-ContentRendererClient::ContentRendererClient() {}
-
-ContentRendererClient::~ContentRendererClient() {}
-
 void ContentRendererClient::RenderThreadStarted() {
-  new RenderProcessObserver();
+  content::RenderThread* thread = content::RenderThread::Get();
+
+  user_agent_settings_.reset(new RendererUserAgentSettings());
+  thread->AddObserver(user_agent_settings_.get());
+  
   new UserScriptSlave();
 
+  net::NetModule::SetResourceProvider(NetResourceProvider);
+
   // Usually enabled only on Android. We want this on mobile, but
   // should be ok everywhere
   blink::WebRuntimeFeatures::enableOrientationEvent(true);
+  // Oxide has no mechanism to display page popups
+  blink::WebRuntimeFeatures::enablePagePopup(false);
+  // Oxide does not support NavigatorContentUtils.
+  // See https://launchpad.net/bugs/1214046
+  blink::WebRuntimeFeatures::enableNavigatorContentUtils(false);
 }
 
 void ContentRendererClient::RenderFrameCreated(
     content::RenderFrame* render_frame) {
   new ScriptMessageDispatcherRenderer(render_frame);
   new WebContentSettingsClient(render_frame);
+#if defined(ENABLE_PLUGINS)
+  new PepperRenderFrameObserver(render_frame);
+#endif
 #if defined(ENABLE_MEDIAHUB)
   new RendererMediaPlayerManager(render_frame);
 #endif
@@ -80,61 +100,34 @@ void ContentRendererClient::RenderViewCreated(
   blink::WebSettings* settings = render_view->GetWebView()->settings();
   settings->setDoubleTapToZoomEnabled(true); // XXX: Make this configurable
 
-  std::string form_factor =
-      base::CommandLine::ForCurrentProcess()->GetSwitchValueASCII(
-        switches::kFormFactor);
-  if (form_factor == switches::kFormFactorTablet ||
-      form_factor == switches::kFormFactorPhone) {
+  if (GetFormFactorHint() == FORM_FACTOR_TABLET ||
+      GetFormFactorHint() == FORM_FACTOR_PHONE) {
     settings->setAllowCustomScrollbarInMainFrame(false);
     settings->setUseWideViewport(true);
     settings->setMainFrameClipsContent(false);
     settings->setShrinksViewportContentToFit(true);
     settings->setUseMobileViewportStyle(true);
+    settings->setTouchEditingEnabled(true);
   }
 }
 
 std::string ContentRendererClient::GetUserAgentOverrideForURL(
     const GURL& url) {
-  GURL u = url;
-
-  // Strip username / password / fragment identifier if they exist
-  if (u.has_password() || u.has_username() || u.has_ref()) {
-    GURL::Replacements rep;
-    rep.ClearUsername();
-    rep.ClearPassword();
-    rep.ClearRef();
-    u = u.ReplaceComponents(rep);
+  if (url.scheme() == content::kChromeUIScheme) {
+    return content::BuildUserAgentFromProduct(
+        base::StringPrintf("Chrome/%s", CHROME_VERSION_STRING));
   }
 
-  // Strip query if we are above the max number of chars
-  if (u.spec().size() > content::GetMaxURLChars() &&
-      u.has_query()) {
-    GURL::Replacements rep;
-    rep.ClearQuery();
-    u = u.ReplaceComponents(rep);
-  }
+  return user_agent_settings_->GetUserAgentOverrideForURL(url);
+}
 
-  // If we are still over, just send the origin
-  if (u.spec().size() > content::GetMaxURLChars()) {
-    u = u.GetOrigin();
-  }
-
-  // Not sure we should ever hit this, but in any case - there
-  // isn't much more we can do now
-  if (u.spec().size() > content::GetMaxURLChars()) {
-    return std::string();
-  }
-
-  bool overridden = false;
-  std::string user_agent;
-
-  content::RenderThread::Get()->Send(new OxideHostMsg_GetUserAgentOverride(
-      u, &user_agent, &overridden));
-  if (!overridden) {
-    return std::string();
-  }
-
-  return user_agent;
+void ContentRendererClient::AddImageContextMenuProperties(
+    const blink::WebURLResponse& response,
+    std::map<std::string, std::string>* properties) {
+  // XXX(oSoMoN): see comment in
+  // oxide::ResourceDispatcherHostDelegate::DispatchDownloadRequest(…).
+  (*properties)[oxide::kImageContextMenuPropertiesMimeType] =
+      response.mimeType().utf8();
 }
 
 #if defined(ENABLE_MEDIAHUB)
@@ -174,14 +167,11 @@ void ContentRendererClient::OverrideCompositorSettings(
     settings->scrollbar_fade_duration_ms = 300;
   }
 
-  std::string form_factor =
-      base::CommandLine::ForCurrentProcess()->GetSwitchValueASCII(
-        switches::kFormFactor);
-  if (form_factor == switches::kFormFactorDesktop) {
-    settings->scrollbar_show_scale_threshold = 1.05f;
-  } else {
-    settings->scrollbar_show_scale_threshold = 1.f;
-  }
+  settings->use_external_begin_frame_source = false;
 }
 
+ContentRendererClient::ContentRendererClient() {}
+
+ContentRendererClient::~ContentRendererClient() {}
+
 } // namespace oxide
