@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -21,32 +21,30 @@
 #include "base/macros.h"
 #include "content/public/renderer/content_renderer_client.h"
 
-namespace base {
-template <typename Type> struct DefaultLazyInstanceTraits;
-}
-
 namespace oxide {
 
 #if defined(ENABLE_MEDIAHUB)
 class RendererMediaPlayerManager;
 #endif
 
+class RendererUserAgentSettings;
 
 class ContentRendererClient final : public content::ContentRendererClient {
  public:
-  // XXX(chrisccoulson): Try not to add anything here
-
- private:
-  friend struct base::DefaultLazyInstanceTraits<ContentRendererClient>;
-
   ContentRendererClient();
   ~ContentRendererClient();
 
+  // XXX(chrisccoulson): Try not to add anything here
+
+ private:
   // content::ContentRendererClient implementation
   void RenderThreadStarted() final;
   void RenderFrameCreated(content::RenderFrame* render_frame) final;
   void RenderViewCreated(content::RenderView* render_view) final;
   std::string GetUserAgentOverrideForURL(const GURL& url) final;
+  void AddImageContextMenuProperties(
+      const blink::WebURLResponse& response,
+      std::map<std::string, std::string>* properties) final;
 #if defined(ENABLE_MEDIAHUB)
   blink::WebMediaPlayer* OverrideWebMediaPlayer(
       blink::WebFrame* frame,
@@ -56,6 +54,8 @@ class ContentRendererClient final : public content::ContentRendererClient {
 #endif
   void OverrideCompositorSettings(cc::LayerTreeSettings* settings) final;
 
+  scoped_ptr<RendererUserAgentSettings> user_agent_settings_;
+
   DISALLOW_COPY_AND_ASSIGN(ContentRendererClient);
 };
 
