@@ -18,8 +18,7 @@
 #ifndef _OXIDE_SHARED_RENDERER_ISOLATED_WORLD_MAP_H_
 #define _OXIDE_SHARED_RENDERER_ISOLATED_WORLD_MAP_H_
 
-#include "base/basictypes.h"
-#include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "url/gurl.h"
 
 namespace oxide {
