@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -19,17 +19,21 @@
 
 #include "oxide_script_message_manager.h"
 
+#include <utility>
+
 #include "base/logging.h"
 #include "base/memory/ref_counted.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_piece.h"
 #include "base/strings/utf_string_conversions.h"
+#include "content/public/child/v8_value_converter.h"
 #include "content/public/renderer/render_frame.h"
 #include "third_party/WebKit/public/web/WebLocalFrame.h"
 #include "third_party/WebKit/public/web/WebScopedMicrotaskSuppression.h"
 #include "ui/base/resource/resource_bundle.h"
 
 #include "shared/common/oxide_constants.h"
+#include "shared/common/oxide_messages.h"
 
 #include "oxide_isolated_world_map.h"
 #include "oxide_script_message_handler_renderer.h"
@@ -117,6 +121,8 @@ v8::Handle<v8::Object> ScriptMessageManager::GetOxideApiObject(
 
   v8::Local<v8::Script> script(v8::Script::Compile(wrapped_src));
 
+  blink::WebScopedMicrotaskSuppression mts;
+
   v8::TryCatch try_catch;
   v8::Local<v8::Function> function(script->Run().As<v8::Function>());
   if (try_catch.HasCaught()) {
@@ -135,21 +141,19 @@ v8::Handle<v8::Object> ScriptMessageManager::GetOxideApiObject(
 
   v8::Local<v8::Object> exports(v8::Object::New(isolate));
 
-  v8::Handle<v8::Value> argv[] = {
+  v8::Local<v8::Value> argv[] = {
     send_message_template->GetFunction(),
     add_message_handler_template->GetFunction(),
     remove_message_handler_template->GetFunction(),
     exports
   };
 
-  {
-    blink::WebScopedMicrotaskSuppression mts;
-    frame_->GetWebFrame()->callFunctionEvenIfScriptDisabled(
-        function,
-        GetV8Context()->Global(),
-        arraysize(argv),
-        argv);
-  }
+  frame_->GetWebFrame()->callFunctionEvenIfScriptDisabled(
+      function,
+      GetV8Context()->Global(),
+      arraysize(argv),
+      argv);
+
   if (try_catch.HasCaught()) {
     LOG(ERROR) << "Caught exception when running script function";
     return v8::Handle<v8::Object>();
@@ -193,7 +197,7 @@ void ScriptMessageManager::SendMessageInner(
 
   v8::Local<v8::Value> msg_want_reply_as_val = args[0];
   v8::Local<v8::Value> msg_id_as_val = args[1];
-  v8::Local<v8::Value> msg_args_as_val = args[2];
+  v8::Local<v8::Value> msg_payload = args[2];
 
   DCHECK(msg_want_reply_as_val->IsBoolean());
   if (!msg_id_as_val->IsString()) {
@@ -202,21 +206,38 @@ void ScriptMessageManager::SendMessageInner(
           isolate, "Invalid message ID")));
     return;
   }
-  DCHECK(msg_args_as_val->IsString());
 
   v8::Local<v8::Boolean> msg_want_reply = msg_want_reply_as_val->ToBoolean();
   v8::Local<v8::String> msg_id = msg_id_as_val.As<v8::String>();
-  v8::Local<v8::String> msg_args = msg_args_as_val.As<v8::String>();
 
-  v8::Handle<v8::Object> handle(
-      script_message_request_object_handler_.NewInstance());
+  scoped_ptr<content::V8ValueConverter> converter(
+      content::V8ValueConverter::create());
+  scoped_ptr<base::Value> payload(
+      converter->FromV8Value(msg_payload, isolate->GetCallingContext()));
 
-  scoped_refptr<ScriptMessageRequestImplRenderer> req =
-      new ScriptMessageRequestImplRenderer(
-        this, next_message_id_++, msg_want_reply->Value(),
-        V8StringToStdString(msg_id),
-        V8StringToStdString(msg_args), handle);
-  req->SendMessage();
+  v8::Handle<v8::Object> handle;
+  int serial = ScriptMessageParams::kInvalidSerial;
+
+  if (msg_want_reply->Value()) {
+    handle = script_message_request_object_handler_.NewInstance();
+
+    scoped_refptr<ScriptMessageRequestImplRenderer> req =
+        new ScriptMessageRequestImplRenderer(
+          this, next_message_id_++, handle);
+    serial = req->serial();
+  }
+
+  ScriptMessageParams params;
+  PopulateScriptMessageParams(serial,
+                              GetContextURL(),
+                              V8StringToStdString(msg_id),
+                              std::move(payload),
+                              &params);
+
+  if (!frame()->Send(new OxideHostMsg_SendMessage(frame()->GetRoutingID(),
+                                                  params))) {
+    return;
+  }
 
   if (msg_want_reply->Value()) {
     args.GetReturnValue().Set(handle);
@@ -360,9 +381,13 @@ ScriptMessageManager::ScriptMessageManager(content::RenderFrame* frame,
 
   if (world_id_ != kMainWorldId) {
     v8::Local<v8::Object> global(context->Global());
-    global->SetAccessor(v8::String::NewFromUtf8(isolate(), "oxide"),
-                        OxideLazyGetter, nullptr,
-                        local_data);
+    global->Set(v8::String::NewFromUtf8(isolate(), "oxide"),
+                GetOxideApiObject(isolate()));
+    // XXX(chrisccoulson): Started failing between Chromium 45.0.2431.0 and
+    //  45.0.2433.0
+    //global->SetAccessor(v8::String::NewFromUtf8(isolate(), "oxide"),
+    //                    OxideLazyGetter, nullptr,
+    //                    local_data);
   }
 }
 
